/** AbulEduAbulEduEditeur
  *
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012-2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2012-2013 Icham Sirat <icham.sirat@ryxeo.com>
  * @author 2012-2013 Éric Seigne <eric.seigne@ryxeo.com>
  * @author 2012-2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */
#include "abuleduediteur.h"
#include "ui_abuleduediteur.h"


/** @todo Sécuriser le code pour que les champs obligatoires soient effectivement bien remplis */

//////////////////////////////////////////////////////////
//                Projet
//////////////////////////////////////////////////////////

AbulEduEditeur::AbulEduEditeur(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduEditeur),
    m_abuleduFile(QSharedPointer<AbulEduFileV1>()),
    m_userLeterrierAppDirectory(""),
    m_titreGeneral(""),
    m_exerciceEnCours(-1),
    m_nombreQuestions(1)
{
    m_localDebug = true;
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<"   Constructeur";
    ui->setupUi(this);
    setAttribute( Qt::WA_DeleteOnClose );
    
    
    m_lastPathImages            = QString(abeApp->getAbeHomepath());
    // Création du fil d'Ariane
    QList<QPair<QString, QString> > listeEtapes;
    listeEtapes.append(qMakePair(trUtf8("Accueil"), trUtf8("Accueil","For short")));
    listeEtapes.append(qMakePair(trUtf8("Questions"), trUtf8("Questions","For short")));
    listeEtapes.append(qMakePair(trUtf8("Sauvegarder"), trUtf8("Sauvegarder","For short")));
    etapes = new AbulEduWidgetAssistantEtapesV1(listeEtapes);
    etapes->abeWidgetAssistantSetFontSize(12, 16, 20);
    etapes->abeWidgetAssistantEnableClick(false);
    etapes->setStyleSheet(QString("background-color: qlineargradient( x1:0 y1:0, x2:1 y2:0, stop:0 #69cc5e, stop:1 #72a2e6)")); // Ne marche pas
    ui->hBoxFilAriane->addWidget(etapes);
    connect(etapes, SIGNAL(signalEtapeHasChanged(int)), this, SLOT(slotChangementPageRequested(int)));
    connect(etapes, SIGNAL(signalQuitterRequested()), this, SLOT(slotCloseAbulEduEditeur()));
    
    // Création du sélecteur d'images locales
    m_fileDialogImages = new QFileDialog(this,
                                         trUtf8("Sélectionner une image"),
                                         m_lastPathImages,trUtf8("Fichiers Images (*.png *.jpg *.JPG *.jpeg *.svg)"));
    ui->vlTabDisque->addWidget(m_fileDialogImages);
    m_fileDialogImages->setOption(QFileDialog::DontUseNativeDialog, true);
    connect(m_fileDialogImages, SIGNAL(accepted()), this, SLOT(slotAcceptClicked()));
    connect(m_fileDialogImages, SIGNAL(fileSelected(QString)), this, SLOT(slotFileSelected(QString)));
    connect(m_fileDialogImages, SIGNAL(rejected()), this, SLOT(slotRejectClicked()));
    
    // Paramétrage du sélecteur d'images sur data
    ui->widgetMediathequeGet->setVisible(true);
    ui->widgetMediathequeGet->abeSetSource("data");
    ui->widgetMediathequeGet->abeSetCustomBouton1(trUtf8("Ajouter"));
    ui->widgetMediathequeGet->abeHideBoutonTelecharger();
    ui->widgetMediathequeGet->abeSetCustomBouton1Download(true);
    ui->widgetMediathequeGet->abeSetDefaultView(AbulEduMediathequeGetV1::abeMediathequeThumbnails);
    connect(ui->widgetMediathequeGet, SIGNAL(signalMediathequeFileDownloaded(int)), this, SLOT(slotAjouterImageMediatheque()));
    
    // Paramétrage du dialogue d'envoi sur médiatheque
    ui->widgetMediathequePush->setVisible(false);
    ui->widgetMediathequePush->abeSetSource("mediatheque");
    
    //    // Paramétrage du sélecteur de fichiers abe
    //    ui->choisisProjet->setStyleSheet(QString("background-color: qlineargradient( x1:0 y1:0, x2:1 y2:0, stop:0 #69cc5e, stop:1 #72a2e6)")); // sinon le abuledufilemanager est transparent !!!
    //    connect(ui->widgetOpenProject, SIGNAL(signalAbeFileSelected()), this, SLOT(slotGetFileForModifications()));
    
    // On se positionne au début
    ui->swAssistant->setCurrentWidget(ui->pageAccueil);
    
    // Creation du répertoire leterrier/imagessequentielles dans le home de l'utilisateur
    // on crée, s'il n'existe pas, le début du chemin chez l'utilisateur
    if (!QDir(abeApp->getAbeHomepath()+"/leterrier/"+abeApp->applicationName()).exists())
    {
        QDir().mkpath(abeApp->getAbeHomepath()+"/leterrier/"+abeApp->applicationName());
        if(m_localDebug) qDebug()<<"Dossier de l'application cree chez l'utilisateur";
    }
    else
    {
        if(m_localDebug) qDebug()<<"Dossier de l'application existe chez l'utilisateur";
    }
    
    m_abuleduTemporaryData = new AbulEduFileV1(this); //Pour stocker les images redimensionnées
    m_repertoireTemporaryData = QDir(m_abuleduTemporaryData->abeFileGetDirectoryTemp()).path();
    m_userLeterrierAppDirectory = abeApp->getAbeHomepath()+"/leterrier/"+abeApp->applicationName();
    
    connect(ui->swAssistant,
            SIGNAL(currentChanged(int)),
            this,
            SLOT(onPageAssistantChanged(int)));
    
    connect(ui->widgetMediathequePush,
            SIGNAL(signalMediathequePushFileUploaded(int)),
            this,
            SLOT(slotFileUploaded(int)));
    m_questions.append(ui->abeQuestion1);
}

AbulEduEditeur::~AbulEduEditeur()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    delete ui;
    delete m_abuleduTemporaryData;
}

void AbulEduEditeur::abeEditeurSetMainWindow(QWidget *mw)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    LeTerrierMainWindow* parent = (LeTerrierMainWindow*) mw;
    
    if (!parent->abeGetMyAbulEduFile()->abeFileGetFileName().baseName().isEmpty())
    {
        //Si on a déjà un fichier abe ouvert c'est lui qu'on édite
        m_abuleduFile = parent->abeGetMyAbulEduFile();
    }
    else
    {
        // Sinon, on en crée un
        m_abuleduFile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);
        m_abuleduFile->setObjectName("temporaryName");
        parent->abeSetMyAbulEduFile(m_abuleduFile); // On en fait profiter la mainwindow
    }
    
    // On remplit la combobox de sélection de langue
    ui->cbLangueRessource->addItems(m_abuleduFile->abeFileGetLOM()->abeLOMgetAvailableLanguages().values());
    QString lang = "";
    if(m_abuleduFile->abeFileGetLOM()->abeLOMgetGeneralLanguage().count() > 0)
    {
        lang = m_abuleduFile->abeFileGetLOM()->abeLOMgetGeneralLanguage().first();
    }
    else
    {
        if (QLocale::system().name().section('_', 0, 0) == "fr")
        {
            lang = "fre";
        }
        else if (QLocale::system().name().section('_', 0, 0) == "en")
        {
            lang = "eng";
        }
        else if (QLocale::system().name().section('_', 0, 0) == "es")
        {
            lang = "spa";
        }
    }
    lang = m_abuleduFile->abeFileGetLOM()->abeLOMgetAvailableLanguages().value(lang);
    ui->cbLangueRessource       ->setCurrentIndex(ui->cbLangueRessource->findText(lang));
    
    // Active ou désactive le pbModifieExerciceEnCours
    if(m_abuleduFile->abeFileGetFileName().fileName()=="" || m_abuleduFile->abeFileGetFileName().fileName()=="temporaryName" )
    {
        ui->pbModifExerciceEnCours->setEnabled(false); // On n'a pas d'exercices en cours
    }
    else
    {
        ui->pbModifExerciceEnCours->setEnabled(true);
    }
    
    // Création d'un répertoire temporaire pour construire le .abe
    m_repertoireTemporaire = QDir(m_abuleduFile->abeFileGetDirectoryTemp()).path();
    
    QDir().mkpath(m_repertoireTemporaire+"/data/");
    // Création d'un fichier de settings dans le répertoire temporaire
    m_settings = new QSettings(m_repertoireTemporaire+"/conf/module.conf", QSettings::IniFormat );
    // Titre de la fenêtre
    setWindowTitle(abeApp->getAbeApplicationLongName()+"--"+trUtf8("Éditeur d'exercices")
                   + m_abuleduFile->abeFileGetFileName().fileName()
                   + "[*]");
}

//////////////////////////////////////////////////////////
//                Projet
//////////////////////////////////////////////////////////

void AbulEduEditeur::on_pbNouveauProjet_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    etapes->abeWidgetAssistantSetEtapeCourante(1);
    etapes->abeWidgetAssistantEnableSuivant(true);
    initialiseNouveauProjet();
}

void AbulEduEditeur::initialiseNouveauProjet()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_titreGeneral = "";
    ui->leTitreGeneral->setText("");
#if QT_VERSION >= 0x040700
    ui->leTitreGeneral->setPlaceholderText(trUtf8("Saisir le titre de la série d'exercices"));
#endif
    
    
    ui->leAuteur->setText(abeApp->getAbeIdentite()->abeGetPreNom()+" "+ abeApp->getAbeIdentite()->abeGetNom());
    setWindowModified(false);
    
}

void AbulEduEditeur::initialiseProjetAModifier()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(m_settings)
    {
        delete m_settings;
    }
    // Nettoyer tout
    // On recrée le QSetting avec le nouveau fichier de conf
    m_settings = new QSettings(m_repertoireTemporaire+"/conf/module.conf", QSettings::IniFormat );
    
    // On s'occupe des paramètres généraux
    m_titreGeneral = m_settings->value("titre").toString();
    ui->cbLangueRessource->setCurrentIndex(ui->cbLangueRessource->findText(m_settings->value("langage").toString()));
    ui->teDescription->setText(m_abuleduFile->abeFileGetLOM()->abeLOMgetGeneralDescription(ui->cbLangueRessource->currentText()).first());
    ui->leTitreGeneral->setText(m_titreGeneral);
    ui->leAuteur->setText(m_settings->value("auteur","").toString());
    if(m_localDebug) qDebug()<<"== m_titreGeneral"<<m_titreGeneral;

    // puis des paramètres des questions
    m_settings->beginGroup("ExerciceQCM");
    ui->chbAfficherConsigneSequence->setChecked(m_settings->value("afficheConsigneSequence").toBool());
    ui->teConsigneSequence->setText(m_settings->value("consigneSequence").toString());

    /* Il faut ici écrire les questions et les réponses, en créant à la volée les objets (blocs de questions et onglets réponses) nécessaires
        m_settings->beginWriteArray("exercice");
        for (int i = 0;i<m_questions.size();i++)
        {
            m_settings->setArrayIndex(i);
            m_settings->setValue("texteQuestion",m_questions[i]->abeQuestionGetQuestion());
            m_settings->beginWriteArray("reponse");
            for(int j = 0;j < m_questions[i]->abeQuestionGetAnswersStrings().size();j++)
            {
                m_settings->setArrayIndex(j);
                {
                    m_settings->setValue("texteReponse",m_questions[i]->abeQuestionGetAnswersStrings()[j].first);
                    m_settings->setValue("imageReponse",m_questions[i]->abeQuestionGetAnswersStrings()[j].second);
                }
            }
            m_settings->endArray();
        }
        m_settings->endArray();*/
    m_settings->endGroup();
}


void AbulEduEditeur::on_pbModifierProjet_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    
    emit openModuleLoaderForEditeur();
    ui->swAssistant->setCurrentWidget(ui->pageQuestions);
    etapes->abeWidgetAssistantSetEtapeCourante(1);
    etapes->abeWidgetAssistantEnablePrecedent(true);
    etapes->abeWidgetAssistantEnableSuivant(true);
    etapes->abeWidgetAssistantEnableClick(true);
}
//////////////////////////////////////////////////////////
//                Généralités
//////////////////////////////////////////////////////////

void AbulEduEditeur::on_leTitreGeneral_editingFinished()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_titreGeneral = ui->leTitreGeneral->text();
    setWindowTitle(abeApp->getAbeApplicationLongName()
                   +"--"
                   +m_titreGeneral
                   +"[*]"); // Titre de la fenêtre
    setWindowModified(true);
}



//////////////////////////////////////////////////////////
//                Page exercices
//////////////////////////////////////////////////////////



void AbulEduEditeur::initialiseNouvelExercice()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    
}

void AbulEduEditeur::ajouteImage()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    
    QString fileName = QFileDialog::getOpenFileName(this, trUtf8("Choisir une image"),
                                                    m_lastPathImages,
                                                    trUtf8("Fichiers Images (*.png *.jpg *.JPG *.jpeg *.svg)"));
    if(!fileName.isNull())
    {
        ajouteImage(fileName);
    }
}

void AbulEduEditeur::ajouteImage(QString fileName)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    // On sauvegarde le dernier repertoire sélectionné par l'utilisateur
    QFileInfo fileinfo(fileName);
    m_lastPathImages = fileinfo.absoluteFilePath();
    fileName = writeTemporaryImageFile(fileName); // On enregistre l'image dans le fichier temporaire
    // Ecrire ce que l'on fait avec l'image
    
}


//////////////////////////////////////////////////////////
//                Utilitaires
//////////////////////////////////////////////////////////


QString AbulEduEditeur::writeTemporaryImageFile(QString chemin)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    QPixmap image = QPixmap(chemin).scaled(500,500, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    QString imageName = chemin.split("/").last();
    imageName = m_repertoireTemporaryData+"/"+imageName.split(".").first() + ".png";
    image.save(imageName);
    return imageName;
}

void AbulEduEditeur::writeTemporaryConfigFile()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    qDebug()<<m_settings->fileName();
    
    // Créer le fichier de conf
    
    m_settings->setValue("titre", m_titreGeneral);
    m_settings->setValue("auteur",ui->leAuteur->text());
    m_settings->setValue("langage",ui->cbLangueRessource->currentText());
//    m_settings->setValue("niveauDifficulte",m_niveauDifficulte-1);
    
    m_settings->beginGroup("ExerciceQCM");
    m_settings->setValue("afficheConsigneSequence",ui->chbAfficherConsigneSequence->isChecked());
    m_settings->setValue("consigneSequence",ui->teConsigneSequence->toPlainText());
        m_settings->beginWriteArray("exercice");
        for (int i = 0;i<m_questions.size();i++)
        {
            m_settings->setArrayIndex(i);
            m_settings->setValue("texteQuestion",m_questions[i]->abeQuestionGetQuestion());
            m_settings->beginWriteArray("reponse");
            for(int j = 0;j < m_questions[i]->abeQuestionGetAnswersStrings().size();j++)
            {
                m_settings->setArrayIndex(j);
                {
                    m_settings->setValue("texteReponse",m_questions[i]->abeQuestionGetAnswersStrings()[j].first);
                    m_settings->setValue("imageReponse",m_questions[i]->abeQuestionGetAnswersStrings()[j].second);
                }
            }
            m_settings->endArray();
        }
        m_settings->endArray();
    m_settings->endGroup();
}

void AbulEduEditeur::prepareExport()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    writeTemporaryConfigFile();
    writeTemporaryDataFiles();
    // On fait la liste des fichiers a envoyer
    QStringList listeFichiers;
    QStringList fichiers = QDir(m_repertoireTemporaire+"/data/").entryList((QDir::Files | QDir::Hidden | QDir::NoSymLinks));
    if(m_localDebug)qDebug()<<__LINE__<<fichiers;
    foreach(QString fic, fichiers)
    {
        listeFichiers.append(m_repertoireTemporaire+"/data/"+fic);
    }
    fichiers = QDir(m_repertoireTemporaire+"/conf/").entryList((QDir::Files | QDir::Hidden | QDir::NoSymLinks));
    foreach(QString fic, fichiers)
    {
        listeFichiers.append(m_repertoireTemporaire+"/conf/"+fic);
    }
    if(m_localDebug)qDebug()<<"Liste des fichiers"<<listeFichiers;
    
    //Les informations pour LOM
    QString codeLangue = m_abuleduFile->abeFileGetLOM()->abeLOMgetAvailableLanguages().key(ui->cbLangueRessource->currentText());
    m_abuleduFile->abeFileGetLOM()->abeLOMsetGeneralTitle(codeLangue,ui->leTitreGeneral->text());
    m_abuleduFile->abeFileGetLOM()->abeLOMsetGeneralDescription(codeLangue,ui->teDescription->document()->toPlainText());
    
    vCard vcard;
    vCardProperty name_prop  = vCardProperty::createName(ui->leAuteur->text(), "");
    vcard.addProperty(name_prop);
    vCardProperty fn_prop    = vCardProperty(VC_FORMATTED_NAME, ui->leAuteur->text());
    vcard.addProperty(fn_prop);
    vCardProperty note_prop  = vCardProperty(VC_NOTE, "");
    vcard.addProperty(note_prop);
    vCardProperty org_prop   = vCardProperty::createOrganization("");
    vcard.addProperty(org_prop);
    vCardProperty url_prop   = vCardProperty(VC_URL, "");
    vcard.addProperty(url_prop);
    
    m_abuleduFile->abeFileGetLOM()->abeLOMaddLifeCycleContributionRole("author", vcard, QDate::currentDate());
    m_abuleduFile->abeFileGetLOM()->abeLOMsetRightsCost("no");
    m_abuleduFile->abeFileGetLOM()->abeLOMsetRightsCopyrightAndOtherRestrictions("yes");
    //    m_abuleduFile->abeFileGetLOM()->abeLOMsetRightsDescription(codeLangue,ui->cbLicence->currentText());
    m_abuleduFile->abeFileGetLOM()->abeLOMsetAnnotationDate(QDate::currentDate());
    m_abuleduFile->abeFileGetLOM()->abeLOMsetGeneralLanguage(codeLangue);
    
    if(m_localDebug)qDebug()<<__LINE__<<listeFichiers<<"repertoire temp"<<m_repertoireTemporaire;
    // On passe les chemins et le type de fichier (abe ou zip) avant d'ouvrir la page de sauvegarde.
    
    m_abuleduFile->abeFileExportPrepare(listeFichiers,QDir(m_repertoireTemporaire).absolutePath(),QString("abe"));
}

void AbulEduEditeur::openConfigFile()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
}

void AbulEduEditeur::closeEvent(QCloseEvent *event)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(isWindowModified())
    {
        // On propose de sauvegarder
        int ret = QMessageBox::warning(this, trUtf8("Éditeur d'exercices"),
                                       trUtf8("Le Fichier a été modifié.\n"
                                              "Voulez-vous sauvegarder les modifications ?"),
                                       QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel,
                                       QMessageBox::Save);
        
        switch(ret)
        {
        case QMessageBox::Cancel: // Bouton annuler : on ne fait rien
            event->ignore();
            break;
        case QMessageBox::Discard: // Bouton Quitter sans enregistrer : on quitte
            event->accept();
            break;
        case QMessageBox::Save:  // Bouton enregistrer : on enregistre
            ui->swAssistant->setCurrentWidget(ui->pageSauvegarde);
            setWindowModified(false); // Pas très utile !
            event->ignore();
            break;
        }
    }
    else
    {
        // On quitte
        event->accept();
    }
}


void AbulEduEditeur::slotCloseAbulEduEditeur()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    emit editeurClosed();
}

void AbulEduEditeur::on_pbAjouteImage_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    etapes->abeWidgetAssistantEnablePrecedent(false);
    etapes->abeWidgetAssistantEnableSuivant(false);
    etapes->abeWidgetAssistantEnableClick(false);
    ui->swAssistant->setCurrentWidget(ui->pageChoixImages);
}



void AbulEduEditeur::onPageAssistantChanged(int page)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<"         page"<<page;
    Q_UNUSED(page)
}


void AbulEduEditeur::slotAjouterImageMediatheque()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(m_localDebug)qDebug()<<ui->widgetMediathequeGet->abeGetFile()->abeFileGetTitle();
    m_abuleduFile->abeFileAddMetaDataFromABB(ui->widgetMediathequeGet->abeGetLOM(),
                                             ui->widgetMediathequeGet->abeGetFile()->abeFileGetContent(0).baseName());
    ajouteImage(ui->widgetMediathequeGet->abeGetFile()->abeFileGetContent(0).absoluteFilePath());
}

void AbulEduEditeur::slotAcceptClicked() // appui sur le bouton ouvrir du sélecteur de fichier
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    m_fileDialogImages->setVisible(true);
}

void AbulEduEditeur::slotFileSelected(QString file)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if(m_localDebug)qDebug()<<__FUNCTION__<<"ligne"<<__LINE__;
    if(m_localDebug) qDebug()<<file;
    if(file!="")
    {
        ajouteImage(file);
    }
}

void AbulEduEditeur::slotRejectClicked() // appui sur le bouton annuler du sélecteur de fichier
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    ui->swAssistant->setCurrentWidget(ui->pageQuestions); // Retour à la page exercice
}

void AbulEduEditeur::slotSaveAcceptClicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
}

void AbulEduEditeur::slotGetFileForModifications()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    // On vient de recevoir un signal de fichier sélectionné
    //    m_abuleduFile = ui->widgetOpenProject->abeGetFile();
    slotOpenFileForModifications();
}

void AbulEduEditeur::slotOpenFileForModifications()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    
    m_repertoireTemporaire = QDir(m_abuleduFile->abeFileGetDirectoryTemp()).path();
    QDir().mkpath(m_repertoireTemporaire+"/data/");
    initialiseProjetAModifier();
    m_exerciceEnCours = 0;
    
    // On efface les anciens fichiers issus du .abe
    m_abuleduFile->abeCleanDirectory(m_repertoireTemporaire+"/data/", m_repertoireTemporaire+"/data/");
    etapes->abeWidgetAssistantEnablePrecedent(true);
    etapes->abeWidgetAssistantEnableSuivant(true);
    etapes->abeWidgetAssistantSetEtapeCourante(1);
}

void AbulEduEditeur::on_pbFinChoixImages_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    etapes->abeWidgetAssistantEnablePrecedent(true);
    etapes->abeWidgetAssistantEnableSuivant(true);
    etapes->abeWidgetAssistantEnableClick(true);
    ui->swAssistant->setCurrentWidget(ui->pageQuestions); // Retour à la page exercice
    //    ui->lwImagesExercice->setCurrentRow(0);
}

void AbulEduEditeur::saveExerciceEnCours()
{
    // Permet de sauvegarder l'exercice en cours d'édition
    
}

void AbulEduEditeur::slotChangementPageRequested(int pageDemandee)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<"Page demandée"<<pageDemandee;;
    
    switch(pageDemandee)
    {
    case 0:
        ui->swAssistant->setCurrentWidget(ui->pageAccueil);
        etapes->abeWidgetAssistantEnablePrecedent(false);
        etapes->abeWidgetAssistantEnableClick(false);
        break;
    case 1:
        ui->swAssistant->setCurrentWidget(ui->pageQuestions);
        etapes->abeWidgetAssistantEnablePrecedent(true);
        etapes->abeWidgetAssistantEnableClick(true);
        break;
    case 2:
        if(m_localDebug) qDebug()<<"ui.sauvegarde";
        ui->swAssistant->setCurrentWidget(ui->pageSauvegarde);
        saveExerciceEnCours(); // Au cas ou ! <-- sauf que il n'y a rien dedans !!
        // On écrit le fichier de config
        writeTemporaryConfigFile();
        writeTemporaryDataFiles();
//        prepareExport(); // Prépare la sauvegarde du projet
//        ui->widgetMediathequePush->setVisible(false);
        etapes->abeWidgetAssistantEnableSuivant(false);
        etapes->abeWidgetAssistantEnableClick(true);
        break;
    default:
        break;
    }
}

void AbulEduEditeur::on_pbMediathequePush_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    ui->swAssistant->setCurrentWidget(ui->pageMediathequePush);
}

void AbulEduEditeur::on_pbEssayer_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    emit tryThisSettings(m_abuleduFile);
}

void AbulEduEditeur::on_pbModifExerciceEnCours_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__<<m_abuleduFile->abeFileGetFileName().fileName();
    slotOpenFileForModifications();
}

void AbulEduEditeur::on_pbSaveOrdi_clicked()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    //    ui->wAbuleduBox->setVisible(true);
    //    ui->widgetMediathequePush->setVisible(false);
    //    ui->frBtnSave->setVisible(false);
    prepareExport();
    emit openModuleUpLoaderForEditeur();
}

void AbulEduEditeur::writeTemporaryDataFiles()
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    /* Il va falloir enregistrer les images */
}

void AbulEduEditeur::slotFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation box, QString chemin, bool Ok)
{
    Q_UNUSED(box)
    
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    if (m_localDebug) qDebug()<<"             Fichier"<<chemin<<" enregistré "<<Ok;
    slotChangementPageRequested(4);
}

void AbulEduEditeur::slotFileUploaded(int code)
{
    if (m_localDebug) qDebug()<<" ++++++++ "<< __FILE__ <<  __LINE__ << __FUNCTION__;
    switch(code)
    {
    case 1:
        if(m_localDebug)qDebug()<<"     Fichier uploadé";
        slotChangementPageRequested(4);
        break;
    default:
        if(m_localDebug)qDebug()<<"     Erreur d'upload";
        break;
    }
}


void AbulEduEditeur::on_btnAjouterQuestion_clicked()
{
    m_nombreQuestions++;
    QString question = trUtf8("Question ")+QString::number(m_nombreQuestions);
    ui->cbQuestion->addItem(question);
    ui->cbQuestion->setCurrentIndex(ui->cbQuestion->findText(question));
    AbulEduQuestionQCM* ques = new AbulEduQuestionQCM();
    ques->setObjectName("abeQuestion"+QString::number(m_nombreQuestions));
    ui->swQuestions->insertWidget(ui->swQuestions->count(),ques);
    ui->swQuestions->setCurrentWidget(ques);
    m_questions.append(ques);
}

void AbulEduEditeur::on_cbQuestion_currentIndexChanged(int index)
{
    ui->swQuestions->setCurrentIndex(index);
}

void AbulEduEditeur::on_chbAfficherConsigneSequence_toggled(bool checked)
{
    ui->teConsigneSequence->setVisible(checked);
}
