/** AbulEduEditeur
  *
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012-2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2012-2013 Icham Sirat <icham.sirat@ryxeo.com>
  * @author 2012-2013 Éric Seigne <eric.seigne@ryxeo.com>
  * @author 2012-2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */


#ifndef ABULEDUEDITEUR_H
#define ABULEDUEDITEUR_H

#include <QMainWindow>
#include <QTemporaryFile>
#include <QSettings>
#include <QDebug>
#include <QFileDialog>
#include <QCloseEvent>
#include <QListWidgetItem>
#include <QStackedWidget>
#include <phonon/phonon>
#include "abuleduapplicationv1.h"
#include "abuleduboxfilemanagerv1.h"
#include "abuledufilev1.h"
#include "abuledumediathequegetv1.h"
#include "abuleduwidgetassistantetapesv1.h"
#include "ui_abuleduediteur.h"
#include "leterriermainwindow.h"

#ifndef DEBUG_ABULEDUEDITEUR
#define DEBUG_ABULEDUEDITEUR 0
#endif
namespace Ui {
class AbulEduEditeur;
}

class AbulEduEditeur : public QWidget
{
    Q_OBJECT
    
public:
    explicit AbulEduEditeur(QWidget *parent = 0);
    ~AbulEduEditeur();
    
    void abeEditeurSetMainWindow(QWidget *mw);
    
private:
    Ui::AbulEduEditeur *ui;
    
    /** Stocke les répertoires de travail (enregistrements, accès au fichier de configuration, au texte) */
    QSharedPointer<AbulEduFileV1>   m_abuleduFile;
    AbulEduFileV1                  *m_abuleduTemporaryData;
    QString                         m_repertoireTemporaryData;
    QString                         m_repertoireTemporaire;
    QString                         m_userLeterrierAppDirectory;
    
    // Valeur des paramètres pour qsettings
    QTemporaryFile  m_config;
    QSettings      *m_settings;
    QString         m_titreGeneral;
    
    int             m_niveauDifficulte;
    int             m_exerciceEnCours;

    /** Stocke le nombre de questions du module */
    int m_nombreQuestions;
    
    /** Pointeur vers la mainwindow dans laquelle l'éditeur est inclus */
    QMainWindow *m_parent;
    QFileDialog *m_fileDialogImages;
    
    /** Le dernier chemin vers un répertoire d'images */
    QString m_lastPathImages;
    
    /** Pointeur vers le widget de navigation */
    AbulEduWidgetAssistantEtapesV1 *etapes;
    
    bool m_localDebug;

    QList<AbulEduQuestionQCM*> m_questions;

    /** Initialise un nouveau projet */
    void initialiseNouveauProjet();

    /** initialise un projet à modifier */
    void initialiseProjetAModifier();

    /** Initialise un nouvel exercice */
    void initialiseNouvelExercice();

    void ajouteImage();
    void ajouteImage(QString fileName);
    QString writeTemporaryImageFile(QString chemin);
    void writeTemporaryConfigFile();

    /** Prépare la sauvegarde des fichiers */
    void prepareExport();


    /** Peuple les variables avec un fichier.conf **/
    void openConfigFile();
private slots:
    void on_pbNouveauProjet_clicked();
    
    void on_pbModifierProjet_clicked();
    
    void on_leTitreGeneral_editingFinished();
    
    void closeEvent(QCloseEvent *event);
    
    void onPageAssistantChanged(int page);
    
    void slotAjouterImageMediatheque();
    
    void slotAcceptClicked();
    void slotFileSelected(QString file);
    void slotRejectClicked();
    
    void slotSaveAcceptClicked();
    
    void slotGetFileForModifications();
    void slotOpenFileForModifications();
    
    void slotCloseAbulEduEditeur();
    void on_pbAjouteImage_clicked();
    void on_pbFinChoixImages_clicked();
    void saveExerciceEnCours();
    
    void slotChangementPageRequested(int pageDemandee);
    
    void on_pbMediathequePush_clicked();
    
    void on_pbEssayer_clicked();
    
    void on_pbModifExerciceEnCours_clicked();
    
    void on_pbSaveOrdi_clicked();
    void writeTemporaryDataFiles();
    void slotFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation box, QString chemin, bool Ok);
    
    void slotFileUploaded(int code);
    
    void on_btnAjouterQuestion_clicked();

    void on_cbQuestion_currentIndexChanged(int index);

    void on_chbAfficherConsigneSequence_toggled(bool checked);

signals:
    
    /** Signal émis quand on quitte l'éditeur */
    void editeurClosed();
    
    /** Signal émis quand on veut ouvrir un module existant */
    void openModuleLoaderForEditeur();
    void openModuleUpLoaderForEditeur();
    /** Signal émis quand on veut essayer les paramètres des exercices */
    void tryThisSettings(QSharedPointer<AbulEduFileV1> file);
};

#endif // ABULEDUEDITEUR_H


