/** Classe de base pour les exercices des logiciels AbulEdu
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/LeTerrierExercice
  * @author 2012-2013 Éric Seigne <eric.seigne@ryxeo.com>
  * @author 2012-2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2012-2013 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef EXERCICEQCM_H
#define EXERCICEQCM_H

#include "abuleducommonstatesv1.h"

class ExerciceQCM : public AbulEduCommonStatesV1
{
    Q_OBJECT
public:
    ExerciceQCM(QWidget *parent, QString repertoireBase);
    ~ExerciceQCM();
private:
    /** Contrôle la possibilité d'affichage des debugs */
    bool m_localDebug;
    
    /** Parent de l'exercice, écrit dans une variable membre pour être accessible de tous les slots */
    QWidget *m_parent;
    
    /** Référence au répertoire dans lequel l'exercice est dézippé */
    QString m_repertoireBase;
    
    /** Référence au fichier de configuration de l'exercice */
    QSettings *m_config;
    
    /** Écriture dans metaObject()->className() dans un attribut de type QString */
    QString m_className;
    
    /** Dimensionne et place les widgets
      * @param ratio le ratio est par défaut celui calculé dans l'AbulEduApplicationV1 */
    void setDimensionsWidgets(float ratio = abeApp->getAbeApplicationDecorRatio());
    
    /* Attributs et méthodes liés à l'interface graphique de l'exercice */
    
    /** L'aire de jeu est une QGraphicsView la plupart du temps, mais vous pouvez la transformer en une autre classe à condition de répercuter partout vos transformations */
    QGraphicsView* gv_AireDeJeu;
    
    /** Pointeur vers la scène de l'attribut gv_AireDeJeu */
    QGraphicsScene *m_sceneAireDeJeu;

    /** QGraphicsProxyWidget de l'aire de jeu
      * Il permet d'intégrer l'aire de jeu dans l'AbulEduExerciceWidgetAireDeTravailV1 */
    QGraphicsProxyWidget *m_proxyAireDeJeu;

    /** Fonction à implémenter pour retirer pour remettre à zéro l'aire de jeu
      * Dans le cas où l'aire de jeu est une QGraphicsView, il s'agit d'enlever les éléments contenus dans sa scène */
    void clearScene();

    /* Attributs et méthodes liés à l'évaluation */
    
    /** Le nombre d'essais effectués pour répondre à la question COMMENTAIRE À REPRENDRE
     * On stocke le numéro de l'exercice puis le nombre de clics sur btnVerifier
     */
    QList<QPair<int, int> > m_resultats;

    /** Affichage de la question posée à l'exercice */
    void afficheQuestion();

    /** Affichage des réponses à choisir */
    void afficheReponsesPossibles();

    /** Fonction à implémenter pour vérifier la réponse suite à un clic sur btnVerifier
      @return nombre d'erreurs */
    int verifieReponse();
    
    /** Fonction à implémenter pour présenter la bonne réponse suite à un clic sur btnCorriger */
    int donneReponse();
    
    /** Construis ce dont a besoin la fonction setAbeLineLog() puis l'appelle */
    void construisLog();

    /** Map des Niveau pour la traduction
      * @levels numéro du niveau, trUtf8("Nom du niveau")
      */
    QMap<int, QString> m_levels;
    
    QString m_questionPosee;
    
    QString m_reponseDonnee;
    
    int m_score;
    
    int m_nbPrintedQuestions;
    
    QString m_reponseAttendue;
    
    QString m_tempsReponse;
    
    /** Nombre de réponses fausses
      * @note -1 correspond à une demande de correction */
    int m_verifieReponse;
    
    /** Identifiant des delayedPostEvent */
    int id_temporisation;
    
    /** Stocke le taux de réussite sous forme de pourcentage */
    qreal m_percent;
    
private slots:
    /** Enregistre la réponse donnée par l'utilisateur et lance la vérification */
    void etudieClic();

    /** Entrée dans l'état "sequence" de l'AbulEduStateMachineV1
      * Passage unique au lancement de la machine à états */
    void slotSequenceEntered();
    
    /** Entrée dans l'état "presentationSequence" de l'AbulEduStateMachineV1
      * Passage unique si la variable onPeutPresenterSequence vaut true, pas de passage si onPeutPresenterSequence vaut false */
    void slotPresenteSequenceEntered();
    
    /** Sortie de l'état "presentationSequence" de l'AbulEduStateMachineV1 */
    void slotPresenteSequenceExited();
    
    /** Entrée dans l'état "presentationExercices" de l'AbulEduStateMachineV1
      * Passage pour chaque exercice si la variable onPeutPresenterExercice vaut true */
    void slotPresentationExerciceEntered();
    
    /** Sortie de l'état "presentationExercices" de l'AbulEduStateMachineV1 */
    void slotPresentationExerciceExited();
    
    /** Entrée dans l'état "realisationExercice" de l'AbulEduStateMachineV1
      * Passage une seule fois au début de chaque exercice */
    void slotRealisationExerciceEntered();
    
    /** Entrée dans l'état "initQuestion" de l'AbulEduStateMachineV1
      * Passage en boucle à chaque nouvelle question
      * Par contre, on n'y repasse pas si on refait la même question */
    void slotInitQuestionEntered();
    
    /** Entrée dans l'état "question" de l'AbulEduStateMachineV1
      * Passage une fois au début de chaque question, puis en cas de réponse incorrecte ou clic sur btnRefaire */
    void slotQuestionEntered();
    
    /** Entrée dans l'état "afficheVerificationQuestion" de l'AbulEduStateMachineV1
      * Passage à chaque clic sur btnVerifier */
    void slotAfficheVerificationQuestionEntered();
    
    /** Entrée dans l'état "finVerificationQuestion" de l'AbulEduStateMachineV1
      * Passage en sortie de l'état "afficheVerificationQuestion" */
    void slotFinVerificationQuestionEntered();
    
    /** Entrée dans l'état "afficheCorrectionQuestion" de l'AbulEduStateMachineV1
      * Passage à chaque clic sur btnCorriger */
    void slotAfficheCorrectionQuestionEntered();
    
    /** Entrée dans l'état "finCorrectionQuestion" de l'AbulEduStateMachineV1
      * Passage en sortie de l'état "afficheCorrectionQuestion" */
    void slotFinCorrectionQuestionEntered();
    
    /** Entrée dans l'état "bilanExercice" de l'AbulEduStateMachineV1
      * Passage pour chaque exercice si la variable onPeutPresenterBilanExercice vaut true */
    void slotBilanExerciceEntered();
    
    /** Entrée dans l'état "bilanSequence" de l'AbulEduStateMachineV1
      * Passage unique en fin de séquence */
    void slotBilanSequenceEntered();
    
    /** Connecté au clic sur btnQuitter */
    void slotQuitter();
    
signals:
    /** Émis dans le destructeur pour signaler que l'exercice est terminé. Connecté au slot "abeBackFromExercice" de la LeTerrierMainWindow */
    void exerciseExited();
};

#endif // EXERCICEQCM_H

