/** Classe de base pour les mainwindows des logiciels AbulEdu (Leterrier-QCM)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-qcm
  * @author 2013 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * 
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/LeTerrierMainWindow
  * @author 2012 Philippe Cadaugade <philippe.cadaugade@ryxeo.com>
  * @author 2012 Éric Seigne <eric.seigne@ryxeo.com>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef LETERRIERMAINWINDOW_H
#define LETERRIERMAINWINDOW_H

/* Inclusion de classes Qt */
#include <QMainWindow>
#include <QDebug>
/* Inclusion de classes AbulEdu */
#include "abuleduapplicationv1.h"
#include "abuledufilev1.h"
#include "abuleduboxfilemanagerv1.h"
#include "abuledupageaccueilv1.h"
#include "activityfilter.h"
#include "exerciceqcm.h"

#ifndef DEBUG_LETERRIERMAINWINDOW
#define DEBUG_LETERRIERMAINWINDOW 0
#endif

namespace Ui {
class LeTerrierMainWindow;
}

class LeTerrierMainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    /* Méthodes */
    /** Constructeur */
    explicit LeTerrierMainWindow(QWidget *parent = 0);
    
    /** Destructeur virtuel pour cause d'objectif de polymorphisme */
    virtual ~LeTerrierMainWindow();
    
    /** Appelle la fonction setDimensionsWidget de la page d'accueil */
    void resizeEvent(QResizeEvent *);
    
    /** Getteur de l'attribut m_abuledufile */
    QSharedPointer<AbulEduFileV1> abeGetMyAbulEduFile();
    
    /** Setteur de l'attribut m_abuledufile */
    void abeSetMyAbulEduFile(QSharedPointer<AbulEduFileV1> file);
    
    /** Permet d'accéder à page d'accueil */
    AbulEduPageAccueilV1* abeGetMyAbulEduAccueil();
    
protected:
    /* Attributs */
    /** Variable membre qui permet de contrôler l'affichage des qDebug */
    bool                     m_localDebug;
    
    /** Variable membre de type AbulEduFileV1* qui représente le module ouvert dans l'application */
    QSharedPointer<AbulEduFileV1> m_abuleduFile;
    
    /** Variable membre de type AbulEduBoxFileManagerV1, initialisée en mode save */
    AbulEduBoxFileManagerV1 *m_abuleduFileManager;
    
    QMap<int, QString>      *m_speechBubbles;
    
    /** Stocke le numéro de l'exercice en cours d'exécution
      * @value la valeur -1 correspond à "pas d'exercice en cours" (valeur d'initialisation)
      * @value la valeur 0 correspond à l'éditeur */
    int                      m_exerciseNumber;
    
    /* Méthodes */
    /** Crée dans le menu Langues de l'interface graphique une entrée pour les langues désirées
      * Seuls l'anglais et le français sont présents au départ
      * Quelques langues sont prêtes à ajouter, en décommentant le bloc qui leur correspond
      * Vous pouvez ajouter d'autres langues sur le même modèle */
    virtual void createLangMenu();
    
private:
    /** Gestion de l'affichage de la fenetre pour les tablettes */
    void showEvent ( QShowEvent * event );
    
    /* Attributs */
    /** Pointeur vers l'interface graphique */
    Ui::LeTerrierMainWindow* ui;
    
    /** QTranslator destiné aux traductions Qt System */
    QTranslator              m_qtTranslator;
    
    /** QTranslator destiné aux traductions spécifiques du logiciel */
    QTranslator              m_myAppTranslator;
    
    /** Page d'accueil dont le fichier de paramètres, settings.conf, doit être créé grâce au logiciel AbulEduEditeurPageAccueil */
    AbulEduPageAccueilV1*    m_abuleduPageAccueil;
    
    /** Stocke l'envie de l'utilisateur de voir les bulles s'afficher toutes les 7 secondes
      * Initialisé à true dans le constructeur et mis à jour par la case à cocher du menu Option/Mode Démo */
    bool                     m_isDemoAvailable;
    
    /** AbulEduMessageBoxV1 utilisé dans le slotDemo() et passé à la fonction finDemo() */
    AbulEduMessageBoxV1*     m_abuleduMessageBox;
    
    /** Pointeur vers un l'AbulEduNetworkAccessManagerV1 de l'application, assignation dans le constructeur */
    AbulEduNetworkAccessManagerV1 *m_nam;
    
    /** Permet de savoir si un exercice est lancé */
    bool m_isExerciseRunning;
    
    /** Permet de savoir si on est en cours d'édition */
    bool m_isEditorRunning;
    
public slots:
    /** Demande une authentification si nécessaire
      * A connecter au signal signalSessionAuthenticated(bool) de l'AbulEduSplashScreen de l'application */
    void slotSessionAuthenticated(bool enable);
    
protected slots:
    /** Lance l'exercice repéré par le numéro de l'attribut m_exerciseNumber
      * @param l'initialisation à 0 est utilisée dans le cas où un exercice est lancé depuis le menu Exercices */
    virtual void slotAbeLaunchExercise(int number = 0);
    
    /** Destiné à être connecté à un signal exited() des exercices */
    virtual void slotAbeBackFromExercise();
    
    /** Permet d'afficher en cas d'inactivité les zones à cliquer pour lancer les exercices
      * Elle est virtuelle pour permettre de déplacer m_abuleduMessageBox en fonction du contenu de la page d'accueil */
    virtual void slotDemo();
    
    /** Cache les zones et bulles de lancement des exercices
      * Appelée par slotDemo() après 4 secondes d'affichage */
    virtual void slotFinDemo();
    
private slots:
    /** Opère le changement de langue désiré
      * Connecté aux entrées du menu Langues */
    void slotChangeLang();
    
    /** Passe à la page du sélecteur de modules (pour l'éditeur) */
    void openModuleLoader();
    
    
    /** Passe à la page du sélecteur de modules (pour l'éditeur) */
    void openModuleUpLoader();
    
    /** Ouvre un module */
    void slotOpenFile();

    /** Sauvegarde un module */
    void slotAfficheEtatEnregistrement(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation enumLocation, QString fileName, bool success);

    /** Actualise le nom de la fenêtre en fonction du nom du logiciel, du nom de l'exercice et du nom de l'utilisateur */
    void setTitle(int authStatus);
    
    /** Permet de gerer les changement d'états d'authentification sso
      * @param code est l'entier envoyé dans le signal ssoAuthStatus(int) de l'AbulEduNetworkAccessManagerV1
      * avec lequel ce slot est connecté */
    void slotSSOStatusChanged(int code);
    
    /** Affiche m_abuleduFileManager, dont le signal signalAbeFileSelected est connecté au slotOpenFile() */
    void on_action_Ouvrir_un_module_triggered();
    
    /** Code à exécuter au clic sur l'entrée de menu Fichier - Editeur de modules */
    void on_action_Editeur_de_modules_triggered();
    
    /** Code à exécuter au clic sur l'entrée de menu Fichier - Quitter */
    void on_actionQuitter_triggered();
    
    /** Affiche le journal d'activités de l'utilisateur connecté */
    void on_action_Journal_de_mes_activit_s_triggered();
    
    /** Ouvre une popup d'authentification */
    void on_action_Authentification_triggered();
    void tryNewSettings(QSharedPointer<AbulEduFileV1> file);
    void backToEditor();
    void editorClosed();
};

#endif // LETERRIERMAINWINDOW_H

