/**
  * Classe AbulEduExerciceCommonV1
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduexercicecommonv1.h"

AbulEduExerciceCommonV1::AbulEduExerciceCommonV1(QObject *parent)
{
    m_localDebug       = 0;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Initialisation des différentes variables
    m_parent           = parent;
    m_exerciceName     = "";
    m_level            = "";
    m_nbTotalQuestions = 0;
    setAbeNumQuestion(0);
    m_skill            = "";
    m_isLivretComplete = true;
    m_arrayLogs.clear();
    m_downloadFilter.clear();
    m_isAnswerShownBeforeStart=false;
    m_numQuestionInSequence  = 0;
    m_nbTotalQuestionsSequence = 0;
    m_evaluation = abe::evalY;
    m_abuleduExerciceLogs = new AbulEduExerciceLogsV1();
}

AbulEduExerciceCommonV1::~AbulEduExerciceCommonV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete m_abuleduExerciceLogs;
}

void AbulEduExerciceCommonV1::setAbeLineLog(QString question, QString answer,
                                      int score, int nbPrintedQuestions, abe::ABE_EVAL evaluation,
                                            QString expected, QString answerDuration, QString answerIntermediate, QString answerProposed)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Construction de la structure de données
    QHash <QString, QString> line;
    line.insert("exerciceName", m_exerciceName);
    line.insert("level", m_level);
    line.insert("score", QString::number(score));
    line.insert("nbTotalQuestions", QString::number(m_nbTotalQuestions));
    line.insert("nbPrintedQuestions", QString::number(nbPrintedQuestions));
    line.insert("question", question);
    line.insert("answer", answer);
    line.insert("evaluation", QString::number(evaluation));
    if(expected != "")
        line.insert("expected", expected);
    if(m_skill != "")
        line.insert("skill", m_skill);
    if(answerDuration != "")
        line.insert("answerDuration", answerDuration);
    if(answerIntermediate != "")
        line.insert("answerIntermediate", answerIntermediate);
    if(answerProposed != "")
        line.insert("answerProposed", answerProposed);

    m_arrayLogs.insert(m_arrayLogs.count(),line);
}

void AbulEduExerciceCommonV1::resetAbeLogs()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayLogs.clear();
}

QHash<int, QHash<QString, QString> >  AbulEduExerciceCommonV1::getPluginLogs(QString login, QString group, QString exerciceName)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayLogs;
    }
    if(exerciceName != "") {
        QHash<int, QHash<QString, QString> > resultat;
        for(int i=0; i<m_arrayLogs.count(); i++) {
            QHash<QString, QString> ligne = m_arrayLogs[i];
            if(ligne["exerciceName"] == exerciceName) {
                resultat.insert(resultat.count(),ligne);
            }
        }
        return resultat;
    }
    else
        return m_arrayLogs;
}

QHash<int, QHash<QString, QString> >  AbulEduExerciceCommonV1::getPluginLogsDownloaded()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_arrayLogsDownloaded;
}

QHash<int, QString> AbulEduExerciceCommonV1::getPluginUsers()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayUsers;
    }
    return m_arrayUsers;
}

QHash<QString, QString> AbulEduExerciceCommonV1::getPluginGroups()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayGroups;
    }
    return m_arrayGroups;
}

QHash<QString, QString> AbulEduExerciceCommonV1::getPluginDomains()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayDomains;
    }
    return m_arrayDomains;
}

QHash<int, QString> AbulEduExerciceCommonV1::getPluginApplications()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayApplications;
    }
    return m_arrayApplications;
}

QHash<int, QString> AbulEduExerciceCommonV1::getPluginSkills()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arraySkills;
    }
    return m_arraySkills;
}

QHash<int, QString> AbulEduExerciceCommonV1::getPluginExercices()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayExercices;
    }
    return m_arrayExercices;
}

QHash<int, QString> AbulEduExerciceCommonV1::getPluginLevels()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  " << m_arrayLevels;
    }
    return m_arrayLevels;
}

void AbulEduExerciceCommonV1::pushAbulEduLogs(QString moduleName)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //2012 on a maintenant notre infra sso, on peut donc expédier les logs sans passer par le plugin
    if(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus() == 1) {
        //La variable qui contient tout ce qu'on expedie au serveur
        QVariantMap tableau;

        //Les informations de base
        tableau["module_name"]                   = moduleName;

        //Et toutes les questions
        for(int i=0; i<m_arrayLogs.count(); i++)
        {
            tableau[QString::number(i)+"_exercice_name"]                 = m_arrayLogs[i]["exerciceName"];
            tableau[QString::number(i)+"_exercice_level"]                = m_arrayLogs[i]["level"];
            tableau[QString::number(i)+"_exercice_skill"]                = m_arrayLogs[i]["skill"];
            tableau[QString::number(i)+"_exercice_score"]                = m_arrayLogs[i]["score"];
            tableau[QString::number(i)+"_exercice_nb_total_questions"]   = m_arrayLogs[i]["nbTotalQuestions"];
            tableau[QString::number(i)+"_exercice_nb_printed_questions"] = m_arrayLogs[i]["nbPrintedQuestions"];
            tableau[QString::number(i)+"_exercice_evaluation"]           = m_arrayLogs[i]["evaluation"];

            tableau[QString::number(i)+"_question_date"]                 = QDateTime::currentDateTime().toString("yyyy-MM-dd HH:mm:ss");
            tableau[QString::number(i)+"_question_question"]             = m_arrayLogs[i]["question"];
            tableau[QString::number(i)+"_question_answer"]               = m_arrayLogs[i]["answer"];
            tableau[QString::number(i)+"_question_expected"]             = m_arrayLogs[i]["expected"];
            tableau[QString::number(i)+"_question_answer_time"]          = m_arrayLogs[i]["time"];
            tableau[QString::number(i)+"_question_answer_intermediate"]  = m_arrayLogs[i]["intermediate"];
            tableau[QString::number(i)+"_question_proposed"]             = m_arrayLogs[i]["proposed"];
            tableau[QString::number(i)+"_question_evaluation"]           = m_arrayLogs[i]["evaluation"];

        }
        tableau["exercice_nb_questions_log"]     = m_arrayLogs.count();

        m_abuleduExerciceLogs->abeLogsPush(tableau);
    }

    /** @todo on garde ou poubelle ? */
    QEvent* logEvent = new QEvent(AbulEduLogsPush);
    QApplication::postEvent(m_parent, logEvent);
}


void AbulEduExerciceCommonV1::setAbeExerciceName(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_exerciceName = s;
}

void AbulEduExerciceCommonV1::setAbeExerciceEvaluation(abe::ABE_EVAL e)
{
    m_evaluation = e;
}


void AbulEduExerciceCommonV1::setAbeSkill(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_skill = s;
}

QString AbulEduExerciceCommonV1::getAbeExerciceName()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_exerciceName;
}

abe::ABE_EVAL AbulEduExerciceCommonV1::getAbeExerciceEvaluation()
{
    return m_evaluation;
}

QString AbulEduExerciceCommonV1::getAbeLevel()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_level;
}

int AbulEduExerciceCommonV1::getAbeNbTotalQuestions()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_nbTotalQuestions;
}

QString AbulEduExerciceCommonV1::getAbeSkill()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_skill;
}

QHash<QString, QString> AbulEduExerciceCommonV1::downloadPluginLogsFilter()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_downloadFilter;
}

void AbulEduExerciceCommonV1::setPluginLogs(QHash<int, QHash<QString, QString> > h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayLogsDownloaded.clear();
    if(h.count() > 0)
        m_arrayLogsDownloaded =  h;
}


void AbulEduExerciceCommonV1::setPluginUsers(QHash<int, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayUsers.clear();
    m_arrayUsers = h;
}

void AbulEduExerciceCommonV1::setPluginGroups(QHash<QString, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayGroups.clear();
    m_arrayGroups = h;
}

void AbulEduExerciceCommonV1::setPluginDomains(QHash<QString, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayDomains.clear();
    m_arrayDomains = h;
}

void AbulEduExerciceCommonV1::setPluginApplications(QHash<int, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayApplications.clear();
    m_arrayApplications = h;
}

void AbulEduExerciceCommonV1::setPluginSkills(QHash<int, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arraySkills.clear();
    m_arraySkills = h;
}

void AbulEduExerciceCommonV1::setPluginExercices(QHash<int, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayExercices.clear();
    m_arrayExercices = h;
}

void AbulEduExerciceCommonV1::setPluginLevels(QHash<int, QString> h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayLevels.clear();
    m_arrayLevels = h;
}


void AbulEduExerciceCommonV1::downloadAbulEduLogs()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduLogsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduUsers()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduUsersDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduGroups()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduGroupsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduDomains()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduDomainsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduApplications()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduApplicationsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduSkills()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduSkillsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduExercices()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduExercicesDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::downloadAbulEduLevels()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QEvent* downloadEvent = new QEvent(AbulEduLevelsDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::setAbeDownloadLogsFilter(QDate dateBegin, QDate dateEnd, QString login, QString group, QString evaluation, QString skill)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_downloadFilter.insert("login",login);
    m_downloadFilter.insert("group",group);
    m_downloadFilter.insert("dateBegin",dateBegin.toString("yyyy-MM-dd"));
    m_downloadFilter.insert("dateEnd",dateEnd.toString("yyyy-MM-dd"));
    m_downloadFilter.insert("evaluation",evaluation);
    m_downloadFilter.insert("skill",skill);
}

void AbulEduExerciceCommonV1::setAbeDebugLevel(int i)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_localDebug = i;
}

void AbulEduExerciceCommonV1::downloadAbulEduLivret(QString niveau, bool isComplete)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_level = niveau;
    m_isLivretComplete = isComplete;

    QEvent* downloadEvent = new QEvent(AbulEduLivretDownload);
    QApplication::postEvent(m_parent, downloadEvent);
}

void AbulEduExerciceCommonV1::setPluginLivret(QHash<int, QHash<int, QString> > h)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_arrayLivret.empty();
    m_arrayLivret = h;
}

QHash<int, QHash<int, QString> > AbulEduExerciceCommonV1::getPluginLivret()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_arrayLivret;
}

bool AbulEduExerciceCommonV1::getAbeIsLivretComplete()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_isLivretComplete;
}

void AbulEduExerciceCommonV1::setAbeIsLivretComplete(bool isComplete)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_isLivretComplete = isComplete;
}


int AbulEduExerciceCommonV1::getLocalLogs(QString exerciceName)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(exerciceName != "") {
        for(int i=0; i<m_arrayLogs.count(); i++) {
            QHash<QString, QString> ligne = m_arrayLogs[i];
            if(ligne["exerciceName"] == exerciceName) {
                m_cache_arrayLogs.insert(m_cache_arrayLogs.count(),ligne.values());
            }
        }
        //        qDebug() << " ****************************************************************** ";
        //        qDebug() << m_cache_arrayLogs;
        //        qDebug() << " ****************************************************************** ";
        return m_cache_arrayLogs.count();
    }
    else
        return m_arrayLogs.count();
}

QStringList AbulEduExerciceCommonV1::getLocalLineLog(int numLigneLog, QStringList masque)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QList < int > indicesAEnvoyer;
    QStringList retour;

    if(m_cache_arrayLogs.count() > 0 && numLigneLog>=0) {
        //On cherche les indices du masque
        if(masque.count()>0) {
            QStringList entete = m_arrayLogs[0].keys();

            //Parcours du masque
            QStringListIterator iMasque(masque);
            while(iMasque.hasNext()) {
                //On ajoute ce numero d'index a notre liste d'indexes a retourner
                indicesAEnvoyer.append(entete.indexOf(iMasque.next()));
            }

            if(indicesAEnvoyer.count()>0) {
                QListIterator < int > iIndice(indicesAEnvoyer);
                while(iIndice.hasNext()) {
                    retour.append(m_cache_arrayLogs[numLigneLog][iIndice.next()]);
                }
            }
        }
        else {
            retour = m_cache_arrayLogs[numLigneLog];
        }
    }
    return retour;
}


void AbulEduExerciceCommonV1::setAbeLevel(QString s)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_level = s;
}

void AbulEduExerciceCommonV1::setAbeNbTotalQuestions(int n)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_nbTotalQuestions = n;
}
