/**
  * Classe
  * @author 2011 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef ABULEDUEXERCICEWIDGETMESSAGEV1_H
#define ABULEDUEXERCICEWIDGETMESSAGEV1_H

#include "ui_abuleduexercicewidgetmessagev1.h"
#include "abuleduapplicationv1.h"
#include "abuleduflatboutonv1.h"
#include <QGraphicsProxyWidget>
#include <QWidget>
#include <QGraphicsPixmapItem>
#include <QTextEdit>
#include <QLabel>

/** @file abuleduexercicewidgetmessagev1.h
  * @version 1
  */

namespace Ui {
    class AbulEduExerciceWidgetMessageV1;
}

class AbulEduExerciceWidgetMessageV1 : public QWidget
{
    Q_OBJECT

public:

    /** Constructeur de la classe
      * @param QWidget *parent, un pointeur sur le widget parent de l'objet, initialise par defaut a 0
      */
    explicit AbulEduExerciceWidgetMessageV1(QWidget *parent = 0);

    /** Destructeur par defaut de la classe
      */
    ~AbulEduExerciceWidgetMessageV1();

    /**
      * Pour fermer le widget message et appeller le destructeur
      */
    void abeWidgetMessageClose();

    Ui::AbulEduExerciceWidgetMessageV1 *ui;

    /** Permet de definir l'image de fond du widget
      * @param chemin, chaine de caractere, le chemin de l'emplacement de l'image
      * @return void
      */
    void abeWidgetMessageSetImageFond(QString chemin);

    void abeWidgetMessageShowImageFond(bool trueFalse);

    bool abeWidgetMessageHasImageFond();

    /** Permet de definir le titre
      * @param titre, chaine de caractere, le titre du message
      * @return void
      */
    void abeWidgetMessageSetTitre(QString titre);

    /** Permet de definir la consigne
      * @param consigne, chaine de caractere, la consigne que l'on veut appliquer
      * @return void
      */
    void abeWidgetMessageSetConsigne(QString consigne);

    /** Permet de definir le texte de l'exercice
      * @param textExercice, chaine de caractere, le texte de l'exercice
      * @return void
      */
    void abeWidgetMessageSetTexteExercice(QString textExercice);

    /** Permet de redimensionner le widgetMessage
      * @brief appel de la fonction AbulEduExerciceWidgetMessageV1::setDimensionWidget()
      * @see setDimensionsWidget()
      * @return void
      */
    void abeWidgetMessageResize();

    /** Permet de rendre visible ou non la zone de texte
      * @param yesNo, booleen, true si on veut l'afficher, sinon false
      * @return void
      */
    void abeWidgetMessageSetZoneTexteVisible(bool yesNo);

    /** Met le booléen m_isDisplayError à vrai */
    void abeWidgetMessageSetIsDisplayError();

    /** Rend le bouton Fermer visible ou pas */
    void abeWidgetSetBtnFermerVisible(bool yesNo);

protected:
    /** Permet de changer d'evenement
      * @param *e, QEvent, un pointeur sur un evenement
      * @link http://qt-project.org/doc/qt-4.8/QEvent.html
      */
    void changeEvent(QEvent *e);

private:
    QGraphicsScene        *gsPrincipale;
    QString                m_cheminFond;
    QGraphicsPixmapItem   *m_fond;
    QPixmap                m_imageFond;
    QFrame *frPrincipale;
    QWidget *widget;
    QVBoxLayout *m_vLayout;
    QLabel *lbTitre;
    QLabel *lbSeparationTitre;
    QTextEdit *teConsigne;
    QLabel *lbSeparationConsigne;
    QTextEdit *teTextExercice;
    QSpacerItem *spacer;
    bool m_localDebug;
    float m_previousRatio;
    bool m_isDisplayError;
    bool m_hasImageFond;
    QWidget* m_parent;

private slots :

    /** Permet de redimensionner le widget à la bonne taille
      * @return void
      */
    void setDimensionsWidget();

    /** Gere l'evenement appuie sur la touche
      */
    void keyPressEvent(QKeyEvent *event);

    /** Gere l'evenement relachement de la touche
      */
    void keyReleaseEvent(QKeyEvent *event);

    /** Gere l'événement relachement de la souris
      * Cache cet AbulEduExerciceWidgetMessageV1 si c'est un "message d'erreur" */
    void mouseReleaseEvent(QMouseEvent * event);

    QString scaleImgWidthHtml(QString html,float ratio);
    QString scaleImgHeightHtml(QString html,float ratio);

    void btnFermer_clicked();

};

#endif // ABULEDUEXERCICEWIDGETMESSAGEV1_H
