/** Classe AbulEduBoxFileManagerV1
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @copyright The GNU Public License (GNU/GPL) v3
  */
  /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUBOXFILEMANAGERV1_H
#define ABULEDUBOXFILEMANAGERV1_H

#ifndef DEBUG_ABULEDUBOXFILEMANAGERV1
#define DEBUG_ABULEDUBOXFILEMANAGERV1 0
#endif

#include <QWidget>
#include <QTreeWidgetItem>
#include <QTableWidgetItem>
#include <QDebug>
#include <QFileDialog>
#include "ui_abuleduboxfilemanagerv1.h"
#include "maiaXmlRpcClient.h"
#include "abuledufilev1.h"

namespace Ui {
class AbulEduBoxFileManagerV1;
}

/**
  * La classe AbulEduBoxFileManagerV1 se propose de remplacer la boite QFileOpen de Qt qui affiche une boite systeme.
  * La AbulEduBoxFileManagerV1 offre des spécificités intéressantes pour notre objectif:
  *   - Affichage simple (on affiche que les vignettes, plutôt à destination du public non lecteur)
  *
  *
  * @image html "Capture-Gestionnaire de fichiers AbulÉduBox.png"
  *
  *
  *   - Affichage complet qui permet de faire des recherches sur sa box perso, la box du groupe, la médiathèque ...
  *
  *
  * @image html "Capture-Gestionnaire de fichiers AbulÉduBox-1.png"
  */

class AbulEduBoxFileManagerV1 : public QWidget
{
    Q_OBJECT
    Q_ENUMS(enumAbulEduBoxFileManagerOpenOrSave)
    Q_ENUMS(enumAbulEduBoxFileManagerDisplaySimpleOrComplete)
    Q_ENUMS(enumAbulEduBoxFileManagerSavingLocation)
    Q_PROPERTY(enumAbulEduBoxFileManagerOpenOrSave openOrSave READ abeGetOpenOrSaveEnum WRITE abeSetOpenOrSaveEnum USER true)
    Q_PROPERTY(enumAbulEduBoxFileManagerDisplaySimpleOrComplete displaySimpleOrComplete READ abeGetDisplaySimpleOrCompleteEnum WRITE abeSetDisplaySimpleOrCompleteEnum USER true)

public:
    enum enumAbulEduBoxFileManagerOpenOrSave { abeOpen, abeSave };
    enum enumAbulEduBoxFileManagerDisplaySimpleOrComplete { abeDisplaySimple, abeDisplayComplete };
    enum enumAbulEduBoxFileManagerSavingLocation {abePC, abeBoxPerso, abeBoxShare, abeMediatheque};
    static inline QMap<enumAbulEduBoxFileManagerSavingLocation,QString> ABE_ENUMTOSTRING() {
        QMap<enumAbulEduBoxFileManagerSavingLocation,QString> map;
        map.insert(abePC,"personnel-pc");
        map.insert(abeBoxPerso,"abebox-personnelle");
        map.insert(abeBoxShare,"abebox-partagee");
        map.insert(abeMediatheque,"abe-mediatheque");
        return map;
    }

    /**
      * Constructeur
      */
    explicit AbulEduBoxFileManagerV1(QWidget *parent = 0, enumAbulEduBoxFileManagerOpenOrSave openOrSave = abeOpen);

    /**
      * Destructeur
      */
    ~AbulEduBoxFileManagerV1();

    /**
      * pour passer un AbulEduFileV1 a cette classe, à quoi ça sert ?
      * -> ça évite d'en créer un nouveau si le développeur de l'application a déjà un objet de ce type
      * @param *file pointeur sur un objet de type AbulEduFileV1
      */
    void abeSetFile(QSharedPointer<AbulEduFileV1> file);

    /**
      * Retourne le pointeur sur un objet AbulEduFileV1 que l'utilisateur a sélectionné
      * @return *AbulEduFileV1
      */
    QSharedPointer<AbulEduFileV1> abeGetFile();

    /**
      * Permet de basculer d'une ouverture de fichier à une boite utilisable pour choisir
      * de sauvegarder un fichier
      * @param openOrSave cf enumAbulEduBoxFileManagerOpenOrSave
      */
    void abeSetOpenOrSaveEnum(enumAbulEduBoxFileManagerOpenOrSave openOrSave);

    /**
      * Permet de retourner l'information
      */
    enumAbulEduBoxFileManagerOpenOrSave abeGetOpenOrSaveEnum() const;

    /** Getteur public de l'attribut privé m_sender */
    QObject* abeGetSender();

    /** Getteur public de l'attribut privé m_sender */
    void abeSetSender(QObject* s);

    /**
      * Permet de basculer d'une ouverture de fichier à une boite simple ou complete
      * de sauvegarder un fichier
      * @param openOrSave cf enumAbulEduBoxFileManagerOpenOrSave
      */
    void abeSetDisplaySimpleOrCompleteEnum(enumAbulEduBoxFileManagerDisplaySimpleOrComplete simpleOrComplete);

    /**
      * Permet de retourner l'information "boite ouverte en mode simple ou complet"
      */
    enumAbulEduBoxFileManagerDisplaySimpleOrComplete abeGetDisplaySimpleOrCompleteEnum() const;

    /**
      * Permet de forcer l'affichage du contenu d'une box
      */
    void abeRefresh(enumAbulEduBoxFileManagerSavingLocation location);

signals:
    /**
      * Sinal lancé lorsqu'on double clique sur un fichier ou qu'un fichier est téléchargé. Il permet au développeur de l'application
      * de connecter le slot qu'il souhaite pour gérer cet évènement.
      *
      * Dans le fichier .h on ajoute un objet de type AbulEduBoxFileManagerV1
      * @code
private:
  AbulEduBoxFileManagerV1* m_abuleduFileManager;

private slots:
  void slotOpenFile();
      * @endcode
      *
      * Ensuite, Dans le constructeur du fichier .cpp on initialise l'objet et on connecte le slot
      *
      * @code
m_abuleduFileManager = new AbulEduBoxFileManagerV1();
connect(m_abuleduFileManager, SIGNAL(signalAbeFileSelected()),this, SLOT(slotOpenFile()));
      * @endcode
      *
      * Et enfin, l'implémentation du slot slotOpenFile():
      *
      * @code
m_abuleduFile = m_abuleduFileManager->abeGetFile();
//On cache le filemanager
m_abuleduFileManager->hide();
      * @endcode
      *
      */
    void signalAbeFileSelected();

    /**
      * Nouvelle implementation, on pousse a migrer de la deprecated vers celle ci
      */
    void signalAbeFileSaved(AbulEduBoxFileManagerV1::enumAbulEduBoxFileManagerSavingLocation quelleBox, QString fileName, bool reussi);

    /**
      * On ferme la fenêtre ou on la cache
      */
    void signalAbeFileCloseOrHide();

private slots:
    /**
      * Réception du flux RPC de liste des fichiers
      */
    void rpcAbulEduBoxFileList(QVariant &v, QNetworkReply *reply);

    /**
      * Réception du flux RPC de récupération d'un fichier
      */
    void rpcAbulEduBoxGetFile(QVariant &v, QNetworkReply *reply);

    /**
      * Gestion des erreurs XMLRPC
      */
    void rpcFault(int code, const QString &errorMessage);

    /**
      * Gestion des erreurs XMLRPC: upload de fichier
      * @todo: vérifier si ce slot est bien appellé en cas de timeout
      */
    void rpcFaultFileUpload(int code, const QString &errorMessage);

    /**
      * Gestion de l'upload de fichiers
      */
    void rpcFileUpload(QVariant &v, QNetworkReply *reply);

    /**
      * Récupère les vignettes sur le serveur de fichiers distants
      */
    void rpcGetThumbnails(QVariant&, QNetworkReply*);

    /**
      * gestion de l'upload pour expédier un fichier sur la box
      * si groupe est non nul ou ne vaut pas "perso" c'est qu'on souhaite
      * sauvegarder le fichier dans un dossier partagé
      */
    void slotUploadFile(QString fileName, QString groupe = "");

    /**
      * Gestion du double clic sur le listing des fichiers
      */
    void on_twComplet_itemDoubleClicked(QTableWidgetItem *item);

    /**
      * Clic sur le bouton rechercher
      */
    void on_leRecherche_returnPressed();

    /**
      * Clic sur le bouton "affichage simple"
      */
    void on_btnSimple_clicked();

    /**
      * Clic sur le bouton "affichage complet"
      */
    void on_btnComplet_clicked();

    void on_lwSimple_itemDoubleClicked(QListWidgetItem *item);

    void on_btnSimpleMonPC_clicked();
    void on_btnSimpleBoxPerso_clicked();
    void on_btnSimpleBoxPartagee_clicked();
    void on_btnSimpleOuvrir_clicked();

    void on_btnCompletMonPC_clicked();
    void on_btnCompletMediatheque_clicked();
    void on_btnCompletBoxPerso_clicked();
    void on_btnCompletBoxPartagee_clicked();
    void on_btnCompletOuvrir_clicked();
    void on_btnCompletCopier_clicked();
    void on_btnCompletColler_clicked();

    /**
      * Supprimer un fichier
      */
    void on_btnCompletSupprimer_clicked();
    void on_btnCompletAjouter_clicked();

    void nettoyageComplet();
    void on_btnNomFichierSauvegarder_clicked();

    /**
      * Special en mode "sauvegarde", le clic sur un item -> nom du fichier dans la zone sauvegarde en bas
      */
    void on_twComplet_itemClicked(QTableWidgetItem *item);

    /**
      * Factorisation du code de sauvegarde
      */
    void sauvegardeFichier(QString filename);

    /**
      * Factorisation du code d'ouverture
      */
    void ouvertureFichier(QString filename, QString groupe = "");

    void on_btnCompletFermer_clicked();
    void on_btnSimpleFermer_clicked();

    void resizeEvent(QResizeEvent *);

    /** Redirection en cas d'échec d'authentification */
    void ifAuthenticateFailed();

    void on_tbRecherche_clicked();

private:
    Ui::AbulEduBoxFileManagerV1 *ui;
    int m_localDebug;
    MaiaXmlRpcClient *m_rpcAbulEduBox;
    AbulEduNetworkAccessManagerV1 *m_nam;
    QSharedPointer<AbulEduFileV1> m_abuleduFile;
    QNetworkReply   *m_networkReplyDownload;
    enumAbulEduBoxFileManagerSavingLocation m_quelleBox;
    QDir m_dirABE;
    QDir m_dirThumbnailsABE;
    enumAbulEduBoxFileManagerOpenOrSave m_openOrSave;
    enumAbulEduBoxFileManagerDisplaySimpleOrComplete m_displaySimpleOrComplete;
    QWidget* m_parent;
    QFrame *m_frmBandeauBackground;

    /** Permet de garder en mémoire un QObject*. Dans le cas où l'affichage de cette AbulEduBoxFileManagerV1 est déclenchée
      * dans un slot, ce QObject* est l'objet dont un signal a provoqué l'exécution de ce slot */
    QObject* m_sender;
};

#endif // ABULEDUBOXFILEMANAGERV1_H
