/** Classe AbulEduMediathequeGetV1
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */


#include "abuledumediathequegetv1.h"

AbulEduMediathequeGetV1::AbulEduMediathequeGetV1(QWidget *parent, QString mediathequeOrData) :
    QWidget(parent),
    AbulEduMediathequeCommon(),
    ui(new Ui::AbulEduMediathequeGetV1)
{
    m_localDebug = DEBUG_ABULEDUMEDIATHEQUEGETV1;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_nam = abeApp->getAbeNetworkAccessManager();
    ui->setupUi(this);
    //Surtout pas modale sinon la popup sso ne peut pas se placer au dessus !
    //    setWindowModality(Qt::ApplicationModal);

    //2013.01.16 : decision pour la tablette
    #ifdef __ABULEDUTABLETTEV1__MODE__
    setWindowModality(Qt::ApplicationModal);
    #endif

    if(mediathequeOrData == "data") {
        m_rpc = new MaiaXmlRpcClient(this);
        abeSetSourceEnum(AbulEduMediathequeGetV1::abeData);
    }
    else {
        m_rpc = new MaiaXmlRpcClient(this);
        abeSetSourceEnum(AbulEduMediathequeGetV1::abeMediatheque);
    }

    //==================== Gestion de la twMediathequeGetComplet qui contient la liste complete des réponses
    ui->twMediathequeGetComplet->verticalHeader()->setResizeMode(QHeaderView::Interactive);
    for(int i = 0; i < ui->twMediathequeGetComplet->columnCount(); i++)
    {
        //La largeur de la colonne
        ui->twMediathequeGetComplet->setColumnWidth(i,100);
        for(int j = 0; j < ui->twMediathequeGetComplet->rowCount(); j++) {
            QTableWidgetItem *tempVide = new QTableWidgetItem("");
            ui->twMediathequeGetComplet->setItem(j,i,tempVide);
        }
    }
    // choix: sur une petite résolution on supprime type et date, sur une grande résolution on
    // affiche tout
    if(width() < 800) {
        ui->twMediathequeGetComplet->setColumnWidth(1,0);
        ui->twMediathequeGetComplet->setColumnWidth(2,0);
    }
    ui->twMediathequeGetComplet->horizontalHeader()->setResizeMode(0,QHeaderView::Stretch);

    m_abuledufile = QSharedPointer<AbulEduFileV1>(new AbulEduFileV1, &QObject::deleteLater);
    m_abuledufile->setObjectName("depuis abuledumediathequeget");
    m_abuleduLOM  = new AbulEduSCOLOMFrV1();

    //Création de l'arborescence de téléchargement et mise en cache des fichiers
    m_dirABE = QDir(abeApp->getAbeHomepath() + "/leterrier/.abebox_cache/" + abeApp->applicationName());
    if(!m_dirABE.exists()) {
        m_dirABE.mkpath(m_dirABE.absolutePath());
    }

    m_dirABB = QDir(abeApp->getAbeHomepath() + "/leterrier/abbbox/");
    if(!m_dirABB.exists()) {
        m_dirABB.mkpath(m_dirABB.absolutePath());
    }

    //Création de l'arborescence de téléchargement et mise en cache des vignettes
    //1. pour les fichiers abe
    m_dirThumbnailsABE = QDir(abeApp->getAbeHomepath() + "/leterrier/.abebox_cache/" + abeApp->applicationName() + "/.thumbnails");
    if(!m_dirThumbnailsABE.exists()) {
        m_dirThumbnailsABE.mkpath(m_dirThumbnailsABE.absolutePath());
    }

    //2. pour les fichiers abb
    m_dirThumbnailsABB = QDir(abeApp->getAbeHomepath() + "/leterrier/abbbox/.thumbnails");
    if(!m_dirThumbnailsABB.exists()) {
        m_dirThumbnailsABB.mkpath(m_dirThumbnailsABB.absolutePath());
    }

    ui->btnCustom1->hide();

    //et par défaut on affiche le widget bienvenue
    ui->stackedWidget->setCurrentWidget(ui->PageRecherche);
    ui->frmMerciNouvelleRecherche->hide();

 #if QT_VERSION >= 0x040700
    ui->leRecherche->setPlaceholderText(trUtf8("Saisissez ici votre recherche ..."));
 #endif

//specificites du mode tablette
#ifndef __ABULEDUTABLETTEV1__MODE__
    ui->tbClose->hide();
#endif

}

AbulEduMediathequeGetV1::~AbulEduMediathequeGetV1()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete ui;
//    m_abuledufile.clear();
}

void AbulEduMediathequeGetV1::abeSetFile(QSharedPointer<AbulEduFileV1> file)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_abuledufile = file;
    m_abuleduLOM = m_abuledufile->abeFileGetLOM();
}

QSharedPointer<AbulEduFileV1> AbulEduMediathequeGetV1::abeGetFile()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_abuledufile;
}

AbulEduSCOLOMFrV1 *AbulEduMediathequeGetV1::abeGetLOM()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_abuleduLOM;
}

void AbulEduMediathequeGetV1::abeStartDownloadLOM()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(ui->stackedWidget->currentWidget() == ui->pageResultats && !ui->twMediathequeGetComplet->selectedItems().isEmpty()) {
        m_docID = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+1).toString();
    }
    else if(ui->stackedWidget->currentWidget() == ui->pageResultatsIcones && !ui->lwMediathequeGetSimple->selectedItems().isEmpty()) {
        m_docID = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+1).toString();
    }

    QList<QVariant> params = QList<QVariant>() << m_docID;

    m_rpc->call("abe.getLOM", params,
                this, SLOT(rpcGetLOM(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    //On affiche le sablier le temps que ça télécharge ...
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::abeSetDebugMode(bool value)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_localDebug = value;
}

bool AbulEduMediathequeGetV1::abeGetDebugMode()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_localDebug;
}

void AbulEduMediathequeGetV1::abeSearchVisible(bool visible)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(visible)
        ui->frmRecherche->show();
    else
        ui->frmRecherche->hide();
}

void AbulEduMediathequeGetV1::abeSearch(QString search)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->leRecherche->setText(search);
    on_btnRecherche_clicked();
}

void AbulEduMediathequeGetV1::abeSetSourceEnum(enumMediathequeOrData mediathequeOrData)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_mediathequeOrData = mediathequeOrData;
    if(mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        m_rpc->setUrl(QUrl("http://data.abuledu.org/Xserver2.php"));
    }
    else {
        m_rpc->setUrl(QUrl("http://mediatheque.abuledu.org/Xserver2.php"));
    }
}

AbulEduMediathequeGetV1::enumMediathequeOrData AbulEduMediathequeGetV1::abeGetSourceEnum() const
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_mediathequeOrData;
}

void AbulEduMediathequeGetV1::abeSetCustomBouton1(QString titre)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(titre == "") {
        ui->btnCustom1->hide();
    }
    else {
        ui->btnCustom1->setText(titre);
        ui->btnCustom1->show();
    }
}

void AbulEduMediathequeGetV1::abeSearchWaitingModeration()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QList<QVariant> params = QList<QVariant>();

    m_rpc->call("abe.getValidationWaitingList", params,
                this, SLOT(rpcGetValidationWaitingList(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    //On affiche le sablier le temps que ça télécharge ...
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::abeSearchMyRessourcesToModify()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QVariantMap tableau;
    tableau["email"] = m_nam->abeGetSSOLogin();

    QList<QVariant> params = QList<QVariant>() << tableau;

    m_rpc->call("abe.getMyRessourcesToModifyList", params,
                this, SLOT(rpcGetMyRessourcesToModifyList(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    //On affiche le sablier le temps que ça télécharge ...
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::abeSetSource(QString mediathequeOrData)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(mediathequeOrData == "data") {
        abeSetSourceEnum(AbulEduMediathequeGetV1::abeData);
        //        m_mediathequeOrData = AbulEduMediathequeGetV1::abeData;
    }
    else {
        abeSetSourceEnum(AbulEduMediathequeGetV1::abeMediatheque);
        //        m_mediathequeOrData = AbulEduMediathequeGetV1::abeMediatheque;
    }
}

void AbulEduMediathequeGetV1::abeSetDefaultView(AbulEduMediathequeGetV1::enumMediathequeView mediathequeView)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(mediathequeView == abeMediathequeList) {
        ui->tbVueListe->click();
    }
    else {
        ui->tbVueIcones->click();
    }
}

void AbulEduMediathequeGetV1::on_btnRecherche_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->leRecherche->text().trimmed() != "") {
        ui->btnImage->setIconeNormale("");
        //en SQL * c'est %
        if(ui->leRecherche->text() == "*") {
            ui->leRecherche->setText("%");
        }

        QListWidgetItem *item = new QListWidgetItem(QIcon(":/abuledumediathequegetv1/supprimer"), ui->leRecherche->text().trimmed());
        ui->lwRechercheMotsCles->addItem(item);
        ui->leRecherche->clear();

        slotLanceRecherche();
    }
}

void AbulEduMediathequeGetV1::slotLanceRecherche()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    abeClear(false);
    //Pour les recherches c'est un peu plus complique, il faut passer un tableau
    QVariantMap tableau;

    QString unmot = "";
    for(int i = 0; i < ui->lwRechercheMotsCles->count(); i++) {
        if(i > 0) {
            unmot += ";";
        }
        unmot += ui->lwRechercheMotsCles->item(i)->text();
    }

    if(unmot == "") {
        return;
    }

    tableau["recherche"]         = unmot;
    tableau["boolcond"]      = "AND";
    tableau["LIST_MATIERES"] = "";
    tableau["LIST_NIVEAUX"]  = "";
    tableau["document_type"]    = "";

    //Quand la recherche est sur les data brutes on ne limite pas le champ a une seule application type
    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
    }
    else {
        if(abeApp->applicationName() != "abuledu-alacarte") {
            tableau["appName"] = abeApp->applicationName();
        }
    }
    tableau["unauteur"]      = "";
    QList<QVariant> params = QList<QVariant>() << tableau;

    m_rpc->call("abe.getSearchList", params,
                this, SLOT(rpcGetSearchList(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    //On affiche le sablier le temps que ça télécharge ...
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::rpcGetSearchList(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    QStringList iconesPasEnCache;
    QStringList iconesFileName;

    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();

    if(v.toMap().value("status").toString() == "200") {
        //Si on est sur une page d'accueil, d'info ou autre on passe sur le résultat
        if(ui->stackedWidget->currentWidget() != ui->pageResultats &&
                ui->stackedWidget->currentWidget() != ui->pageResultatsIcones) {
            ui->stackedWidget->setCurrentWidget(ui->pageResultats);
        }

        int num = 0;
        for(int i = 0; i < v.toMap().count(); i++) {
            //        qDebug() << "On tourne pour " << i;
            QMap<QString,QVariant> monItem = v.toMap().value(QString::number(i)).toMap();
            QString titre = QString::fromUtf8(monItem.value("docTitle").toByteArray().constData());
            //Si le titre est vide on passe au suivant (bug coté serveur)
            //c'est à cause de ça qu'on a la variable num
            if(titre.trimmed().isEmpty()) {
                continue;
            }

            QString Size;
            uint taille = monItem.value("docSize").toUInt();
            // taille en Go
            if (taille > 1073741824) {
                Size = QString(" %1 Go").arg((double) taille / 1073741824, 0, 'f', 1);
            }
            // En Mo
            else if (taille > 1048576) {
                Size = QString(" %1 Mo").arg((double) taille / 1048576, 0, 'f', 1);
            }
            // En Ko
            else if (taille > 1024) {
                Size = QString(" %1 Ko").arg(taille / 1024);
            }
            // En octet
            else {
                Size = QString(" %1 octets").arg(taille);
            }

            //Au passage on fais un test de sauvegarde locale "mise en cache"
            QString docFileName = QString::fromUtf8(monItem.value("docFileName").toByteArray().constData());
            QString docThumbnail = docFileName;
            QFileInfo fi;
            if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
                fi.setFile(m_dirThumbnailsABB.absolutePath() + "/" + docThumbnail.replace(".abb",".png"));
            }
            else {
                fi.setFile(m_dirThumbnailsABE.absolutePath() + "/" + docThumbnail.replace(".abe",".png"));
            }

            if(!fi.exists()) {
                iconesPasEnCache << monItem.value("docId").toString();
                iconesFileName   << fi.fileName();
            }

            //Ensuite l'auteur
            QString auteur = QString::fromUtf8(monItem.value("docAuteur").toByteArray().constData());

            //Le résumé
            QString resume = QString::fromUtf8(monItem.value("docResume").toByteArray().constData());

            //Les mots clés
            QString motscles = QString::fromUtf8(monItem.value("docMotscles").toByteArray().constData());

            //La somme MD5 pour verifier si le fichier local est le bon par rapport au fichier distant
            //ou s'il faut actualiser le cache
            QString md5 = QString::fromUtf8(monItem.value("docMD5").toByteArray().constData());

            //Le type mime
            QString docMimeType = QString::fromUtf8(monItem.value("docMime").toByteArray().constData());

            //Le LOM
            QByteArray docLOM = QByteArray::fromBase64(monItem.value("docLOM").toByteArray());

            ajouteItemList(num, titre, monItem.value("docType").toString(), monItem.value("docDate").toString(), Size,
                           monItem.value("docId").toString(), monItem.value("URI").toString(), fi,
                           auteur, monItem.value("docNote").toInt(), resume, motscles, md5, docFileName, docMimeType, docLOM);
            num++;
        }

        ui->lwRechercheHierarchie->clear();
        if(!v.toMap().value("hierarchie").isNull()) {
            QStringList liste = QString::fromUtf8(v.toMap().value("hierarchie").toByteArray()).split(";");
            for(int i = 0; i < liste.size(); i++) {
                QString value = liste.at(i);
                QListWidgetItem *item = new QListWidgetItem(QIcon(""),value);
                ui->lwRechercheHierarchie->addItem(item);
            }
        }

        if(!v.toMap().value("TA").isNull()) {
            QStringList liste = QString::fromUtf8(v.toMap().value("TA").toByteArray()).split(";");
            for(int i = 0; i < liste.size(); i++) {
                QString value = liste.at(i);
                if(ui->lwRechercheHierarchie->findItems(value,Qt::MatchExactly).size() == 0) {
                    QListWidgetItem *item = new QListWidgetItem(QIcon(""),value);
                    ui->lwRechercheHierarchie->addItem(item);
                }
            }
        }

        if(!v.toMap().value("RS").isNull()) {
            QStringList liste = QString::fromUtf8(v.toMap().value("RS").toByteArray()).split(";");
            for(int i = 0; i < liste.size(); i++) {
                QString value = liste.at(i);
                if(ui->lwRechercheHierarchie->findItems(value,Qt::MatchExactly).size() == 0) {
                    QListWidgetItem *item = new QListWidgetItem(QIcon(""),value);
                    ui->lwRechercheHierarchie->addItem(item);
                }
            }
        }

        if(iconesPasEnCache.size() > 0) {
            QVariantMap tableau;
            tableau["ids"]        = iconesPasEnCache.join(",");
            tableau["fileNames"]  = iconesFileName.join(",");

            if(m_localDebug) {
                qDebug() << "<< on cherche " << tableau;
            }

            QApplication::setOverrideCursor(Qt::WaitCursor);

            QList<QVariant> params = QList<QVariant>() << tableau;
            m_rpc->call("abe.getThumbnails", params,
                        this, SLOT(rpcGetThumbnails(QVariant&, QNetworkReply*)),
                        this, SLOT(rpcFault(int, const QString &)));
        }
    }
    else {
        ui->stackedWidget->setCurrentWidget(ui->pageEchec);
        QString recherche;
        for(int i = 0; i < ui->lwRechercheMotsCles->count(); i++) {
            recherche.append(" + " + ui->lwRechercheMotsCles->item(i)->text());
        }
        ui->leRechercheRappel->setText(recherche);
        ui->btnTelecharger->setEnabled(false);
        ui->btnSignaler->setEnabled(true);
        ui->teCommentairesRappel->clear();
    }
}

void AbulEduMediathequeGetV1::rpcGetFile(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();
    m_rpc->activateProgressBar(false);

    //Soit un fichier temporaire
    //QTemporaryFile *file = new QTemporaryFile(QDir::tempPath() + "/AbulEduMediathequeGetV1-XXXXXXX" + v.toMap().value("ID").toString() + ".abe");


    //Soit un fichier dont le nom est evident pour qu'on puisse ne pas le re-telecharger dans 20 secondes
    //si on veut le réouvrir / rechercher à nouveau
    //QFile file(QDir::tempPath() + "/AbulEduMediathequeGetV1-" + m_docID + ".abe");
    //ou son nom officiel sur la boutique
    QString filename;
    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        filename = (m_dirABB.absolutePath() + "/" + v.toMap().value("name", "AbulEduMediathequeGetV1-" + m_docID ).toString());
    }
    else {
        filename = (m_dirABE.absolutePath() + "/" + v.toMap().value("name", "AbulEduMediathequeGetV1-" + m_docID ).toString());
    }
    QFile file(filename);

    if(file.open(QIODevice::WriteOnly)) {
        QByteArray temp = QByteArray::fromBase64(v.toMap().value("content").toByteArray());
        file.write(temp, temp.size());
        file.close();

        //A noter, c'est une mauvaise idee de passer par un qdatastream: il ajoute quelques octets en entete qui
        //plantent tout !
        //QDataStream out(&file);
        //out << temp;
        //file.close();
        if(m_localDebug) {
            qDebug() << "Fichier sauvegarde dans :" << file.fileName() << "taille " << v.toMap().value("size").toString()
                     << " a comparer avec " << temp.size() << "\n\n"
                     << " taille base64 : " << v.toMap().value("base64size").toString() << " -> " << v.toMap().value("content").toByteArray().size();
        }

        handleOpenFile(filename);
    }
    else {
        //Erreur, impossible de télécharger/écrire le fichier
        emit signalMediathequeFileDownloaded(-1);
    }
}

void AbulEduMediathequeGetV1::handleOpenFile(QString filename) {
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //cas particulier si on est avec alacarte ...
    if(abeApp->applicationName() == "abuledu-alacarte") {
        //On peut essayer de l'ouvrir via le systeme
        QDesktopServices::openUrl(filename);
        emit signalMediathequeFileDownloaded(0);
    }
    else {
        //afaire mise a jour pour gerer les fichiers .abe
        if(filename.endsWith(".abe") || filename.endsWith(".abb")) {
            m_abuledufile->abeFileOpen(filename,"zip");
            m_abuleduLOM= m_abuledufile->abeFileGetLOM();
            emit signalMediathequeFileDownloaded(1);
        }
        else {
            //On peut essayer de l'ouvrir via le systeme
            //(Question, à vérifier, je crois que c'est le cas particulier où on télécharge un .deb un .exe ou autre ?)
            QDesktopServices::openUrl(filename);
            emit signalMediathequeFileDownloaded(0);
        }
    }
}

void AbulEduMediathequeGetV1::rpcGetThumbnails(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();

    for(int i = 0; i < v.toMap().count(); i++) {
        //        qDebug() << "On tourne pour " << i;
        QMap<QString,QVariant> monItem = v.toMap().value(QString::number(i)).toMap();

        QString docFileName = QString::fromUtf8(monItem.value("docFileName").toByteArray().constData());
        QFileInfo fi;
        if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
            fi.setFile(m_dirThumbnailsABB.absolutePath() + "/" + docFileName.replace(".abb",".png"));
        }
        else {
            fi.setFile(m_dirThumbnailsABE.absolutePath() + "/" + docFileName.replace(".abe",".png"));
        }

        QFile file(fi.absoluteFilePath());
        if(file.open(QIODevice::WriteOnly)) {
            QByteArray temp = QByteArray::fromBase64(monItem.value("docThumbnail").toByteArray());
            file.write(temp, temp.size());
            file.close();

            //Il faudrait rafraichir le ui.lwMediathequeGetSimple
            QList <QListWidgetItem*> listeItems = ui->lwMediathequeGetSimple->findItems(fi.fileName(),Qt::MatchExactly);
            if(!listeItems.isEmpty()) {
                lwMediathequeGetSimpleSetIcon(listeItems.at(0), fi);
            }
        }
    }

}

void AbulEduMediathequeGetV1::rpcSendAlertNoRessource(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();
    ui->stackedWidget->setCurrentWidget(ui->PageRecherche);
    ui->frmMerciNouvelleRecherche->show();
    //Et on cache dans 5 secondes
    QTimer::singleShot(5000,ui->frmMerciNouvelleRecherche,SLOT(hide()));
}

void AbulEduMediathequeGetV1::rpcGetValidationWaitingList(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Pour l'instant c'est strictement la même chose :)
    abeClear(true);
    rpcGetSearchList(v,reply);
}

void AbulEduMediathequeGetV1::rpcGetMyRessourcesToModifyList(QVariant &v, QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //qDebug() << v ;

    //Pour l'instant c'est strictement la même chose :)
    abeClear(true);
    rpcGetSearchList(v,reply);
}

void AbulEduMediathequeGetV1::rpcGetLOM(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    //Le GetLOM
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();

    QMap<QString,QVariant> monItem = v.toMap();
    m_abuleduLOM->abeLOMsetXML(QByteArray::fromBase64(monItem.value("docLOM").toByteArray()));
    emit signalMediathequeLomDownloaded();
}

void AbulEduMediathequeGetV1::rpcVote(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    //Le GetLOM
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }
    QApplication::restoreOverrideCursor();
}

void AbulEduMediathequeGetV1::slotDownloadFinished(QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Si ce n'est pas pour moi je droppe
    if(reply->objectName() != "AbulEduMediathequeGetDownload") {
        return;
    }

    disconnect(m_nam, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotDownloadFinished(QNetworkReply*)));
    /** @todo */
    if(m_localDebug) {
        qDebug() << "A implementer, on sauvegarde le fichier sur le disque dur ...";
    }
}

void AbulEduMediathequeGetV1::rpcFault(int code, const QString &errorMessage)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "===================================";
        qDebug() << code << errorMessage;
        qDebug() << "===================================";
    }
    m_rpc->activateProgressBar(false);
    QApplication::restoreOverrideCursor();

    //Tentative de réouverture d'une session SSO en cas de timeout
    if(code == -32300) {

//2013.01.16 : decision pour la tablette
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        m_nam->abeSSOLogin();
#endif
    }
}

void AbulEduMediathequeGetV1::on_twMediathequeGetComplet_itemSelectionChanged()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->twMediathequeGetComplet->selectedItems().count() > 0) {
        QMap<int,QVariant> values;
        for(int i = 1; i <= 11; i++) {
            values.insert(Qt::UserRole+i,ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+i));
        }
        communItemSelectionChange(values);
    }
}

void AbulEduMediathequeGetV1::on_btnTelecharger_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QString md5;
    QString docFileName;

    if(ui->stackedWidget->currentWidget() == ui->pageResultats && !ui->twMediathequeGetComplet->selectedItems().isEmpty()) {
        m_docID = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+1).toString();
        md5 = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+8).toString();
        docFileName = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+9).toString();
    }
    else if(ui->stackedWidget->currentWidget() == ui->pageResultatsIcones && !ui->lwMediathequeGetSimple->selectedItems().isEmpty()) {
        m_docID = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+1).toString();
        md5 = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+8).toString();
        docFileName = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+9).toString();
    }
    else {
        return;
    }

    if(m_localDebug) {
        qDebug() << "AbulEduMediathequeGetV1::on_btnTelecharger_clicked : on telecharge " << m_docID << " " << docFileName;
    }

    //On vérifie si on n'a pas déjà ce fichier en cache
    QFileInfo fi;
    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        fi.setFile(m_dirABB.absolutePath() + "/" + docFileName);
    }
    else {
        fi.setFile(m_dirABE.absolutePath() + "/" + docFileName);
    }

    if(m_localDebug) {
        qDebug() << "on demande le document " << docFileName << " dont la somme MD5 est " << md5;
        qDebug() << "sur le disque il devrait etre ici " << fi.absoluteFilePath();
    }
    //Si le fichier existe, on calcule sa somme md5 et on compare
    if(fi.exists()) {
        QString md5verif = m_abuledufile->abeFileGetMD5(fi.absoluteFilePath());
        //La somme md5 est identique, on ne télécharge pas
        if(md5verif == md5) {
            if(m_localDebug) {
                qDebug() << "Ce fichier est déjà la";
            }
            handleOpenFile(fi.absoluteFilePath());
            return;
        }
        else {
            if(m_localDebug) {
                qDebug() << "Ce fichier est déjà la mais la somme md5 differe : " << md5verif;
            }
        }
    }

    //On garde dans un coin la docID pour recomposer le nom du fichier telecharge
    QList<QVariant> params = QList<QVariant>() << m_docID;

    m_rpc->activateProgressBar(true);

    m_rpc->call("abe.getFile", params,
                this, SLOT(rpcGetFile(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));

    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::on_btnCustom1_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    emit signalMediathequeCustomBouton1Clicked();
}

void AbulEduMediathequeGetV1::on_cbAvertirQuandDispo_toggled(bool checked)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->leAdresseMailRappel->setEnabled(checked);
    ui->lblAdresseMail->setEnabled(checked);
}

void AbulEduMediathequeGetV1::on_btnSignaler_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->btnSignaler->setEnabled(false);
    //Pour l'instant on ne gère que pour data
    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        QVariantMap tableau;
        tableau["mot"]               = ui->leRechercheRappel->text().trimmed();
        tableau["commentaires"]      = ui->teCommentairesRappel->toPlainText().trimmed();
        tableau["email"]             = ui->leAdresseMailRappel->text().trimmed();
        QList<QVariant> params = QList<QVariant>() << tableau;

        m_rpc->call("abe.sendAlertNoRessource", params,
                    this, SLOT(rpcSendAlertNoRessource(QVariant&, QNetworkReply*)),
                    this, SLOT(rpcFault(int, const QString &)));
        //On affiche le sablier le temps que ça télécharge ...
        QApplication::setOverrideCursor(Qt::WaitCursor);
        QApplication::processEvents();
    }
}

void AbulEduMediathequeGetV1::ajouteItemList(int num, QString titre, QString type, QString date, QString size, QString docId,
                                           QString URI, QFileInfo ficInfo, QString contributeur, int note,
                                           QString resume, QString motscles, QString md5, QString docFileName, QString docMimeType,
                                           QByteArray docLOM)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << num << titre << type << note;
    }

    // =================================================== la liste d'items
    QTableWidgetItem *item1 = new QTableWidgetItem(titre);
    QTableWidgetItem *item2 = new QTableWidgetItem(type);
    QTableWidgetItem *item3 = new QTableWidgetItem(date);
    QTableWidgetItem *item4 = new QTableWidgetItem(size);

    item1->setData(Qt::UserRole+1,docId);
    item1->setData(Qt::UserRole+2,URI);
    item1->setData(Qt::UserRole+3,ficInfo.absoluteFilePath());
    item1->setData(Qt::UserRole+4,contributeur);
    item1->setData(Qt::UserRole+5,note);
    item1->setData(Qt::UserRole+6,resume);
    item1->setData(Qt::UserRole+7,motscles);
    item1->setData(Qt::UserRole+8,md5);
    item1->setData(Qt::UserRole+9,docFileName);
    item1->setData(Qt::UserRole+10,docMimeType);
    item1->setData(Qt::UserRole+11,docLOM);

    ui->twMediathequeGetComplet->setRowCount(num);
    ui->twMediathequeGetComplet->insertRow(num);
    ui->twMediathequeGetComplet->setItem(num, 0, item1);
    ui->twMediathequeGetComplet->setItem(num, 1, item2);
    ui->twMediathequeGetComplet->setItem(num, 2, item3);
    ui->twMediathequeGetComplet->setItem(num, 3, item4);


    // =================================================== la liste en icones
    //Pour tests on affiche toutes les vignettes d'un répertoire
    QListWidgetItem *itemIcone = new QListWidgetItem(ficInfo.fileName());

    itemIcone->setData(Qt::UserRole+1,docId);
    itemIcone->setData(Qt::UserRole+2,URI);
    itemIcone->setData(Qt::UserRole+3,ficInfo.absoluteFilePath());
    itemIcone->setData(Qt::UserRole+4,contributeur);
    itemIcone->setData(Qt::UserRole+5,note);
    itemIcone->setData(Qt::UserRole+6,resume);
    itemIcone->setData(Qt::UserRole+7,motscles);
    itemIcone->setData(Qt::UserRole+8,md5);
    itemIcone->setData(Qt::UserRole+9,docFileName);
    itemIcone->setData(Qt::UserRole+10,docMimeType);
    itemIcone->setData(Qt::UserRole+11,docLOM);

    //On cache le texte ... c'est rapide et le moins pire que j'ai trouvé
    QFont invisible(abeApp->font().family(),1);
    itemIcone->setFont(invisible);
    QFileInfo ficone;

    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        ficone.setFile(m_dirThumbnailsABB.absolutePath() + "/" + ficInfo.completeBaseName() + ".png");
    }
    else {
        ficone.setFile(m_dirThumbnailsABE.absolutePath() + "/" + ficInfo.completeBaseName() + ".png");
    }
    lwMediathequeGetSimpleSetIcon(itemIcone, ficone);
    ui->lwMediathequeGetSimple->insertItem(num,itemIcone);
}

void AbulEduMediathequeGetV1::communItemSelectionChange(QMap<int,QVariant> val)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "  ************" << val.value(Qt::UserRole+3);
    }
    //On a stocké dans le UserRole+3 le thumbnail lors du téléchargement de la liste

    ui->leID->setText(val.value(Qt::UserRole+1).toString());
    QPixmap p(val.value(Qt::UserRole+3).toString());
    ui->btnImage->setIcon(p);
    ui->leContrib->setText(val.value(Qt::UserRole+4).toString().left(20));
    ui->pbAppreciation->setValue(val.value(Qt::UserRole+5).toInt());

    if(m_localDebug) {
        qDebug() << val.value(Qt::UserRole+5);
    }

    ui->teResume->setPlainText(val.value(Qt::UserRole+6).toString());
    ui->teMotsCles->setText(val.value(Qt::UserRole+7).toString());
    QString docFileName = val.value(Qt::UserRole+9).toString();

    QFileInfo fi;
    if(m_mediathequeOrData == AbulEduMediathequeGetV1::abeData) {
        fi.setFile(m_dirABB.absolutePath() + "/" + docFileName);
    }
    else {
        fi.setFile(m_dirABE.absolutePath() + "/" + docFileName);
    }

    if(fi.exists()) {
        m_abuledufile->abeFileSetFilename(fi.absoluteFilePath());
        m_abuleduLOM = m_abuledufile->abeFileGetLOM();
    }
    else {
        m_abuleduLOM->abeLOMsetXML(val.value(Qt::UserRole+11).toByteArray());
    }

    m_docID = val.value(Qt::UserRole+1).toString();

    ui->btnTelecharger->setEnabled(true);

    //Si on est authentifies on peut voter ...
    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        ui->btnAppreciation->setEnabled(true);
        ui->sbAppreciation->setEnabled(true);
    }
    else {
        ui->btnAppreciation->setEnabled(false);
        ui->sbAppreciation->setEnabled(false);
    }
}

void AbulEduMediathequeGetV1::lwMediathequeGetSimpleSetIcon(QListWidgetItem *itemIcone, QFileInfo ficone)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QIcon icone;
    //Si l'icone n'existe pas on ...
    if(ficone.exists()) {
        icone = QIcon(ficone.absoluteFilePath());//pour la mettre  à coté de l'item
    }
    else {
        icone = QIcon(":/abuledufilev1/fond-vide");//pour la mettre  à coté de l'item
    }
    itemIcone->setIcon(icone); // ajout de la petite icone sur l'item
}

void AbulEduMediathequeGetV1::on_lwMediathequeGetSimple_itemSelectionChanged()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->lwMediathequeGetSimple->selectedItems().count() > 0) {
        QMap<int,QVariant> values;
        for(int i = 1; i <= 11; i++) {
            values.insert(Qt::UserRole+i,ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+i));
        }
        communItemSelectionChange(values);
    }
}

void AbulEduMediathequeGetV1::on_tbVueIcones_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->tbVueListe->setChecked(false);
    ui->tbVueIcones->setChecked(true);
    ui->stackedWidget->setCurrentWidget(ui->pageResultatsIcones);
}

void AbulEduMediathequeGetV1::on_tbVueListe_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->tbVueListe->setChecked(true);
    ui->tbVueIcones->setChecked(false);
    ui->stackedWidget->setCurrentWidget(ui->pageResultats);
}

void AbulEduMediathequeGetV1::abeCheckDoublon(QString uri)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QList<QVariant> params = QList<QVariant>() << uri;
    m_rpc->call("abe.checkDoublon", params,
                this, SLOT(rpcCheckDoublon(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

QString AbulEduMediathequeGetV1::abeGetCurrentSelectedItemURI()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->stackedWidget->currentWidget() == ui->pageResultats && !ui->twMediathequeGetComplet->selectedItems().isEmpty()) {
        return ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+2).toString();
    }
    else if(ui->stackedWidget->currentWidget() == ui->pageResultatsIcones && !ui->lwMediathequeGetSimple->selectedItems().isEmpty()) {
        return ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+2).toString();
    }
    else {
        return "";
    }
}

void AbulEduMediathequeGetV1::abeRemoveCurrentSelectedItem()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(ui->stackedWidget->currentWidget() == ui->pageResultats && !ui->twMediathequeGetComplet->selectedItems().isEmpty()) {
        ui->twMediathequeGetComplet->removeRow(ui->twMediathequeGetComplet->selectedItems().at(0)->row());
    }
    else if(ui->stackedWidget->currentWidget() == ui->pageResultatsIcones && !ui->lwMediathequeGetSimple->selectedItems().isEmpty()) {
        ui->lwMediathequeGetSimple->removeItemWidget(ui->lwMediathequeGetSimple->selectedItems().at(0));
    }
}

void AbulEduMediathequeGetV1::abeHideBoutonTelecharger()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->btnTelecharger->hide();
}

void AbulEduMediathequeGetV1::abeHideInfoPanel(bool hide)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(hide) {
        ui->frmInfoPanel->hide();
    }
    else {
        ui->frmInfoPanel->show();
    }
}

void AbulEduMediathequeGetV1::abeSetCustomBouton1Download(bool enable)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(enable) {
        connect(ui->btnCustom1,SIGNAL(clicked()),ui->btnTelecharger,SLOT(click()),Qt::UniqueConnection);
    }
    else {
        disconnect(ui->btnCustom1,SIGNAL(clicked()),ui->btnTelecharger,SLOT(click()));
    }
}

void AbulEduMediathequeGetV1::abeCustomBouton1SetDownload(bool enable)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    this->abeSetCustomBouton1Download(enable);
}

void AbulEduMediathequeGetV1::abeStartDownload()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->btnTelecharger->click();
}

void AbulEduMediathequeGetV1::abeClear(bool clearMotsCles)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->twMediathequeGetComplet->setSortingEnabled(false);
    ui->twMediathequeGetComplet->clearContents();
    ui->twMediathequeGetComplet->setRowCount(0);
    ui->lwMediathequeGetSimple->clear();

    ui->leID->clear();
    ui->teMotsCles->clear();
    ui->teResume->clear();
    ui->leContrib->clear();
    ui->pbAppreciation->setValue(0);
    ui->btnImage->setIconeNormale("");
    if(clearMotsCles) {
        ui->lwRechercheMotsCles->clear();
    }
}


void AbulEduMediathequeGetV1::rpcCheckDoublon(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_localDebug > 1) {
        qDebug() << "=============================";
        qDebug() << v;
        qDebug() << "=============================";
    }

    QApplication::restoreOverrideCursor();
    QString monItem = v.toMap().value("docId").toString();
    if(monItem != "-1") {
        if(m_localDebug) {
            qDebug() << monItem;
        }
        QMessageBox::warning(this, trUtf8("abuledu-data"),
                             trUtf8("Attention il semblerait que cette ressource ait déjà été indexée !\nVérifiez à l'aide de la recherche..."),
                             QMessageBox::Ok,
                             QMessageBox::Ok);
    }
}

void AbulEduMediathequeGetV1::on_lwRechercheMotsCles_itemClicked(QListWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    delete item;
    slotLanceRecherche();
}

void AbulEduMediathequeGetV1::on_lwRechercheHierarchie_itemClicked(QListWidgetItem *item)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    ui->lwRechercheMotsCles->clear();
    ui->leRecherche->setText(item->text());
    on_btnRecherche_clicked();
}

void AbulEduMediathequeGetV1::on_lwMediathequeGetSimple_itemDoubleClicked(QListWidgetItem *item)
{
    Q_UNUSED(item);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnTelecharger_clicked();
}

void AbulEduMediathequeGetV1::on_twMediathequeGetComplet_itemDoubleClicked(QTableWidgetItem *item)
{
    Q_UNUSED(item);
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    on_btnTelecharger_clicked();
}

void AbulEduMediathequeGetV1::on_btnImage_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //On affiche l'image en grand ... la visionneuse d'icham ?
    AbulEduVisionneuseImageV1 *visio = new AbulEduVisionneuseImageV1(this);

    QString uri;
    QString mime;
    if(ui->stackedWidget->currentWidget() == ui->pageResultats && !ui->twMediathequeGetComplet->selectedItems().isEmpty()) {
        uri  = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+9).toString();
        mime = ui->twMediathequeGetComplet->selectedItems().at(0)->data(Qt::UserRole+10).toString();
    }
    else if(ui->stackedWidget->currentWidget() == ui->pageResultatsIcones && !ui->lwMediathequeGetSimple->selectedItems().isEmpty()) {
        uri  = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+9).toString();
        mime = ui->lwMediathequeGetSimple->selectedItems().at(0)->data(Qt::UserRole+10).toString();
    }

    if(mime == "image/jpeg") {
//        qDebug() << uri << " et " << mime;

        visio->ouvrirFicher(QUrl("http://data-cache.abuledu.org/512/" + uri.replace(".abb",".jpg")));
//        visio->setWindowModality(Qt::ApplicationModal);
#ifdef __ABULEDUTABLETTEV1__MODE__
        visio->showFullScreen();
#else
        visio->show();
#endif
    }
}

void AbulEduMediathequeGetV1::on_btnAppreciation_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QVariantMap tableau;
    tableau["id"]    = m_docID;
    tableau["vote"]  = QString::number(ui->sbAppreciation->value());
    tableau["login"] = abeApp->getAbeIdentite()->abeGetLogin();

    QList<QVariant> params = QList<QVariant>() << tableau;

    m_rpc->call("abe.vote", params,
                this, SLOT(rpcVote(QVariant&, QNetworkReply*)),
                this, SLOT(rpcFault(int, const QString &)));
    //On affiche le sablier le temps que ça télécharge ...
    QApplication::setOverrideCursor(Qt::WaitCursor);
    QApplication::processEvents();
}

void AbulEduMediathequeGetV1::on_tbClose_clicked()
{
    //On fait juste un hide, ptet qu'on a affaire à une appli qui n'a qu'un seul objet
    //et qui l'affiche plusieurs fois
    hide();
}

void AbulEduMediathequeGetV1::on_leRecherche_editingFinished()
{
    ui->btnRecherche->setFocus();
}
