/** Classe AbulEduVCardComboBox
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/Format_LOM_LOMFR_ScoLOMFR
  * @see http://www.lom-fr.fr/lomfr-unt/unisciel/modeles-de-fiche/LOMFRTemplate-Unisciel-v1.0-complet.xml/view
  *
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2012 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */

#include "abuleduvcardcombobox.h"

AbulEduVCardComboBox::AbulEduVCardComboBox(QWidget *parent) :
    QComboBox(parent)
{
    setEditable(true);
    connect(this, SIGNAL(editTextChanged(QString)),this, SLOT(abeEditText(QString)), Qt::UniqueConnection);
    m_completer = new QCompleter();
    m_initialCompleter = new QCompleter();
    m_validator = new QRegExpValidator(this);
    m_nam = abeApp->getAbeNetworkAccessManager();
    m_delayRequest = new QTimer();
    connect(m_delayRequest, SIGNAL(timeout()), this, SLOT(delayReached()), Qt::UniqueConnection);
    m_isSearchInProgress = false;
    m_isCompleterActivated = false;

    m_rpc = new MaiaXmlRpcClient(QUrl("https://webadminxml.abuledu.net/login.php"), this);

    m_localDebug = DEBUG_ABULEDUVCARDCOMBOBOX;
}

vCard AbulEduVCardComboBox::abeGetVCard(QString name)
{
    if(m_localDebug) {
        qDebug() << "****************************************************************************";
        qDebug()<<"AbulEduVCardComboBox::abeGetVCard("<<name<<")";
    }
    for (int i=0;i<m_vCardList.count();i++)
    {
        vCard v = m_vCardList.at(i);
        qDebug()<<"vCard cherchee avec "<<v;
        if (v.getFullName() == name)
        {
            if(m_localDebug) {
                qDebug()<<"vCard trouvee avec "<<name;
                qDebug()<<m_vCardList.at(i);
            }
            return m_vCardList.at(i);
        }
    }
    if(m_localDebug) {
        qDebug()<<"vCard PAS trouvee avec "<<name;
        qDebug() << "****************************************************************************";
    }
    return vCard();
}

vCard AbulEduVCardComboBox::abeGetCurrentVCard()
{
    if(m_localDebug) {
        qDebug() << m_vCardList.at(0);
    }
    if(m_vCardList.isEmpty())
        return vCard();

    return m_vCardList.last();
}

bool AbulEduVCardComboBox::abeAddVCard(vCard card)
{
    m_vCardList << card;
    return true;
}

void AbulEduVCardComboBox::abeEditText(const QString &entity)
{
    m_entity = entity;

    if (m_localDebug) {
        qDebug()<<"AbulEduVCardComboBox::abeEditText("<<m_entity<<")";
    }

    //On ne lance pas de recherche s'il y en a déjà une en cours
    if ((m_entity.size() > 2) && (m_isSearchInProgress == false))
    {
        if (m_localDebug) {
            qDebug() << "par ici";
        }
        //On remet le timer à zéro -> permet de faire une recherche un peu moins rapide
        m_delayRequest->stop();
        m_delayRequest->start(1250);
    }
    else {
        if (m_localDebug) {
            qDebug() << "par LA : " << m_isSearchInProgress << " et " << m_entity.size();
        }
        m_completer = m_initialCompleter;
    }
}

void AbulEduVCardComboBox::rpcGetSearchVCard(QVariant &v, QNetworkReply *reply)
{
    Q_UNUSED(reply);
    if (m_localDebug) {
        qDebug() << "AbulEduVCardComboBox::rpcGetSearchVCard : " << v.toMap();
    }
    QApplication::restoreOverrideCursor();
    //Parcours des vcards
    m_vCardList.clear();
    QStringList liste;
    for(int i = 0; i < v.toMap().count(); i++) {
        vCard vcard = vCard::fromByteArray(v.toMap().value(QString::number(i)).toMap().value("vcard").toByteArray()).first();

        if (m_localDebug) {
            qDebug() << "AbulEduVCardComboBox::rpcGetSearchVCard : " << vcard;
        }

        m_vCardList.append(vcard);
        //        liste << v.toMap().value(QString::number(i)).toMap().value("login").toString();
        if (m_localDebug) qDebug() << vcard.toByteArray();
        if(vcard.getFullName().trimmed() != "") {
            if (m_localDebug) {
                qDebug() << " --> ok on l'ajoute !";
            }
            liste << vcard.getFullName();
        }
    }

    m_completer = new QCompleter(liste,this);
    m_completer.data()->setCompletionRole(Qt::EditRole);
    m_completer.data()->setCaseSensitivity(Qt::CaseInsensitive);

    //    blockSignals(true);
    setCompleter(m_completer.data());
    //    blockSignals(false);

    m_completer.data()->complete();
    connect(m_completer, SIGNAL(activated(QString)), this, SLOT(completerActivated(QString)), Qt::UniqueConnection);
    m_isSearchInProgress=false;
}

void AbulEduVCardComboBox::completerActivated(QString value)
{
    if (m_localDebug) {
        qDebug() << "AbulEduVCardComboBox::completerActivated : " << value;
    }
    m_isCompleterActivated = true;
}

void AbulEduVCardComboBox::delayReached()
{
    if (m_localDebug) {
        qDebug() << "AbulEduVCardComboBox::delayReached";
    }
    m_delayRequest->stop();

    //Si on a cliqué sur le completer et on le remet a false pour le prochain coup
    //si l'envie nous venait de chercher une autre vcard
    if(m_isCompleterActivated == true) {
        m_isCompleterActivated = false;
        return;
    }
    //Et sinon , c'est qu'on a pas cliqué sur le completer -> on lance la recherche reseau

    //uniquement si on est authentifié
    if(m_nam->abeSSOAuthenticationStatus() == 1) {
        //Pour les recherches c'est un peu plus complique, il faut passer un tableau
        QList<QVariant> params = QList<QVariant>() << m_entity;
        m_isSearchInProgress = true;
        m_rpc->call("abuledu.sso.getVcards", params,
                    this, SLOT(rpcGetSearchVCard(QVariant&, QNetworkReply*)),
                    this, SLOT(rpcFault(int, const QString &)));
        //On affiche le sablier le temps que ça télécharge ...
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    }
    else {
    //        m_nam->abeSSOLogin();
    }
}

void AbulEduVCardComboBox::rpcFault(int code, const QString &errorMessage)
{
    if (m_localDebug)
    {
        qDebug() << "===================================";
        qDebug() << "AbulEduVCardComboBox::rpcFault : " << code << errorMessage;
        qDebug() << "===================================";
    }
    //Tentative de réouverture d'une session SSO en cas de timeout
    //    if(code == -32300) {
    //        if (m_localDebug) qDebug() << "AbulEduVCardComboBox::rpcFault : abessoauthstatus: " << m_nam->abeSSOAuthenticationStatus();
    //        m_nam->abeSSOLogin();
    //    }
    QApplication::restoreOverrideCursor();
    m_isSearchInProgress=false;
}
