/** Classe AbulEduVisionneuseImageV1
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012 Icham Sirat <icham.sirat@ryxeo.com>
  * @author 2012 Éric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */

#ifndef ABULEDUVISIONNEUSEIMAGE_H
#define ABULEDUVISIONNEUSEIMAGE_H

#ifndef DEBUG_ABULEDUVISIONNEUSEIMAGE
#define DEBUG_ABULEDUVISIONNEUSEIMAGE 0
#endif

#include <QMainWindow>
#include <QUrl>
#include <QPixmap>
#include <QLabel>
#include <QPrinter>
#include <QScrollArea>
#include <QScrollBar>
#include <QFileDialog>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include "abuledunetworkaccessmanagerv1.h"
#include "abuleduapplicationv1.h"

class QAction;
class QLabel;
class QMenu;
class QScrollArea;
class QScrollBar;

class AbulEduVisionneuseImageV1 : public QMainWindow
{
    Q_OBJECT

public:
    AbulEduVisionneuseImageV1(QWidget *parent = 0);
    ~AbulEduVisionneuseImageV1();

    /** Ouvre une image locale */
    void ouvrirFicher(QString chemin);

    /** Téléchargement d'une image */
    void ouvrirFicher(QUrl uri);

    /** Réglage en millisecondes du temps maximum d'affichage de la visionneuse
      * @note Valeur par défaut : 5000 */
    void fixeTempsAffichageMax(int ms);

public slots:
    void ajusterFenetreAuMieux();

private slots:
    void ouvrir();
    void imprimer();
    void zoomIn();
    void zoomOut();
    void tailleNormale();
    void ajusterFenetre();
    void aPropos();
    void slotUrlDownloaded(QNetworkReply* reply);

private:
    void creerActions();
    void creerMenus();
    void majActions();
    void scaleImage(double facteur);
    void adjustScrollBar(QScrollBar *scrollBar, double facteur);
    void mouseReleaseEvent(QMouseEvent *event);

    QLabel *imageLabel;
    QScrollArea *scrollArea;
    double scaleFactor;
    QWidget *m_parent;
    bool m_localDebug;

    QAction *a_ouvrir;
    QAction *a_imprimer;
    QAction *a_quitter;
    QAction *a_zoomIn;
    QAction *a_zoomOut;
    QAction *a_tailleNormal;
    QAction *a_ajusterFenetre;
    QAction *a_aPropos;
    QAction *a_aProposQt;

    QMenu *menu_fichier;
    QMenu *menu_vue;
    QMenu *menu_aide;

    AbulEduNetworkAccessManagerV1 *m_nam;

    /** QTimer utilisé par défaut pour éviter le double-clic. On peut grâce à lui définir une durée d'affichage maximum. */
    QTimer* m_timer;

    QPrinter m_printer;
};

#endif // ABULEDUVISIONNEUSEIMAGE_H
