/**
  * Bouton plat et transparent, developpé initialement pour le projet
  * Imageo (le-terrier d'AbulEdu) et repris dans les classes de base
  * d'AbulÉdu (merci Thomas !)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-imageo
  * @author 2011 Thomas Grosjean-Pasut <thomas.grosjeanpasut@ryxeo.com>
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  *
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuleduflatboutonv1.h"
#include <QMouseEvent>
#include <QDebug>

AbulEduFlatBoutonV1::AbulEduFlatBoutonV1(QWidget* parent) :
    QPushButton(parent)
{
    m_localDebug = false;
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS   = new QMap<QString, QString>();
    m_icones        = new QMap<QString, QString>();
    m_paddingLeft   = 0;
    m_paddingRight  = 0;
    m_paddingTop    = 0;
    m_paddingBottom = 0;
    m_timer         = new QTimer();

#ifdef __ABULEDUTABLETTEV1__MODE__
    setDisableTimeout(500);
//    setFlat(false);
#endif

    setFocusPolicy(Qt::NoFocus);

    // Couleurs par défaut
    setCouleursTexte(QColor(Qt::black), QColor(Qt::blue), QColor(Qt::blue), QColor(Qt::gray));
    setCouleursFond(QColor(Qt::transparent), QColor(Qt::transparent), QColor(Qt::transparent), QColor(Qt::transparent));

    // Pas d'icones par défaut
    setIcones("","","");

    setDisableTimeout(0);
    connect(this, SIGNAL(clicked()), this, SLOT(slotClicked()), Qt::UniqueConnection);

    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (2)";
}

AbulEduFlatBoutonV1::~AbulEduFlatBoutonV1()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    delete m_couleursCSS;
    delete m_icones;
    delete m_timer;

    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (2)";
}


void AbulEduFlatBoutonV1::setCouleursTexte(QColor normale, QColor survol, QColor pressed, QColor disabled)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(normale.isValid())
    {
        setCouleurTexteNormale(normale);
    }
    if(survol.isValid())
    {
        setCouleurTexteSurvol(survol);
    }
    if(pressed.isValid())
    {
        setCouleurTextePressed(pressed);
    }
    if(disabled.isValid())
    {
        setCouleurTexteDisabled(disabled);
    }
    updateCSS();

    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (2)";
}

void AbulEduFlatBoutonV1::setCouleurTexteNormale(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurTexteNormale", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurTexteNormale()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurTexteNormale"));
}

void AbulEduFlatBoutonV1::setCouleurTexteSurvol(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurTexteSurvol", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurTexteSurvol()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurTexteSurvol"));
}

void AbulEduFlatBoutonV1::setCouleurTextePressed(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurTextePressed", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurTextePressed()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurTextePressed"));
}

void AbulEduFlatBoutonV1::setCouleurTexteDisabled(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurTexteDisabled", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurTexteDisabled()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurTexteDisabled"));
}

void AbulEduFlatBoutonV1::setCouleursFond(QColor normale, QColor survol, QColor pressed, QColor disabled)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(normale.isValid())
    {
        setCouleurFondNormale(normale);
    }
    if(survol.isValid())
    {
        setCouleurFondSurvol(survol);
    }
    if(pressed.isValid())
    {
        setCouleurFondPressed(pressed);
    }
    if(disabled.isValid())
    {
        setCouleurFondDisabled(disabled);
    }
    updateCSS();
}

void AbulEduFlatBoutonV1::setCouleurFondNormale(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurFondNormale", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurFondNormale()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurFondNormale"));
}

void AbulEduFlatBoutonV1::setCouleurFondSurvol(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurFondSurvol", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurFondSurvol()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurFondSurvol"));
}

void AbulEduFlatBoutonV1::setCouleurFondPressed(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurFondPressed", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurFondPressed()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurFondPressed"));
}

void AbulEduFlatBoutonV1::setCouleurFondDisabled(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_couleursCSS->insert("couleurFondDisabled", toRGBA(couleur));
    updateCSS();
}

QColor AbulEduFlatBoutonV1::getCouleurFondDisabled()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return fromRGBA(m_couleursCSS->value("couleurFondDisabled"));
}

void AbulEduFlatBoutonV1::setIcones(QString normale, QString survol, QString pressed, QString disabled)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(QFile(normale).exists())
        m_icones->insert("iconeNormale", normale);
    else
        m_icones->insert("iconeNormale", "");

    if(QFile(survol).exists())
        m_icones->insert("iconeSurvol", survol);
    else
        m_icones->insert("iconeSurvol", "");

    if(QFile(pressed).exists())
        m_icones->insert("iconePressed", pressed);
    else
        m_icones->insert("iconePressed", "");

    if(QFile(disabled).exists())
        m_icones->insert("iconeDisabled", disabled);
    else
        m_icones->insert("iconeDisabled", "");

    updateCSS();
}

void AbulEduFlatBoutonV1::setIconeNormale(QString normale)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";
    }

    if(QFile(normale).exists()) {
        m_icones->insert("iconeNormale", normale);
    }
    else {
        if(m_localDebug) {
            qDebug()  << __FUNCTION__ << " : le fichier n'existe pas : " << normale;
        }
        m_icones->insert("iconeNormale", "");
    }
    updateCSS();
}

QString AbulEduFlatBoutonV1::getIconeNormale()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_icones->value("iconeNormale");
}

void AbulEduFlatBoutonV1::setIconeSurvol(QString survol)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(QFile(survol).exists())
        m_icones->insert("iconeSurvol", survol);
    else
        m_icones->insert("iconeSurvol", "");
    updateCSS();
}

QString AbulEduFlatBoutonV1::getIconeSurvol()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_icones->value("iconeSurvol");
}

void AbulEduFlatBoutonV1::setIconePressed(QString pressed)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(QFile(pressed).exists())
        m_icones->insert("iconePressed", pressed);
    else
        m_icones->insert("iconePressed", "");
    updateCSS();
}

QString AbulEduFlatBoutonV1::getIconePressed()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_icones->value("iconePressed");
}

void AbulEduFlatBoutonV1::setIconeDisabled(QString disabled)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    if(QFile(disabled).exists())
        m_icones->insert("iconeDisabled", disabled);
    else
        m_icones->insert("iconeDisabled", "");
    updateCSS();
}

QString AbulEduFlatBoutonV1::getIconeDisabled()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_icones->value("iconeDisabled");
}

QColor AbulEduFlatBoutonV1::fromRGBA(QString couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    QColor c;
    if(couleur.startsWith("rgba(")) {
        couleur.remove("rgba(");
        couleur.remove(")");
        c.setRgb(couleur.split(",").at(0).toInt(),
                 couleur.split(",").at(1).toInt(),
                 couleur.split(",").at(2).toInt(),
                 couleur.split(",").at(3).toInt());
    }
    return c;
}

QString AbulEduFlatBoutonV1::toRGBA(QColor couleur)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    QString s="";
    if(couleur.isValid())
    {
        s.append("rgba(");
        s.append(QString::number(couleur.red()));
        s.append(",");
        s.append(QString::number(couleur.green()));
        s.append(",");
        s.append(QString::number(couleur.blue()));
        s.append(",");
        s.append(QString::number(couleur.alpha()));
        s.append(")");
    }
    else
    {
        s="(0,0,0,0)";
    }
    return s;
}

void AbulEduFlatBoutonV1::updateCSS()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_cssFlatBouton.clear();
//    m_cssFlatBouton.append(QString("QPushButton > *{color:red;}"));
    // Etat normal du bouton------------------------------------------------------------------
    m_cssFlatBouton.append(QString("QPushButton{border: none; color:"+m_couleursCSS->value("couleurTexteNormale")+";"));

    m_cssFlatBouton.append(m_texteAlignement);

    //Ajout d'un déplacement horizontal si nécessaire
    if(m_paddingLeft > 0) {
        m_cssFlatBouton.append(QString("padding-left: %1px").arg(m_paddingLeft)+";");
    }

    if(m_paddingRight > 0) {
        m_cssFlatBouton.append(QString("padding-right: %1px").arg(m_paddingRight)+";");
    }

    if(m_paddingTop > 0) {
        m_cssFlatBouton.append(QString("padding-top: %1px").arg(m_paddingTop)+";");
    }

    if(m_paddingBottom > 0) {
        m_cssFlatBouton.append(QString("padding-bottom: %1px").arg(m_paddingBottom)+";");
    }


    m_cssFlatBouton.append(QString("background-repeat: no-repeat;"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondNormale")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeNormale").size() > 0)
    {
        m_cssFlatBouton.append(QString("border-image:url('"+m_icones->value("iconeNormale")+"');"));
    }
    m_cssFlatBouton.append(QString("image-position: center;}\n"));

    // Etat normal du survolé du bouton-------------------------------------------------------
    m_cssFlatBouton.append(QString("QPushButton:hover{color:"+m_couleursCSS->value("couleurTexteSurvol")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondSurvol")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeSurvol").size() > 0)
    {
        m_cssFlatBouton.append(QString("border-image:url('"+m_icones->value("iconeSurvol")+"');"));
    }
    m_cssFlatBouton.append(QString("image-position: center;}\n"));

    // Etat pressé du bouton----------------------------------------------------------------
    m_cssFlatBouton.append(QString("QPushButton:pressed{color:"+m_couleursCSS->value("couleurTextePressed")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondPressed")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconePressed").size() > 0)
    {
        m_cssFlatBouton.append(QString("border-image:url('"+m_icones->value("iconePressed")+"');"));
    }
    m_cssFlatBouton.append(QString("image-position: center;}\n"));

    // Etat disabled du bouton------------------------------------------------------------------
    m_cssFlatBouton.append(QString("QPushButton:disabled{border: none; color:"+m_couleursCSS->value("couleurTexteDisabled")+";"));
    // Ajout d'une couleur de fond
    m_cssFlatBouton.append(QString("background-color:"+m_couleursCSS->value("couleurFondDisabled")+";"));
    // Ajout d'une icone s'il y en a une
    if(m_icones->value("iconeDisabled").size() > 0)
    {
        m_cssFlatBouton.append(QString("border-image:url('"+m_icones->value("iconeDisabled")+"');"));
    }
    else
    {
        m_cssFlatBouton.append(QString("image:none")+";"); // Plus d'icône
    }
    m_cssFlatBouton.append(QString("image-position: center;}\n"));

    m_cssFlatBouton.append(QString("QToolTip {background: #f5ffbb; text: black; border-color: #000; border-width: 1px; border-style:solid; }\n"));

    if (m_localDebug) qDebug() << "============================" << m_cssFlatBouton;
    setStyleSheet(m_cssFlatBouton);
}

void AbulEduFlatBoutonV1::setCSSFlatBouton(QString css)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";
    m_cssFlatBouton = css; // Attention cette valeur sera écrasée par tout changement dans la feuille de style
    setStyleSheet(m_cssFlatBouton);
}
void AbulEduFlatBoutonV1::setTexteAlignement(Qt::Alignment alignement)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_alignement = alignement;
    m_texteAlignement = QString("text-align: ");

    if(alignement.testFlag(Qt::AlignLeft)) {
        m_texteAlignement += QString("left");
    }
    if(alignement.testFlag(Qt::AlignRight)) {
        m_texteAlignement += QString("right");
    }
    if(alignement.testFlag(Qt::AlignCenter)) {
        m_texteAlignement += QString("center");
    }
    if(alignement.testFlag(Qt::AlignJustify)) {
        m_texteAlignement += QString("justify");
    }
    if(alignement.testFlag(Qt::AlignTop)) {
        m_texteAlignement += QString("top");
    }
    if(alignement.testFlag(Qt::AlignBottom)) {
        m_texteAlignement += QString("bottom");
    }
    m_texteAlignement += QString(";");
    /*
    case Qt::AlignVCenter:
        m_texteAlignement = QString("vertical-align: middle;");
        break;
    }
    */
    updateCSS();
}

Qt::Alignment AbulEduFlatBoutonV1::getTexteAlignement()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_alignement;
}

void AbulEduFlatBoutonV1::setTextePaddingLeft(qreal padding)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_paddingLeft = qFloor(padding);
    updateCSS();
}

qreal AbulEduFlatBoutonV1::getTextePaddingLeft()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_paddingLeft;
}

void AbulEduFlatBoutonV1::setTextePaddingRight(qreal padding)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_paddingRight = qFloor(padding);
    updateCSS();
}

qreal AbulEduFlatBoutonV1::getTextePaddingRight()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_paddingRight;
}

void AbulEduFlatBoutonV1::setTextePadding(qreal left, qreal top, qreal right, qreal bottom)
{
    m_paddingLeft   = qFloor(left);
    m_paddingRight  = qFloor(right);
    m_paddingTop    = qFloor(top);
    m_paddingBottom = qFloor(bottom);
}

void AbulEduFlatBoutonV1::mouseReleaseEvent(QMouseEvent *e)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    QPushButton::mouseReleaseEvent(e);
    if(m_timer->interval() > 0) {
        m_timer->start();
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
#endif
        this->setEnabled(false);
    }
}


void AbulEduFlatBoutonV1::setDisableTimeout(int msec)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_timer->setInterval(msec);
    connect(m_timer,SIGNAL(timeout()),this,SLOT(slotDisableTimeout()), Qt::UniqueConnection);
}

void AbulEduFlatBoutonV1::abeSetDebugMode(bool debug)
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    m_localDebug = debug;
}

bool AbulEduFlatBoutonV1::abeGetDebugMode()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    return m_localDebug;
}

void AbulEduFlatBoutonV1::enterEvent(QEvent *)
{
    emit signalSurvolIn();
}

void AbulEduFlatBoutonV1::leaveEvent(QEvent *)
{
    emit signalSurvolOut();
}

void AbulEduFlatBoutonV1::slotDisableTimeout()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__ << " (1)";

    this->setEnabled(true);
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
    QApplication::restoreOverrideCursor();
#endif
    m_timer->stop();
}

void AbulEduFlatBoutonV1::slotClicked()
{
    if(m_localDebug)
        qDebug()  << __FILE__ <<  __LINE__ << __PRETTY_FUNCTION__;
    emit signalSurvolIn();
}
