/**
  * Bouton plat et transparent, developpé initialement pour le projet
  * Imageo (le-terrier d'AbulEdu) et repris dans les classes de base
  * d'AbulÉdu (merci Thomas !)
  *
  * Utilisation: https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduFlatBoutonV1
  *
  * #AbulEduFlatBoutonV1
  * include(lib/abuleduflatboutonv1/abuleduflatboutonv1.pri)
  *
  * @see https://redmine.ryxeo.com/projects/leterrier-imageo
  * @author 2011 Thomas Grosjean-Pasut <thomas.grosjeanpasut@ryxeo.com>
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  *
  * @see The GNU Public License (GPL)
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  *  for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUFLATBOUTONV1_H
#define ABULEDUFLATBOUTONV1_H

#include <QPushButton>
#include <QGraphicsSceneMouseEvent>
#include <QFile>
#include <QTimer>
#include <QApplication>
#include <QtCore/qmath.h>

class AbulEduFlatBoutonV1 : public QPushButton
{
    Q_OBJECT
    Q_PROPERTY(bool DebugMode READ abeGetDebugMode WRITE abeSetDebugMode USER true)

    Q_PROPERTY(QColor CouleurTexteNormale  READ getCouleurTexteNormale  WRITE setCouleurTexteNormale  USER true)
    Q_PROPERTY(QColor CouleurTexteSurvol   READ getCouleurTexteSurvol   WRITE setCouleurTexteSurvol   USER true)
    Q_PROPERTY(QColor CouleurTextePressed  READ getCouleurTextePressed  WRITE setCouleurTextePressed  USER true)
    Q_PROPERTY(QColor CouleurTexteDisabled READ getCouleurTexteDisabled WRITE setCouleurTexteDisabled USER true)

    Q_PROPERTY(Qt::Alignment TexteAlignement READ getTexteAlignement WRITE setTexteAlignement USER true)

    Q_PROPERTY(qreal TextePaddingLeft READ getTextePaddingLeft WRITE setTextePaddingLeft USER true)

    Q_PROPERTY(QColor CouleurFondNormale  READ getCouleurFondNormale  WRITE setCouleurFondNormale  USER true)
    Q_PROPERTY(QColor CouleurFondSurvol   READ getCouleurFondSurvol   WRITE setCouleurFondSurvol   USER true)
    Q_PROPERTY(QColor CouleurFondPressed  READ getCouleurFondPressed  WRITE setCouleurFondPressed  USER true)
    Q_PROPERTY(QColor CouleurFondDisabled READ getCouleurFondDisabled WRITE setCouleurFondDisabled USER true)

    Q_PROPERTY(QString iconeNormale  READ getIconeNormale  WRITE setIconeNormale  USER true)
    Q_PROPERTY(QString IconeSurvol   READ getIconeSurvol   WRITE setIconeSurvol   USER true)
    Q_PROPERTY(QString IconePressed  READ getIconePressed  WRITE setIconePressed  USER true)
    Q_PROPERTY(QString IconeDisabled READ getIconeDisabled WRITE setIconeDisabled USER true)

public:
    AbulEduFlatBoutonV1(QWidget* parent = 0);
    ~AbulEduFlatBoutonV1();

    /** couleur normale du texte */
    void setCouleurTexteNormale(QColor couleur);
    QColor getCouleurTexteNormale();

    /** couleur du texte lors du survol */
    void setCouleurTexteSurvol(QColor couleur);
    QColor getCouleurTexteSurvol();

    /** couleur du texte lorsqu'on appuie sur le bouton */
    void setCouleurTextePressed(QColor couleur);
    QColor getCouleurTextePressed();

    /** couleur du texte lorsque le bouton est disabled */
    void setCouleurTexteDisabled(QColor couleur);
    QColor getCouleurTexteDisabled();

    /** permet d'affecter les couleurs en un seul appel */
    void setCouleursTexte(QColor normal, QColor survol = QColor(QColor::Invalid), QColor pressed = QColor(QColor::Invalid), QColor disabled = QColor(QColor::Invalid));

    /** alignement du texte */
    void setTexteAlignement(Qt::Alignment alignement = Qt::AlignHCenter);
    Qt::Alignment getTexteAlignement();

    /** écartement à la bordure gauche du texte */
    void setTextePaddingLeft(qreal padding);
    qreal getTextePaddingLeft();

    /** écartement à la bordure droite du texte */
    void setTextePaddingRight(qreal padding);
    qreal getTextePaddingRight();

    void setTextePadding(qreal left, qreal top, qreal right, qreal bottom);


    /** la couleur de fond du bouton lorsque son état est normal */
    void setCouleurFondNormale(QColor couleur);
    QColor getCouleurFondNormale();

    /** la couleur de fond du bouton lors du survol */
    void setCouleurFondSurvol(QColor couleur);
    QColor getCouleurFondSurvol();

    /** la couleur de fond du bouton lors du clic */
    void setCouleurFondPressed(QColor couleur);
    QColor getCouleurFondPressed();

    /** la couleur de fond du bouton quand il est disabled */
    void setCouleurFondDisabled(QColor couleur);
    QColor getCouleurFondDisabled();

    /** permet d'affecter les couleurs de fond en un seul appel */
    void setCouleursFond(QColor normal, QColor survol = QColor(QColor::Invalid), QColor pressed = QColor(QColor::Invalid), QColor disabled = QColor(QColor::Invalid));

    /** l'icone du bouton normal */
    void setIconeNormale(QString normale);
    QString getIconeNormale();

    /** l'icone du bouton lors du survol */
    void setIconeSurvol(QString survol);
    QString getIconeSurvol();

    /** l'icone du bouton cliqué */
    void setIconePressed(QString pressed);
    QString getIconePressed();

    /** l'icone du bouton disabled */
    void setIconeDisabled(QString disabled);
    QString getIconeDisabled();

    /** affectation des icones en un seul appel */
    void setIcones(QString normale, QString survol = "", QString pressed = "", QString disabled = "" );

    /** Permet de passer une feuille de style au AbulEduFlatBouton
      * @warning Attention, tout appel à une autre fonction détruira cette feuille de style !
      * @warning La feuille de style passée doit être valide : il n'y a pas de parsing préalable !
      */
    void setCSSFlatBouton(QString css);
    void setDisableTimeout(int msec);

    /** Active (ou pas) le mode debug */
    void abeSetDebugMode(bool debug);
    bool abeGetDebugMode();

protected:
    void enterEvent(QEvent *);
    void leaveEvent(QEvent *);

private:
    /** Map contenant les couleurs sous forme de QString rgba
      * prêtes à inclure dans une feuille de style : "(int r, int g, int b, int a)"
      */
    QMap<QString , QString> *m_couleursCSS;

    /** QMap contenant les chemins vers les fichiers d'image-icone */
    QMap<QString, QString> *m_icones;

    /** Contient la feuille de style des AbulEduFlatBoutonV1 */
    QString m_cssFlatBouton;

    /** Contient l'espacement entre la bordure et le texte (padding) à gauche */
    int m_paddingLeft;

    /** Contient l'espacement entre la bordure et le texte (padding) à gauche */
    int m_paddingRight;

    int m_paddingTop;
    int m_paddingBottom;

    /** L'alignement du texte (css et Qt::Alignment) */
    QString m_texteAlignement;
    Qt::Alignment m_alignement;

    bool m_changeIcon;
    /** Convertit une QColor en QString "(int r, int g, int b, int a)" */
    QString toRGBA(QColor couleur);

    /** Convertit une QString "(int r, int g, int b, int a)" en QColor */
    QColor fromRGBA(QString couleur);

    /** Reconstruit la CSS du FlatBoutonV1 */
    void updateCSS();

    QTimer *m_timer;
    void mouseReleaseEvent(QMouseEvent *e);

    /** Flag pour le mode debug */
    bool m_localDebug;

public slots:
    void slotDisableTimeout();
    void slotClicked();

signals:
    void signalSurvolIn();
    void signalSurvolOut();

};

#endif // ABULEDUFLATBOUTONV1_H
