/** Classe AbulEduDownloadProgressBar : popup de téléchargement
  *
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledudownloadprogressbar.h"
#include "ui_abuledudownloadprogressbar.h"

AbulEduDownloadProgressBar::AbulEduDownloadProgressBar(QNetworkReply *reply, QWidget *parent) :
    QWidget(parent),
    ui(new Ui::AbulEduDownloadProgressBar)
{
    m_localDebug = false;
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduDownloadProgressBar::AbulEduDownloadProgressBar parent ";
    }

    //Surtout pas sinon les logiciels "plantent": ils font un new dans le constructeur mais ensuite juste des show ... et nous dans le close
    //on se suicide ... donc le 2° show -> plantage
    //    setAttribute(Qt::WA_DeleteOnClose);
    ui->setupUi(this);

    ui->progressBar->setMaximum(0);
    m_reply = reply;

    //qu'on uploade ou downloade peu importe ... on affiche la progression :)
    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(slotLogicielDownloadInProgress(qint64,qint64)), Qt::UniqueConnection);
    m_time.start();

    //On efface le message par defaut pour éviter le sentiment étrange lors de l'ouverture de la popup
    abeUpdateStatus("");

    QDesktopWidget *desktop = QApplication::desktop();
    QRect screenRect = desktop->availableGeometry(this);
    int desktop_width = screenRect.width();
    int desktop_height = screenRect.height();
    this->move((desktop_width-this->width())/2, (desktop_height-this->height())/2);
}

AbulEduDownloadProgressBar::~AbulEduDownloadProgressBar()
{
    delete ui;
}

void AbulEduDownloadProgressBar::abeUpdateStatus(QString message)
{
    ui->lblTexte->setText(message);
}

void AbulEduDownloadProgressBar::slotLogicielDownloadInProgress(qint64 a, qint64 b)
{
    qDebug() << a << " " << b;
    QString message;
    int tempsrestant = 0;
    int total        = 0;
    //Si le maximum de la barre n'est pas déjà posé
    if(ui->progressBar->maximum() <= 0) {
        if(b > 0) {
            ui->progressBar->setMaximum(b);
            m_total = b;
        }
        //Gestion du cas particulier des download via XMLRPC
        else {
            if(m_reply) {
                total = m_reply->rawHeader("Content-length2").toInt();
                if(m_localDebug) {
                    qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
                    qDebug() << "Reponse serveur   : " << m_reply->rawHeader("Content-length").toInt();
                    qDebug() << "Reponse serveur 2 : " << m_reply->rawHeader("Content-length2").toInt();
                }
                if(total > 0) {
                    m_total = total;
                    ui->progressBar->setMaximum(total);
                    b = total;
                }
            }
        }
    }
    if(m_total > 0) {
        b = m_total;
    }

    QString unite      = trUtf8("o");
    QString uniteTemps = trUtf8("secondes");
    int actuel       = a/1024;

    int vitesse = (a/m_time.elapsed()) * 1000; // En o/sec
    if(a > 0 && b > 0 && vitesse > 0) {
        int reste = b - a;
        tempsrestant = reste / vitesse;
        if(tempsrestant > 60) {
            tempsrestant = reste / vitesse / 60;
            uniteTemps = trUtf8("minutes");
        }
        else {
            tempsrestant++ ;
        }
        if ( vitesse > 1024){
            vitesse /= 1024;
            unite = trUtf8("Ko");
        }

        if(m_total > 1024) {
            total = m_total / 1024;
        }

        if(m_localDebug) {
            qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
            qDebug() << "Valeur de a : " << a << " " << "Tps passé : "<< m_time.elapsed() <<" Tps restant : " << tempsrestant << " actuel : " << actuel << " total " << total << " vitesse " << vitesse;
        }
        message = trUtf8("Temps restant : %1 %2 -- %3 sur %4 %5 (%6 Ko/s)")
                .arg(QString::number(tempsrestant))
                .arg(uniteTemps)
                .arg(QString::number(actuel))
                .arg(QString::number(total))
                .arg(unite)
                .arg(QString::number(vitesse));
    }
    else {
        message = trUtf8("Téléchargement en cours : %1 Ko/s")
                .arg(QString::number(vitesse));
    }

    abeUpdateStatus(message);
    if(a  > 0) {
        ui->progressBar->setValue(a);
    }

}

void AbulEduDownloadProgressBar::slotLogicielUploadInProgress(qint64 a, qint64 b)
{
    QString message;
    int tempsrestant = 0;
    int total        = 0;
    //Si le maximum de la barre n'est pas déjà posé
    if(ui->progressBar->maximum() <= 0) {
        if(b > 0) {
            ui->progressBar->setMaximum(b);
        }
    }

    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduDownloadProgressBar::slotLogicielUploadInProgress " << a << " et " << b;
    }

    QString unite      = trUtf8("Ko");
    QString uniteTemps = trUtf8("secondes");
    int actuel       = a/1024;
    int vitesse      = a/m_time.elapsed(); //En Ko/sec
    if(b > 0 && vitesse > 0) {
        total = b/1024;
        int reste = total - actuel;
        tempsrestant = reste / vitesse;
        if(tempsrestant > 60) {
            tempsrestant = reste / vitesse / 60;
            uniteTemps = trUtf8("minutes");
        }
        if(m_localDebug) {
            qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
            qDebug() << "Temps passé : " << m_time.elapsed() << " actuel : " << actuel << " total " << total << " vitesse " << vitesse;
        }
        message = trUtf8("Temps restant : %1 %2 -- %3 sur %4 %5 (%6 Ko/s)")
                .arg(QString::number(tempsrestant))
                .arg(uniteTemps)
                .arg(QString::number(actuel))
                .arg(QString::number(total))
                .arg(unite)
                .arg(QString::number(vitesse));
    }
    else {
        message = trUtf8("Téléchargement en cours : %1 Ko/s")
                .arg(QString::number(vitesse));
    }

    abeUpdateStatus(message);
    if(a  > 0) {
        ui->progressBar->setValue(a);
    }
}

QByteArray AbulEduDownloadProgressBar::fileNameFromContentDisposition(const QByteArray &header)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    foreach(QByteArray item, header.split(';') )
    {
        int equalSignPos = item.indexOf('=');
        if (equalSignPos == -1)
            continue;
        QString key = item.left(equalSignPos).trimmed();
        if(key.compare("filename", Qt::CaseInsensitive) != 0)
            continue;

        QByteArray value = item.mid(equalSignPos+1).trimmed();
        if(value.startsWith('\"') && value.endsWith('\"'))
        {
            value = value.mid(1,value.length()-2);
        }
        return QByteArray::fromPercentEncoding(value);
    }
    return QByteArray();
}

bool AbulEduDownloadProgressBar::abeSetUploadBar(bool upload)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduDownloadProgressBar::abeSetUploadBar " << upload;
    }

    if(upload) {
//        disconnect(m_reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(slotLogicielDownloadInProgress(qint64,qint64)));
        connect(m_reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(slotLogicielUploadInProgress(qint64,qint64)),Qt::UniqueConnection);
    }
    else {
//        disconnect(m_reply,SIGNAL(uploadProgress(qint64,qint64)),this,SLOT(slotLogicielUploadInProgress(qint64,qint64)));
        connect(m_reply,SIGNAL(downloadProgress(qint64,qint64)),this,SLOT(slotLogicielDownloadInProgress(qint64,qint64)),Qt::UniqueConnection);
    }

    return true;
}

void AbulEduDownloadProgressBar::on_btnCancel_clicked()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //On essaye de stopper le transfert en cours
    m_reply->close();
}
