/** Classe AbulEduNetworkAccessManagerV1
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledunetworkaccessmanagerv1.h"
#include "abuleduapplicationv1.h"
#include "abuledussologinv1.h"

AbulEduNetworkAccessManagerV1::AbulEduNetworkAccessManagerV1(QObject *parent) :
    QNetworkAccessManager(parent)
{
    m_localDebug       = false;
    m_networkWhiteList.clear();
    setObjectName(QString("abeNetworkAccessManager"));
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1: creation, id " << this->objectName();
    }
    clear();

    m_ssoUsePopup           = true; //par défaut on ouvre une popup si on a besoin d'une authentification SSO
    //    m_ssoAuthenticator = QSharedPointer<QAuthenticator>(new QAuthenticator);
    m_ssoAuthenticator = new QAuthenticator();

    /** Ajout des certificats SSL racine de nos serveurs https (la version de Qt utilisée sous windows ne semble pas avoir ces certificats) */
    QStringList certificats;
    certificats << ":/abuledunetworkaccessmanagerv1/sslrapidssl-01" << ":/abuledunetworkaccessmanagerv1/sslgeotrust-01"
                << ":/abuledunetworkaccessmanagerv1/ssladdtrust-01" << ":/abuledunetworkaccessmanagerv1/sslpositive-01";
    QListIterator<QString> i(certificats);
    while (i.hasNext()) {
        QFile certif(i.next());
        if (certif.open(QIODevice::ReadOnly | QIODevice::Text)) {
            QByteArray c = certif.readAll();
            QSslCertificate cert(c);
            QList<QSslCertificate> certificates = QSslSocket().caCertificates();
            if(!certificates.contains(cert)) {
                if(m_localDebug) {
                    qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
                    qDebug() << " Ajout de " << certif.fileName();
                }
                QSslSocket().addDefaultCaCertificate(cert);
            }
            else {
                if(m_localDebug) {
                    qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
                    qDebug() << " Certificat deja present " << certif.fileName();
                }
            }
            certif.close();
        }
    }

    //Passage de ticket SSO entre alacarte et les clients
    //Si on est dans un logiciel alors on est client de abuledu-alacarte-srv
    //et on lance un "serveur" sous notre nom
    if(abeApp->applicationName() != "abuledu-alacarte") {
        m_localSocketServer = new AbulEduLocalSocketIpcServerV1(abeApp->applicationName() + "-" + abeApp->getAbeUserLogin(), this);
        m_localSocketClient = new AbulEduLocalSocketIpcClientV1("abuledu-alacarte-serveur-" + abeApp->getAbeUserLogin(), this);
        connect(m_localSocketServer, SIGNAL(abeSignalMessageReceived(QString)), this, SLOT(slotLocalSocketMessageReceived(QString)),Qt::UniqueConnection);

        //Etn on essaye de demander a alacarte s'il n'a pas les infos SSO
        if(m_login == "" || m_pass == "") {
            //On essaye de demander a alacarte s'il a pas les infos ?
            m_localSocketClient->abeSlotSendMessageToServer("HELLO:" + abeApp->applicationName() + ":ssoRequest:","abuledu-alacarte-serveur-" + abeApp->getAbeUserLogin());
        }

    }
    else {
        m_localSocketServer = new AbulEduLocalSocketIpcServerV1("abuledu-alacarte-serveur-" + abeApp->getAbeUserLogin(), this);
        m_localSocketClient = new AbulEduLocalSocketIpcClientV1("none", this);
        connect(m_localSocketServer, SIGNAL(abeSignalMessageReceived(QString)), this, SLOT(slotLocalSocketMessageReceived(QString)),Qt::UniqueConnection);
    }

    //connexion sur la gestion de l'authentification si c'est necessaire
    connect(this, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)), this, SLOT(slotProxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),Qt::UniqueConnection);
    //connexion sur la gestion des erreurs LDAP
    connect(this, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this, SLOT(slotSslErrors(QNetworkReply*,QList<QSslError>)),Qt::UniqueConnection);
    //connexion sur l'authentification SSO
    connect(this,SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)),this,SLOT(slotSSOAuth(QNetworkReply*,QAuthenticator*)), Qt::UniqueConnection);

    // =========================================================== Gestion PROXY
    abeActivateProxySettings();
    abeProxyConfigTest("start");
}

AbulEduNetworkAccessManagerV1::~AbulEduNetworkAccessManagerV1()
{

}

void AbulEduNetworkAccessManagerV1::setAbeLoginPass(QString login, QString pass)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1::setAbeLoginPass (1) : " << login << " " << pass;
    }
    m_login = login;
    m_pass  = pass;
    m_ssoAuthentifiedStatus = 0;
    m_passAsked = 0;
    m_ssoInProgress = false;
    if(!m_ssoAuthenticator->isNull()) {
        m_ssoAuthenticator->setUser(login);
        m_ssoAuthenticator->setPassword(pass);
    }
}

void AbulEduNetworkAccessManagerV1::abeSSOLogin()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1::abeSSOLogin (1) " << this->objectName();
        qDebug() << " sso in progress ? " << m_ssoInProgress;
    }
    if(m_ssoInProgress) {
        if(m_localDebug) {
            qDebug()  <<  "  " << __FUNCTION__ << " return a";
        }
        return;
    }

    //Si jamais login / pass sont vide on affiche la popup ?
    if(m_login == "" || m_pass == "") {
        //evite d'afficher 10 fois la popup si on change de bureau
        if(m_ssoPopupIsShow == false && m_ssoUsePopup == true) {
            m_ssoInProgress = true;
            AbulEduSSOLoginV1 *a = new AbulEduSSOLoginV1();
            a->showFullScreen();
            m_ssoPopupIsShow = true;
            connect(a, SIGNAL(destroyed()), this, SLOT(slotSSOPopupClosed()),Qt::UniqueConnection);
            connect(a, SIGNAL(signalClosed()), this, SLOT(slotSSOPopupClosed()),Qt::UniqueConnection);
        }
        if(m_localDebug) {
            qDebug()  <<  "  " << __FUNCTION__ << " return b";
        }
        return;
    }


    //--------------------------------------------------
    //20121105 : on profite de l'authentification pour récupérer l'identité de l'utilisateur vu que c'est possible de faire tout d'un coup !
    QNetworkRequest request;
    request.setRawHeader("User-Agent", QString(abeApp->applicationName() + "/" + abeApp->applicationVersion()).toAscii());
    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
    request.setUrl(QUrl("https://webadminxml.abuledu.net/login.php"));
    request.url().setUserName(m_login);
    request.url().setPassword(m_pass);

    QList<QVariant> params = QList<QVariant>();
    m_maiaObject = new MaiaObject(this);
    QNetworkReply* reply = this->post(request,
                                      m_maiaObject->prepareCall("abuledu.sso.login", params).toUtf8() );
    reply->setObjectName("ssoLogin");
    connect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotFinishedSSOLogin(QNetworkReply*)),Qt::UniqueConnection);

    //Grosse amélioration pour gérer les "erreurs de mots de passe", bien plus facile et efficace que de gérer un
    //pointeur sur un QAuthenticator qui se carapate et fait des erreurs de segmentation !
    //    QUrl urlLogin("https://webadminxml.abuledu.net/login.php");
    //    urlLogin.setPassword(m_pass);
    //    urlLogin.setUserName(m_login);
    //    request.setUrl(urlLogin);
    //    QNetworkReply *reply = this->get(request);
    //    reply->setObjectName("ssoLogin");
    //    connect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotFinishedSSOLogin(QNetworkReply*)),Qt::UniqueConnection);
    if(m_localDebug) {
        qDebug()  <<  "  " << __FUNCTION__ << " return c";
    }
}

void AbulEduNetworkAccessManagerV1::abeSSOLogout()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1::abeSSOLogout (1) " << this->objectName();
    }
    if(m_ssoAuthentifiedStatus != -1) {
        clear();
        abeApp->getAbeIdentite()->abeClear();
        m_ssoAuthentifiedStatus = -1;

        //Fermeture de la "session" websso
        QNetworkRequest request;
        request.setUrl(QUrl("https://webadminxml.abuledu.net/logout.php"));
        request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
        //    request.setAttribute(QNetworkRequest::AuthenticationReuseAttribute,QVariant( QNetworkRequest::Manual ));
        //    cookieJar()->deleteLater();
        //    QNetworkCookieJar *c = new QNetworkCookieJar(this);
        //    setCookieJar(c);

        connect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotFinishedSSOLogout(QNetworkReply*)),Qt::UniqueConnection);
        QNetworkReply *reply = this->get(request);
        reply->setObjectName("ssoLogout");
    }
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1::abeSSOLogout (2)";
    }
}

int AbulEduNetworkAccessManagerV1::abeSSOAuthenticationStatus()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_ssoAuthentifiedStatus;
}

/** @warning: Solution rapide de recherche d'un proxy, je n'analyse pas le javascript retourne et cherche uniquement une ligne PROXY ...*/
void AbulEduNetworkAccessManagerV1::finishedSlotWPAD(QNetworkReply* reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1 finishedSlotWPAD : " << reply->errorString();
    }
    if(reply->objectName() == "WPADRequest") {
        if (reply->error() == QNetworkReply::NoError)
        {
            // lectures des données issues du "reply"
            QByteArray contenu = reply->readAll();
            if(contenu.contains("PROXY")) {
                QByteArray temp = contenu.right(contenu.length() - contenu.indexOf("PROXY ") - 6); //6 = longueur du mot "PROXY "
                QString envproxy = temp.left(temp.indexOf(";"));
                if(m_localDebug) {
                    qDebug() << "AbulEduNetworkAccessManagerV1:finishedSlotWPAD Proxy trouvé: " << envproxy;
                }
                QRegExp exp(":");
                QStringList split = envproxy.split(exp);

                if(m_localDebug) {
                    qDebug() << "AbulEduNetworkAccessManagerV1:finishedSlotWPAD Proxy detected :: " << split;
                }
                QNetworkProxy proxy;
                proxy.setType(QNetworkProxy::HttpProxy);
                proxy.setHostName(split.at(0));
                proxy.setPort(split.at(1).toInt());
                QNetworkProxy::setApplicationProxy(proxy);
                this->setProxy(proxy);
            }
        }
        disconnect(this,SIGNAL(finished(QNetworkReply*)),this, SLOT(finishedSlotWPAD(QNetworkReply*)));
        reply->deleteLater();
    }
}

void AbulEduNetworkAccessManagerV1::abeSSOPopupEnable(bool enable)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_ssoUsePopup = enable;
}

void AbulEduNetworkAccessManagerV1::slotSSOAuth(QNetworkReply *networkReply, QAuthenticator *authenticator)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << this->objectName();
    }
    if(m_ssoAuthenticator->isNull()) {
        //        m_ssoAuthenticator = QSharedPointer<QAuthenticator>(authenticator);
        m_ssoAuthenticator = authenticator;
    }

    //Si on est en phase de logout, c'est pas la peine d'aller plus loin
    if(networkReply->objectName() == "ssoLogout") {
        authenticator->setUser("");
        authenticator->setPassword("");
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  (0a) " << m_login << " " << m_pass;
        }
        return;
    }
    if(m_login == "" && m_pass == "") {
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  login et password vides";
        }
        return;
    }

    //Si le login / pass est déjà passé c'est que c'est pas bon
    if(m_passAsked > 0 && authenticator->user() == m_login && authenticator->password() == m_pass) {
        m_ssoAuthentifiedStatus = -1;
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  login ou mot de passe incorrect";
        }
    }
    else {
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  (1) " << m_login << " " << m_pass;
            qDebug() << "  " << __FUNCTION__ << "  (2) " << authenticator->user() << " " << authenticator->password();
        }
        authenticator->setUser(m_login);
        authenticator->setPassword(m_pass);
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  (3) " << authenticator->user() << " " << authenticator->password();
        }
        //Detection loop
        //if(m_localDebug)
        m_ssoAuthentifiedStatus = 1;
    }
    m_passAsked++;
    if(m_localDebug) {
        qDebug() << "  " << __FUNCTION__ << "  (4)";
    }
}

void AbulEduNetworkAccessManagerV1::slotFinishedSSOLogin(QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1::slotFinishedSSOLogin (1) " << reply->objectName();
    }

    m_ssoAuthentifiedStatus = 0;
    if(reply->objectName() == "ssoLogin") {
        if(m_localDebug) {
            qDebug() << "  " << __FUNCTION__ << "  -> Code retour de reply : " << reply->error();
        }
        if(reply->error() == QNetworkReply::NoError) {
            //Tout va bien
            m_ssoAuthentifiedStatus = 1;
            //Si on a donné une destination automatique apres authentification réussie on y va
            if(m_destinationOnLoginSuccess != "") {
                QTimer::singleShot(10, m_destinationOnLoginSuccessObject, m_destinationOnLoginSuccess);
                m_destinationOnLoginSuccess = "";
            }
            QByteArray login(m_login.toUtf8());
            QByteArray pass(m_pass.toUtf8());

            //On passe l'identité a l'application via l'objet MaiaXML
            AbulEduIdentitesV1 *identite = abeApp->getAbeIdentite();
            QString response = QString::fromUtf8(reply->readAll());
            if(m_maiaObject) {
                connect(m_maiaObject, SIGNAL(aresponse(QVariant&,QNetworkReply*)), identite, SLOT(slotSetFromNetworkReply(QVariant&,QNetworkReply*)),Qt::UniqueConnection);
                //attention parseResponse supprime l'objet
                m_maiaObject->parseResponse(response, reply);
            }

            //Reponse en privé
            //HELLO:ssoAnswer:jgkeljgieajgiaejgera6432654
            //Si on est alacarte on propage sur tous les autres logiciels
            if(abeApp->applicationName() == "abuledu-alacarte") {
                m_localSocketClient->abeSlotSendMessageToServers("IPCSSOLOGIN:" + login.toBase64() + ":" + pass.toBase64());
            }
            //Sinon, on remonte à alacarte l'information
            else {
                m_localSocketClient->abeSlotSendMessageToServer("IPCSSOLOGIN:" + login.toBase64() + ":" + pass.toBase64(),"abuledu-alacarte-serveur-" + abeApp->getAbeUserLogin());
            }
        }
        else if(reply->error() == QNetworkReply::AuthenticationRequiredError) {
            //Si on a donné une destination automatique apres échec d'authentification on y va
            if(m_destinationOnLoginFailure != "") {
                QTimer::singleShot(10, m_destinationOnLoginFailureObject, m_destinationOnLoginFailure);
                m_destinationOnLoginFailure = "";
                //C'est la qu'on teste si on a fait plusieurs tentatives d'authentification
                if(m_passAsked >= 1) {
                    m_ssoAuthentifiedStatus = -1;
                }
                m_ssoInProgress = false;

                if(m_localDebug) {
                    qDebug() << "  " << __FUNCTION__ << "  authentification erreur -> pourquoi ssoAuth n'est pas lance mystere ?";
                }
            }
            else {
                if(m_localDebug) {
                    qDebug() << "  " << __FUNCTION__ << "  authentification erreur et m_passAsked : " << m_passAsked;
                }

                qDebug() << "  avec " << m_login << " et " << m_pass;
            }
        }
        //Autre erreur de code retour
        else {
            m_ssoAuthentifiedStatus = -2;
        }
        //        else if()
        reply->deleteLater();
        disconnect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotFinishedSSOLogin(QNetworkReply*)));
    }
    //    reply->deleteLater();
    emit ssoAuthStatus(m_ssoAuthentifiedStatus);

    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1::slotFinishedSSOLogin (2)";
    }
}

void AbulEduNetworkAccessManagerV1::slotFinishedSSOLogout(QNetworkReply *reply)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "AbulEduNetworkAccessManagerV1::slotFinishedSSOLogout (1)" << reply->objectName();
    }

    if(reply->objectName() == "ssoLogout") {
        if(m_localDebug) {
            qDebug() << __FUNCTION__ << "  Code retour de reply : " << reply->error();
        }

        if (reply->error() == QNetworkReply::NoError)
        {

            //Possible qu'il y ait un redirect pour épuiser la session websso par exemple
            QVariant possibleRedirectUrl = reply->attribute(QNetworkRequest::RedirectionTargetAttribute);
            if(m_localDebug) {
                qDebug() << "  redirect : " << possibleRedirectUrl.toString();
            }
            if(possibleRedirectUrl.toString() != "" && possibleRedirectUrl.toString() != reply->url().toString()) {
                if(m_localDebug) {
                    qDebug() << "  on suit le redirect ... ";
                }
                QNetworkReply *reply2 = get(QNetworkRequest(possibleRedirectUrl.toString()));
                reply2->setObjectName("ssoLogout");
                connect(this,SIGNAL(finished(QNetworkReply*)),this,SLOT(slotFinishedSSOLogout(QNetworkReply*)), Qt::UniqueConnection);
            }
            else {
                m_ssoAuthentifiedStatus = 0;
                disconnect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(slotFinishedSSOLogout(QNetworkReply*)));

                //Si on est alacarte on propage le sso logout
                if(abeApp->applicationName() == "abuledu-alacarte") {
                    m_localSocketClient->abeSlotSendMessageToServers("SSOLOGOUT:");
                }
                else {
                    m_localSocketClient->abeSlotSendMessageToServer("SSOLOGOUT:","abuledu-alacarte-serveur-" + abeApp->getAbeUserLogin());
                }
            }
            clear();
        }
        reply->deleteLater();
    }
    emit ssoAuthStatus(m_ssoAuthentifiedStatus);
}

void AbulEduNetworkAccessManagerV1::slotSSOPopupClosed()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__  << m_ssoPopupIsShow;
    }

    m_ssoPopupIsShow = false;
    m_ssoInProgress = false;

    //Si on ne s'est pas authentifié (erreur de mot de passe par exemple) et qu'on ferme la fenêtre
    //il faut remettre les variables a vide pour provoquer l'affichage de la popup lors du prochain
    //appel a SSOLogin
    if(m_ssoAuthentifiedStatus != 1) {
        m_login = "";
        m_pass  = "";
    }

    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__  << m_ssoPopupIsShow;
    }
}

void AbulEduNetworkAccessManagerV1::slotLocalSocketMessageReceived(QString message)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << message;
    }

    //Si on est alacarte
    if(abeApp->applicationName() == "abuledu-alacarte") {
        //Contenu du message
        //HELLO:leterrier-calculment:ssoRequest
        QStringList params = message.split(":");

        if(params.size() > 0) {
            //Alacarte recoit un HELLO -> on fait suivre le login/pass et on ajoute ce destinataire à notre liste de diff
            if(params.at(0) == "HELLO") {
                if(m_localDebug) {
                    qDebug() << "  " << __FUNCTION__ << "  " << params;
                }
                //On ajoute ce client dans notre liste de contacts
                if(params.size() > 1) {
                    if(params.at(1) != "none") {
                        m_localSocketClient->abeSlotAddRemoteServer(params.at(1) + "-" + abeApp->getAbeUserLogin());
                    }
                }
                QByteArray login;
                QByteArray pass;
                if(m_login != "") {
                    login = m_login.toUtf8();
                    pass = m_pass.toUtf8();
                }
                else {
                    login = "anonymous";
                    pass = "none";
                }
                //Reponse en privé
                //HELLO:ssoAnswer:jgkeljgieajgiaejgera6432654
                m_localSocketClient->abeSlotSendMessageToServer("HELLO:ssoAnswer:" + login.toBase64() + ":" + pass.toBase64(), params.at(1) + "-" + abeApp->getAbeUserLogin());
                emit ssoAuthPropagated(m_login);
            }
            //Alacarte recoit un QUIT -> on supprime ce destinataire de notre liste de diff
            else if(params.at(0) == "QUIT") {
                if(m_localDebug) {
                    qDebug() << "  " << __FUNCTION__ << "  " << params;
                }
                //On supprime ce client de notre liste
                if(params.size() > 1)
                    m_localSocketClient->abeSlotDelRemoteServer(params.at(1) + "-" + abeApp->getAbeUserLogin());
                QByteArray login(m_login.toUtf8());
                //Réponse en privé au revoir
                //QUIT:ssoAnswer:erics
                m_localSocketClient->abeSlotSendMessageToServer("QUIT:ssoAnswer:" + login.toBase64() + ":", params.at(1) + "-" + abeApp->getAbeUserLogin());
            }
            //Alacarte recoit un SSOLOGIN
            else if(params.at(0) == "IPCSSOLOGIN") {
                p_ipcSSOLogin(params);
                //Et on propage à toutes les applications
                /** @todo optimiser: ne pas re-envoyer a celui qui nous l'a envoyé */
                m_localSocketClient->abeSlotSendMessageToServers(message);
            }
            //Alacarte recoit un SSOLOGOUT
            else if(params.at(0) == "SSOLOGOUT") {

            }
        }
    }
    //Sinon c'est qu'on est une application autre que alacarte
    else {
        QStringList params = message.split(":");
        if(params.size() > 0) {
            //on reçoit un HELLO, normalement le login & pass suivent
            if(params.at(0) == "HELLO") {
                //Contenu du message
                QByteArray login = QByteArray::fromBase64(params.at(2).toAscii());
                QByteArray pass  = QByteArray::fromBase64(params.at(3).toAscii());
                if(m_localDebug) {
                    qDebug() << "  " << __FUNCTION__ << "  Apres le Base64Decode : " << login << " et " << pass;
                }

                //Si alacarte a retourné des choses intéressantes ...
                if(login != "" && login != "anonymous") {
                    setAbeLoginPass(login,pass);
                    abeSSOLogin();
                }
            }
            //Alerte nouvelle ouverture de session SSO
            else if(params.at(0) == "IPCSSOLOGIN") {
                p_ipcSSOLogin(params);
            }
            //Alerte nouvelle ouverture de session SSO
            else if(params.at(0) == "SSOLOGOUT") {
                abeSSOLogout();
            }
        }
    }
}

//Gestion de l'authentification proxy ...
void AbulEduNetworkAccessManagerV1::slotProxyAuthenticationRequired(QNetworkProxy proxy, QAuthenticator *authenticator)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << m_proxyPassAsked;
    }

    //On cherche voir si on a pas deja l'info sauvegardee
    QSettings settings("abuledu","AbulEduProxySettings");
    settings.beginGroup("proxy");

    //Si c'est la 1ere fois qu'on essaye ces parametres ... ok mais
    //sinon il faut la popup (dans ce cas c'est que les enregistrements sauvegardes ne sont plus bon)
    if((settings.value("server").toString() == this->proxy().hostName()) && (m_proxyPassAsked == 0)) {
        m_proxyLogin->clear();
        m_proxyLogin->append(settings.value("login").toString());
        m_proxyPass->clear();
        m_proxyPass->append(settings.value("pass").toString());
        QByteArray passc;
        passc.append(settings.value("pass").toString());
        authenticator->setUser(settings.value("login").toString());
        authenticator->setPassword(QByteArray::fromHex(QByteArray::fromBase64(passc)));
        QNetworkRequest redirection(m_networkReply->url());
        m_networkReply = this->get(redirection);
        //Detection loop
        m_proxyPassAsked++;
    }
    //et sinon direction popup
    else if(m_proxyPassAsked == 1) {
        AbulEduProxyPasswordDialog *dlg = new AbulEduProxyPasswordDialog(m_proxyLogin,m_proxyPass);
        dlg->setAbeLoginPassDialogNetworkReply(m_networkReply);
        dlg->setAbeLoginPassDialogAuthenticator(authenticator);
        dlg->show();
        m_proxyPassAsked++;
    }
}

void AbulEduNetworkAccessManagerV1::slotReplyError(QNetworkReply::NetworkError code)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__ << code << sender()->objectName();
    }

    //Si on est en train de tester le proxy
    if(sender()->objectName().startsWith("proxyTesting")) {
        qDebug()  << " proxy : " << __FILE__ <<  __LINE__ << __FUNCTION__ << code << sender()->objectName();
        QString complement;
        if(abeApp->applicationName() != "abuledu-alacarte") {
            complement = trUtf8("\nNote: Veuillez utiliser le logiciel abuledu-alacarte pour reconfigurer vos paramètres proxy.");
        }

        /// @todo: uniquement si pas de réseau
        //        QMessageBox *alertBox=new QMessageBox(QMessageBox::Warning,trUtf8("Problème !!"),trUtf8("Impossible de joindre votre serveur proxy.\nCode d'erreur %1.%2").arg(code).arg(complement));
        //        alertBox->show();
        QApplication::restoreOverrideCursor();
    }
}

void AbulEduNetworkAccessManagerV1::slotSslErrors(QNetworkReply *reply, const QList<QSslError> &errors)
{
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1 : slotSslErrors : " << errors;
        qDebug() << "AbulEduNetworkAccessManagerV1 : slotSslErrors : " << reply->errorString();
        qDebug() << "AbulEduNetworkAccessManagerV1 : slotSslErrors : " << reply->url();
        qDebug() << "AbulEduNetworkAccessManagerV1 : slotSslErrors : " << reply->sslConfiguration().peerCertificateChain();
    }

    /** @todo: attention, j'active cette option a cause de  QT 4.7 qui n'a pas nos certificats racine cheap (comodo) mais
        c'est clairement une mauvaise "solution" ! */
    //reply->ignoreSslErrors();
}

void AbulEduNetworkAccessManagerV1::slotReplyProxyNoError()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << " proxy: no error ?" << sender()->objectName();
    }
    if(sender()->objectName() == "proxyTesting-update") {
        QApplication::restoreOverrideCursor();
        QMessageBox *alertBox=new QMessageBox(QMessageBox::Warning,trUtf8("Configuration enregistrée"),trUtf8("La configuration du proxy est enregistrée, veuillez relancer le logiciel pour qu'elle prenne effet."));
        alertBox->show();
    }
    QApplication::restoreOverrideCursor();
}

void AbulEduNetworkAccessManagerV1::clear()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_ssoAuthentifiedStatus = 0;
    m_passAsked = 0;
    m_ssoInProgress = false;
    m_ssoPopupIsShow = false;
    m_login.clear();
    m_pass.clear();

    m_destinationOnLoginSuccess = "";
    m_destinationOnLoginSuccessObject = NULL;
    m_destinationOnLoginFailure = "";
    m_destinationOnLoginFailureObject = NULL;

}

void AbulEduNetworkAccessManagerV1::p_ipcSSOLogin(QStringList params)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(params.at(0) == "IPCSSOLOGIN") {
        //Contenu du message
        QByteArray login = QByteArray::fromBase64(params.at(1).toAscii());
        QByteArray pass  = QByteArray::fromBase64(params.at(2).toAscii());
        if(m_localDebug) {
            qDebug() << " ============================== Apres le Base64Decode : " << login << " et " << pass;
        }

        //Si le login n'est pas vide, qu'il n'est pas anonymous
        if(login != "" && login != "anonymous") {
            //Evite de tourner en récursif (auth -> ipc message auth -> auth -> ipc message auth etc.)
            if(login != m_login && pass != m_pass) {
                setAbeLoginPass(login,pass);
                abeSSOLogin();
            }
        }
    }
}

bool AbulEduNetworkAccessManagerV1::p_proxyConfigNoProxy()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_proxySettingsEnum = abeProxyDisabled;
    return true;
}

bool AbulEduNetworkAccessManagerV1::p_proxyConfigFromSystemEnv()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    //Beaucoup plus simple qu'avant !
    m_proxySettingsEnum = abeProxySystemEnv;
    QNetworkProxyFactory::setUseSystemConfiguration(true);
    return true;

//    QString envproxy = qgetenv("http_proxy");
//    if(envproxy.length() > 0)
//    {
//        if(m_localDebug) {
//            qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
//            qDebug() << "AbulEduNetworkAccessManagerV1 Proxy detected :: " << envproxy;
//        }
//        envproxy.remove("/");
//        //On découpe le proxy
//        QRegExp exp(":");
//        QStringList split = envproxy.split(exp);

//        if(m_localDebug) {
//            qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
//            qDebug() << "AbulEduNetworkAccessManagerV1 Proxy detected decoupe tableau :: " << split;
//        }
//        QNetworkProxy proxy;
//        proxy.setType(QNetworkProxy::HttpProxy);
//        proxy.setHostName(split[1]);
//        proxy.setPort(split[2].toInt());
//        QNetworkProxy::setApplicationProxy(proxy);
//        this->setProxy(proxy);
//        m_proxySettingsEnum = abeProxySystemEnv;
//        return true;
//    }
//    return false;
}

bool AbulEduNetworkAccessManagerV1::p_proxyConfigFromWPAD()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Détection automatique: WPAD
    QUrl adresseWPAD("");
    QHostInfo wpadSingleResolve = QHostInfo::fromName("wpad");
    if (wpadSingleResolve.error() != QHostInfo::NoError) {
        if(m_localDebug) {
            qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
            qDebug() << "AbulEduNetworkAccessManagerV1 Lookup failed:" << wpadSingleResolve.errorString();
        }
        //On essaye avec le domaine complet ...
        QHostInfo wpadDomainResolve = QHostInfo::fromName("wpad." + QHostInfo::localDomainName());
        if (wpadDomainResolve.error() != QHostInfo::NoError) {
            if(m_localDebug) {
                qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
                qDebug() << "AbulEduNetworkAccessManagerV1 Lookup failed:" << wpadDomainResolve.errorString();
            }
        }
        else {
            //On télécharge le fichier wpad pour l'analyser ...
            adresseWPAD = "http://wpad." + QHostInfo::localDomainName() + "/wpad.dat";
        }
    }
    else {
        //On télécharge le fichier wpad pour l'analyser ...
        adresseWPAD = "http://wpad/wpad.dat";
    }
    if(! adresseWPAD.isEmpty()){
        connect(this, SIGNAL(finished(QNetworkReply*)),this, SLOT(finishedSlotWPAD(QNetworkReply*)),Qt::UniqueConnection);
        QNetworkReply* reply = this->get(QNetworkRequest(adresseWPAD));
        reply->setObjectName("WPADRequest");

        //attente active du slot proxy ... attention risque de no limit ?
        QEventLoop loop;
        connect(reply, SIGNAL(finished()), &loop, SLOT(quit()),Qt::UniqueConnection);
        loop.exec();
        m_proxySettingsEnum = abeProxyAuto;
        return true;
    }
    return false;
}

/** @todo */
bool AbulEduNetworkAccessManagerV1::p_proxyConfigFromURI()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    qDebug() << " a faire !!";

//    p_proxyConfigTest();
    return false;
}

bool AbulEduNetworkAccessManagerV1::p_proxyConfigManual()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QNetworkProxy proxy;

    QSettings settings("abuledu","AbulEduProxySettings");
    settings.beginGroup("proxy");

    proxy.setType(QNetworkProxy::HttpProxy);
    proxy.setHostName(settings.value("HTTPserver","proxy").toString());
    proxy.setPort(settings.value("HTTPport","3128").toInt());
    proxy.setUser(settings.value("login","").toString());
    proxy.setPassword(settings.value("pass","").toString());

    QNetworkProxy::setApplicationProxy(proxy);
    this->setProxy(proxy);

    return true;
}

bool AbulEduNetworkAccessManagerV1::abeProxyConfigTest(QString contexte)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QApplication::setOverrideCursor(Qt::WaitCursor);

    QUrl url;
    url.setUrl("http://www.google.fr");

    if(m_localDebug) {
        qDebug() << " proxy: on teste : " << proxy().hostName() << ":" << proxy().port();
        qDebug() << " proxy: avec " << url;
    }

    QNetworkReply *reply = get(QNetworkRequest(url));
    reply->setObjectName("proxyTesting-" + contexte);

    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(slotReplyError(QNetworkReply::NetworkError)));
    connect(reply, SIGNAL(finished()), this, SLOT(slotReplyProxyNoError()));
}

QNetworkReply * AbulEduNetworkAccessManagerV1::post(const QNetworkRequest &request, QIODevice *data)
{
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1 : post1 " << objectName();
    }
    return QNetworkAccessManager::post(request, data);
}

QNetworkReply * AbulEduNetworkAccessManagerV1::post(const QNetworkRequest &request, const QByteArray &data)
{
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1 : post2 " << objectName();
        qDebug() << abeGetSSOLogin() << " et " << abeGetSSOPassword();
        qDebug() << "url : " << request.url();
    }
    return QNetworkAccessManager::post(request, data);
}

QNetworkReply *AbulEduNetworkAccessManagerV1::get(const QNetworkRequest &request)
{
    if(m_localDebug) {
        qDebug() << "AbulEduNetworkAccessManagerV1 : get1 " << objectName() << " login: " << abeGetSSOLogin() << " et " << abeGetSSOPassword();
        qDebug() << "url : " << request.url();
    }
    return QNetworkAccessManager::get(request);
}

void AbulEduNetworkAccessManagerV1::abeOnLoginSuccessGoto(QObject *objectDestination, const char *slotDestination)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_destinationOnLoginSuccess = slotDestination;
    m_destinationOnLoginSuccessObject = objectDestination;
}

void AbulEduNetworkAccessManagerV1::abeOnLoginFailureGoto(QObject *objectDestination, const char *slotDestination)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_destinationOnLoginFailure = slotDestination;
    m_destinationOnLoginFailureObject = objectDestination;
}

AbulEduNetworkAccessManagerV1::enumAbulEduNetworkAccessManagerProxySettings AbulEduNetworkAccessManagerV1::abeGetProxySettingsEnum() const
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return m_proxySettingsEnum;
}

void AbulEduNetworkAccessManagerV1::abeSetProxySettingsEnum(AbulEduNetworkAccessManagerV1::enumAbulEduNetworkAccessManagerProxySettings settings)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_proxySettingsEnum = settings;

    if(settings == AbulEduNetworkAccessManagerV1::abeProxyDisabled) {
        QNetworkProxy proxy;
        proxy.setType(QNetworkProxy::NoProxy);
        QNetworkProxy::setApplicationProxy(proxy);
        this->setProxy(proxy);
    }
}

void AbulEduNetworkAccessManagerV1::abeSaveProxySettings()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QSettings settings("abuledu","AbulEduProxySettings");
    settings.beginGroup("proxy");
    settings.setValue("configuration",m_proxySettingsEnum);
    if(m_proxySettingsEnum == abeProxyManual) {
        settings.setValue("HTTPserver",proxy().hostName());
        settings.setValue("HTTPport",proxy().port());
    }
    else {
        settings.setValue("HTTPserver","");
        settings.setValue("HTTPport","");
    }
    settings.sync();

    abeActivateProxySettings();
    //    qDebug() << "Save : " << proxy().hostName() << " et " << proxy().port();
}

void AbulEduNetworkAccessManagerV1::abeActivateProxySettings()
{
    //On passe en mode "pas de proxy"
    QNetworkProxy proxy;
    proxy.setType(QNetworkProxy::NoProxy);
    QNetworkProxy::setApplicationProxy(proxy);
    this->setProxy(proxy);

    // On regarde si on a une configuration sauvegardée
    QSettings settings("abuledu","AbulEduProxySettings");
    settings.beginGroup("proxy");
    int value = settings.value("configuration",-1).toInt();
    switch(value) {
    case -1:
        //Aucune configuration on lance les recherches (automatique)
        if( !p_proxyConfigFromSystemEnv()) {
            p_proxyConfigFromWPAD();
        }
        break;
    case 0:
        m_proxySettingsEnum = abeProxyDisabled;
        p_proxyConfigNoProxy();
        break;
    case 1:
        m_proxySettingsEnum = abeProxyAuto;
        p_proxyConfigFromWPAD();
        break;
    case 2:
        m_proxySettingsEnum = abeProxyManual;
        p_proxyConfigManual();
        break;
    case 3:
        m_proxySettingsEnum = abeProxyAutoURI;
        p_proxyConfigFromURI();
        break;
    case 4:
        m_proxySettingsEnum = abeProxySystemEnv;
        p_proxyConfigFromSystemEnv();
        break;
    }
}

void AbulEduNetworkAccessManagerV1::abeSetProxyManualSettings(QString serveurHTTP, QString portHTTP)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() << "Avant : " << proxy().hostName() << " et " << proxy().port();
        qDebug() << "On souhaite avoir : " << serveurHTTP << " et " << portHTTP;
    }
//    proxy().setHostName(serveurHTTP);
//    proxy().setPort(portHTTP.toInt());

    QNetworkProxy proxy;
    proxy.setType(QNetworkProxy::HttpProxy);
    proxy.setHostName(serveurHTTP);
    proxy.setPort(portHTTP.toInt());
    QNetworkProxy::setApplicationProxy(proxy);
    this->setProxy(proxy);

    //    qDebug() << "Apres : " << this->proxy().hostName() << " et " << this->proxy().port();
}

void AbulEduNetworkAccessManagerV1::abeSetNetworkWhiteList(QStringList domains)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_networkWhiteList.clear();
    if(domains.count() > 0) {
        m_networkWhiteList.append("abuledu.net");
        m_networkWhiteList.append("abuledu.org");
        m_networkWhiteList.append("ryxeo.com");
        m_networkWhiteList.append(domains);
    }
    if(m_localDebug) {
        qDebug()  << "  " << m_networkWhiteList;
    }
}

QNetworkReply *AbulEduNetworkAccessManagerV1::createRequest( Operation op,
                                                             const QNetworkRequest &req,
                                                             QIODevice *outgoingData )
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QNetworkRequest myReq( req );

    if ( m_networkWhiteList.count() > 0 && ! m_networkWhiteList.contains(req.url().host()) ) {
        if(m_localDebug) {
            qDebug()  << "  on filtre";
        }

        myReq.setUrl( QUrl( QString("forbidden://localhost/") ) );
    }
    else {
        if(m_localDebug) {
            qDebug()  << "  on laisse passer";
        }

    }
    QNetworkReply *reply = QNetworkAccessManager::createRequest( op, myReq, outgoingData );
    return reply;
}
