/** Classe AbulEduPageAccueilV1
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduPageAccueilV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUPAGEACCUEILV1_H
#define ABULEDUPAGEACCUEILV1_H

#include <QGraphicsView>
#include <QSettings>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QDir>
#include <QGridLayout>

#include "abuleduapplicationv1.h"
#include "abuleduaccueilzonev1.h"
#include "abuleduaccueilbullev1.h"
#include "abuledugraphicmenuv1.h"

class AbulEduPageAccueilV1 : public QGraphicsView
{
    Q_OBJECT
public:
    /** Constructeur : on lui passe le fichier de conf à partir duquel
      * la page d'accueil sera construite.
      * @param config pointeur vers le fichier de conf
      * @param listeTexteBulles pointeur vers la Qmap des numéro des zone et du texte correspondant (utile pour traduction)
      */
    explicit AbulEduPageAccueilV1(QMap<int, QString>*listeTexteBulles, QWidget *parent);

    /** Affiche les bulles actives ou les masque */
    void abePageAccueilMontreBulles(bool vraiFaux);

    /** Desactive les zones sensibles */
    void abePageAccueilDesactiveZones(bool vraiFaux);
    /** Retourne le pointeur du menu graphique */
    AbulEduGraphicMenuV1 * abePageAccueilGetMenu();

    /** Retourne la liste des zones cliquables de la page d'accueil */
    QList<AbulEduZoneV1 *> abePageAccueilGetZones();

    /** Pointeur vers le bouton qui permet de revenir à un éditeur masqué */
    QPushButton* abePageAccueilGetBtnRevenirEditeur();

    void setDimensionsWidgets();

private:
    int m_localDebug;
    QGraphicsScene *m_scene;
    QSettings *m_config;
    QMap<int, QString>*m_listeTexteBulles;
    QPixmap m_imageFond;
    QSize m_tailleIlmageFond;
    float m_proportionImageFond;
    float m_ratio;
    AbulEduZoneV1 *m_zone;
    QList<AbulEduZoneV1 *> m_listeZones;
    AbulEduGraphicMenuV1 *m_parchemin;
    void installPageAccueil();
    void resizeEvent(QResizeEvent *);
    void showEvent(QShowEvent *);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);

    QPushButton* m_btnRevenirEditeur;
signals:
    /** Signal émis par le bouton pressé
      * @param numero le numéro du bouton pressé
      */
    void boutonPressed(int numero);

    void dimensionsChangees();
public slots:

};

#endif // ABULEDUPAGEACCUEILV1_H
