/**
  * Classe AbulEduZoneV1
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduPageAccueilV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#ifndef ABULEDUZONEV1_H
#define ABULEDUZONEV1_H
#include <QObject>
#include <QGraphicsRectItem>
#include "abuleduaccueilbullev1.h"
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneMoveEvent>
#include <QFileInfo>

class AbulEduZoneV1 : public QObject, public  QGraphicsRectItem
{
    Q_OBJECT
public:
    explicit AbulEduZoneV1(int numero, QRectF zone, float ratio, QString cheminBulle, QString texteBulle, QFont font, QPoint posBulle, int offsetV, bool actif = 1, QGraphicsItem *parent = 0);
    ~AbulEduZoneV1();
    /** Modifie la taille de la zone sensible d'un facteur ratio
      * @param ratio le taux d'agrandissement
      */
    void abeZoneResizeRect(float ratio);

    /** Fixe la taille de la zone sensible
      * @param rect le rect de la zone
      */
    void abeZoneSetRect(QRectF rect);

    /** Récupère un pointeur vers la bulle
      * @return le pointeur de la bulle
      */
    AbulEduBulleV1 *abeZoneGetBulle();

    /** Récupère le rect de la zone sensible
      * @return le rect de la zone sensible
      */
    QRectF abeZoneGetRect();

    /** Retourne le numéro de la zone sensible
      * @return le numéro de la zone sensible
      */
    int abeZoneGetNumero();

    /** Détermine si la zone est sensible ou pas
      * @param YesNo true : la zone est sensible, false : la zone n'est pas sensible ni visible
      */
    void abeZoneSetActif(bool YesNo);

    /** teste si la zone est active */
    bool abeZoneIsActif();

    /** Dessine le contour des zones pour les délimiter 
      * @param bool YesNo, true : le contour des zones est dessiné, false : pas de contour dessiné
      *	@param QColor color, la couleur désirée pour le dessin du contour
      * @param int thickness, l'epaisseur du trait (2 par défaut)
      * @param Qt::PenStyle style, le style du trait (DashLine par défaut = pointillé)
      */
    void abeZoneDrawRect(bool YesNo, QColor color = Qt::black, int thickness = 2, Qt::PenStyle style = Qt::DashLine);
private:
    int m_numero;
    QRectF m_rect;
    QString m_cheminBulle;
    QString m_texteBulle;
    QPoint m_posBulle;
    QFont m_font;
    int m_offsetV;
    bool m_actif;
    float m_ratio;
    AbulEduBulleV1 *m_bulle;

public slots:
    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);

};

#endif // ABULEDUZONEV1_H
