/**
  * Classe AbulEduPageAccueilV1
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduPageAccueilV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

#include "abuledupageaccueilv1.h"
#include <QDebug>

AbulEduPageAccueilV1::AbulEduPageAccueilV1(QSettings *config, QMap<int, QString>*listeTexteBulles, QWidget *parent) :
    QGraphicsView(parent)
{
    m_listeTexteBulles = listeTexteBulles;
    m_config = config;
    m_config->setIniCodec("UTF-8");
    m_listeZones.clear();
    setCacheMode(QGraphicsView::CacheBackground);
    setFrameShape(QFrame::NoFrame);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    m_scene = new QGraphicsScene(parent);
    setScene(m_scene);
    setAlignment(Qt::AlignTop | Qt::AlignLeft);
    installPageAccueil();
}

void AbulEduPageAccueilV1::installPageAccueil()
{
    QFileInfo settingsPath(m_config->fileName());

    //    qDebug() << "Le fichier QSettings de config pour pagedaccueil est " << monpath.absoluteDir();
    // Installer le fond, le contour
    if(m_config->value("cheminFond").toString() !="")
    {
        m_imageFond.load(settingsPath.absolutePath() + "/images/" + m_config->value("cheminFond").toString());
//        qDebug()<<"Chemin Fond"<<settingsPath.absolutePath() + "/images/" + m_config->value("cheminFond").toString()<< "image size"<<m_imageFond.size();
        m_tailleIlmageFond = m_imageFond.size();
        m_proportionImageFond = (float)m_tailleIlmageFond.width()
                                / (float)m_tailleIlmageFond.height();
    }

    parentWidget()->setStyleSheet(m_config->value("couleurFond").toString());
    parentWidget()->setStyleSheet(parentWidget()->styleSheet() + QString("margin: 0; border-width: 0"));
    // Installer les zones sensibles et leur bulle
    int size= m_config->beginReadArray("zone");
    for(int i=0; i<size; i++)
    {
        m_config->setArrayIndex(i);
        QFont font = m_config->value("police").value<QFont>();
        font.setPixelSize(m_config->value("pixelSize").toInt());
        m_zone = new AbulEduZoneV1(i,
                                   m_config->value("rect").toRectF(),
                                   1,
                                   settingsPath.absolutePath() + "/images/" + m_config->value("cheminBulle").toString(),

                                   m_listeTexteBulles->value(i),
                                   font,
                                   m_config->value("posBulle").toPoint(),
                                   m_config->value("offsetV").toInt(),
                                   m_config->value("actif").toBool());

        m_listeZones.append(m_zone);
        m_scene->addItem(m_zone);
        m_scene->addItem(m_zone->abeZoneGetBulle());
        m_zone->setPos(0,0);
    }
    m_config->endArray();

    // créer le menu graphique
    m_parchemin = new AbulEduGraphicMenuV1(this);
    m_parchemin->setVisible(true);
    // Redimensionner le tout
    qDebug()<<"/////////////////////////////////////////////Installe page accueil";
    setDimensionsWidgets();
}

void AbulEduPageAccueilV1::resizeEvent(QResizeEvent *)
{

    if(isVisible()){
       qDebug()<<"                AbulEduPageAccueilV1::resizeEvent(QResizeEvent *)";
       setDimensionsWidgets();

    }
}

void AbulEduPageAccueilV1::showEvent(QShowEvent *)
{
//    setDimensionsWidgets();
}

void AbulEduPageAccueilV1::setDimensionsWidgets()
{
    float largeurScene = (float)parentWidget()->width();
    float hauteurScene = (float)parentWidget()->height();
    float proportionFondAccueil = largeurScene / hauteurScene;
    if(m_proportionImageFond > proportionFondAccueil)
    {
        // Redimensionnement horizontal
        m_ratio = largeurScene / (float)m_tailleIlmageFond.width();
        setFixedSize(QSize(largeurScene,
                           largeurScene / m_proportionImageFond));

    }
    else
    {
        // redimensionnement vertical
        m_ratio = hauteurScene / (float)m_tailleIlmageFond.height();
        setFixedSize(QSize(hauteurScene * m_proportionImageFond,
                           hauteurScene));
    }

        // Placement de la page d'accueil (centrée)
    move((largeurScene - this->width()) /2,
         (hauteurScene - this->height()) /2);
    setSceneRect(0,0,this->width(), this->height());
    setBackgroundBrush(m_imageFond.scaled(QSize(this->width(),this->height()),Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
        //redimensionnement des Zones sensibles et des bulles
    for(int i = 0; i < m_listeZones.size(); i++)
    {
        m_listeZones[i]->abeZoneResizeRect(m_ratio);
    }
    abeApp->setAbeApplicationDecorRatio(m_ratio);



    emit dimensionsChangees();

}

void AbulEduPageAccueilV1::mousePressEvent(QMouseEvent *event)
{
    QGraphicsView::mousePressEvent(event);
    if(event->button() == Qt::RightButton)
    {
        // On trouve les bulles
        abePageAccueilMontreBulles(true);
    }
}

void AbulEduPageAccueilV1::mouseReleaseEvent(QMouseEvent *event)
{

    if(event->button() == Qt::RightButton)
    {
        abePageAccueilMontreBulles(false);
    }
    else if(event->button() == Qt::LeftButton)
    {
        if((items(event->pos())).size()>0) // Il y a quelquechose sous le curseur
        {
            QList<QGraphicsItem *>listeItems = items(event->pos()); // On récupère la liste des items sous la souris
            for(int i = 0; i < listeItems.size(); i++)
            {
                if(listeItems[i]->type() == 3) // C'est un QGraphicsRectItem, donc une zone sensible (normalement ! )
                {
                    AbulEduZoneV1 *zone = static_cast<AbulEduZoneV1 *>(listeItems[i]);
                    if(zone->abeZoneIsActif())
                    {
                        emit boutonPressed(zone->abeZoneGetNumero());
                    }
                }
            }
        }
        else
        {
            event->ignore();
        }
    }
    QGraphicsView::mouseReleaseEvent(event);
}

void AbulEduPageAccueilV1::keyPressEvent(QKeyEvent *event)
{
    if(event->key()==Qt::Key_Space)
    {
        // On trouve les bulles
        if(!event->isAutoRepeat()){
            abePageAccueilMontreBulles(true);
        }
    }
    QGraphicsView::keyPressEvent(event);
}

void AbulEduPageAccueilV1::keyReleaseEvent(QKeyEvent *event)
{
    if(event->key()==Qt::Key_Space)
    {
        if(!event->isAutoRepeat()){
        abePageAccueilMontreBulles(false);
        }
    }
    QGraphicsView::keyReleaseEvent(event);
}
void AbulEduPageAccueilV1::abePageAccueilMontreBulles(bool vraiFaux)
{
    foreach(AbulEduZoneV1 *zone, m_listeZones)
    {
        if(zone->abeZoneIsActif())
        {
            zone->abeZoneGetBulle()->setVisible(vraiFaux);
        }
    }
}

void AbulEduPageAccueilV1::abePageAccueilDesactiveZones(bool vraiFaux)
{
    if(vraiFaux){
        foreach(AbulEduZoneV1 *zone, m_listeZones)
        {
            zone->abeZoneSetActif(false);
        }
    }
    else{
        // Réactiver les zones sensibles en fonction de la config
        int size= m_config->beginReadArray("zone");
        for(int i=0; i<size; i++)
        {
            m_config->setArrayIndex(i);
            m_listeZones[i]->abeZoneSetActif( m_config->value("actif").toBool());
        }
        m_config->endArray();
    }
}

AbulEduGraphicMenuV1 * AbulEduPageAccueilV1::abePageAccueilGetMenu()
{
    return m_parchemin;
}

QList<AbulEduZoneV1 *> AbulEduPageAccueilV1::abePageAccueilGetZones()
{
    return m_listeZones;
}


