/** Classe AbulEduSplashScreenV1
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @author http://developer.qt.nokia.com/wiki/QSplashScreen_replacement_for_semitransparent_images
  * @see The GNU Public License (GNU/GPL) v3
  * @see cc-by-sa for code from qt.nokia.com
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduSplashScreenV1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */

/**
 * @note Utilisation possible depuis le main.cpp (exemple1, splashscreen classique)
 * @code
#include "version.h"
#include "mainwindow.h"
#include "abuleduapplicationv1.h"
#include "abuledusplashscreenv1.h"

int main(int argc, char *argv[])
{
    AbulEduApplicationV1 a(argc, argv,VER_INTERNALNAME_STR, VER_PRODUCTVERSION_STR, VER_COMPANYDOMAIN_STR, "leterrier");
    a.setAbeApplicationLongName(QObject::trUtf8(VER_FILEDESCRIPTION_STR));
    MainWindow *w;

    AbulEduSplashScreenV1 *splash = new AbulEduSplashScreenV1(0, false);
    splash->show();
    for (int i = 1; i <= 5; i++)
    {
        splash->showMessage(QObject::trUtf8("Chargement en cours, merci de patienter ... %1%").arg(QString::number(i*20)), Qt::AlignBottom | Qt::AlignHCenter, Qt::black);
        //On lance le constructeur de la mainwindows en "arrière plan" lors du 1er passage
        //ca permet d'avoir les requetes reseau en arriere plan (mise a jour etc.) c'est "cool" :)
        if(i == 1) {
            w = new MainWindow(0);
            splash->setMainWindow(w);
        }
        qApp->processEvents();
        AbulEduSleeperThread::msleep(1000);
    }
    w->show();
    return a.exec();
}
@endcode

 * @note Utilisation possible depuis le main.cpp (exemple2: avec les boutons authentifié/anonyme)
 * @code
#include "version.h"
#include "mainwindow.h"
#include "abuleduapplicationv1.h"
#include "abuledusplashscreenv1.h"

int main(int argc, char *argv[])
{
    AbulEduApplicationV1 a(argc, argv,VER_INTERNALNAME_STR, VER_PRODUCTVERSION_STR, VER_COMPANYDOMAIN_STR, "leterrier");
    a.setAbeApplicationLongName(QObject::trUtf8(VER_FILEDESCRIPTION_STR));
    MainWindow *w;

    AbulEduSplashScreenV1 *splash = new AbulEduSplashScreenV1(0, true);
    splash->show();
    for (int i = 1; i <= 5; i++)
    {
        splash->showMessage(QObject::trUtf8("Chargement en cours, merci de patienter ... %1%").arg(QString::number(i*20)), Qt::AlignBottom | Qt::AlignHCenter, Qt::black);
        //On lance le constructeur de la mainwindows en "arrière plan" lors du 1er passage
        //ca permet d'avoir les requetes reseau en arriere plan (mise a jour etc.) c'est "cool" :)
        if(i == 1) {
            w = new MainWindow(0);
            splash->setMainWindow(w);
        }
        qApp->processEvents();
        AbulEduSleeperThread::msleep(1000);
    }
    w->hide();
    return a.exec();
}

//Et dans l'objet w

void MainWindow::slotSessionAuthenticated(bool enable)
{
  if (m_localDebug) qDebug() << "MainWindow::slotSessionAuthenticated" << enable;
  if(enable)
    abeApp->getAbeNetworkAccessManager()->abeSSOLogin();
  showMaximized();
}

@endcode
*/

#include "abuledusplashscreenv1.h"
#include <QLabel>

#if defined(Q_WS_X11)
extern void qt_x11_wait_for_window_manager(QWidget *m_mainWindow);
#endif

/**
  * Autre constructeur où on indique qu'on souhaite avoir les boutons anonyme/authentifié
  */
AbulEduSplashScreenV1::AbulEduSplashScreenV1(QWidget *parent, bool askAuth, Qt::WindowStates showMode)
    : QWidget(parent)
{
    m_localDebug = false;
    m_showMode = showMode;
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    m_askAuth = askAuth;
    setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);
    //plus |Qt::SplashScreen : un clic sur le splash provoque sa disparition !
    setAttribute(Qt::WA_DeleteOnClose);

    //On connecte le signal de l'application: si elle a un message a afficher il faut qu'il arrive au dessus/alaplace
    //du splashscreen
    connect(abeApp, SIGNAL(signalApplicationMessageStart()), this, SLOT(hide()));
    connect(abeApp, SIGNAL(signalApplicationMessageEnd()), this, SLOT(show()));

    //Si au bout de 10 secondes pas de réponses on passe a la fin du chargement ...
    QTimer::singleShot(10000,this,SLOT(slotFinChargement()));
    //Et si jamais alacarte nous a passé une identité on essaye de la récupérer
    connect(abeApp->getAbeIdentite(),SIGNAL(signalLoaded()),this,SLOT(slotFinChargement()));

    m_flagEnd    = false;
    m_labelPhoto = new QLabel(this);

    QFont fonteBoutons(abeApp->font().family(), 22,QFont::DemiBold);
    fonteBoutons.setLetterSpacing(QFont::AbsoluteSpacing,4);
    m_buttonFont = fonteBoutons;
    setAttribute(Qt::WA_TranslucentBackground, true);
    m_pixmap = QPixmap(":/abuledusplashscreenv1/splashscreen");
    setFixedSize(m_pixmap.size());

    //On centre le splashscreen
    QRect frect = frameGeometry();
    frect.moveCenter(QDesktopWidget().availableGeometry().center());
    move(frect.topLeft());

    //Evite d'avoir le fond noir quand on est sous linux sans compiz
    this->setMask(m_pixmap.mask());
}

AbulEduSplashScreenV1::~AbulEduSplashScreenV1()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
}

void AbulEduSplashScreenV1::clearMessage()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_message.clear();
    repaint();
}

void AbulEduSplashScreenV1::showMessage(const QString& message, int alignment/* = Qt::AlignLeft*/, const QColor& color/* = Qt::black*/)
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_message  = message;
    m_alignment = alignment;
    m_color  = color;
    repaint();
}

void AbulEduSplashScreenV1::showButtonShop()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    AbulEduFlatBoutonV1 *btnBuy = new AbulEduFlatBoutonV1(this);
    btnBuy->setObjectName("btnBuy");
    btnBuy->setCursor(Qt::PointingHandCursor);
    btnBuy->setCouleursTexte(QColor("#ecf2f4"),Qt::white,Qt::white,Qt::lightGray);
    btnBuy->setTextePaddingRight(5);
    btnBuy->setTexteAlignement(Qt::AlignRight);
    QFont fonteBouton;
    if(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus() == 1)
    {
        btnBuy->setText(trUtf8("Changer d'utilisateur"));
        btnBuy->setIconeNormale(QString(":/button/change"));
        btnBuy->setIconePressed(QString(":/button/change-over"));
        btnBuy->setIconeSurvol(QString(":/button/change-over"));
        bool tailleOK = false;
        do
        {
            m_buttonFont.setPointSize(m_buttonFont.pointSize()-1);
            fonteBouton = m_buttonFont;
            fonteBouton.setLetterSpacing(QFont::PercentageSpacing,115);
            QFontMetrics fm(fonteBouton);
            if (fm.boundingRect(btnBuy->text()).width() < 300) tailleOK = true;
            if (m_localDebug) {
                qDebug()  << "  " << __FILE__ << fm.boundingRect(btnBuy->text()).width();
            }
        }
        while(tailleOK == false);
        btnBuy->connect(btnBuy,SIGNAL(clicked()),this,SLOT(slotChangeLogin()));
    }
    else
    {
        btnBuy->setText(trUtf8("Multi-utilisateurs"));
        btnBuy->setIconeNormale(QString(":/button/buy"));
        btnBuy->setIconeSurvol(QString(":/button/buy-over"));
        fonteBouton = m_buttonFont;
        btnBuy->connect(btnBuy,SIGNAL(clicked()),this,SLOT(slotOpenShop()));
    }
    btnBuy->setFont(fonteBouton);
    btnBuy->setFixedSize(QPixmap(":/button/buy").size());
    btnBuy->move(438,350);
    btnBuy->show();
}

void AbulEduSplashScreenV1::paintEvent(QPaintEvent* pe)
{
    Q_UNUSED(pe);
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QRect aTextRect(rect());
    aTextRect.setRect(aTextRect.x(), aTextRect.y()+5, aTextRect.width()-10, aTextRect.height()-10);

    QPainter aPainter(this);
    aPainter.drawPixmap(rect(), m_pixmap);
    aPainter.setPen(m_color);
    aPainter.drawText(aTextRect, m_alignment, m_message);
}

/**
 * Gestion des slots des deux boutons
 */
void AbulEduSplashScreenV1::on_btnAuthenticated_clicked()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    emit signalSessionAuthenticated(true);
    m_mainWindow->setWindowState(m_showMode);
    //finish();
}

void AbulEduSplashScreenV1::on_btnAnonymous_clicked()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    emit signalSessionAuthenticated(false);
    m_mainWindow->setWindowState(m_showMode);
    finish();
}

void AbulEduSplashScreenV1::slotOpenShop()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QDesktopServices::openUrl(QUrl("http://www.ryxeo.com/boutique/fr/16-espace-de-stockage"));
    //Pour éviter que le splash reste au dessus du navigateur web
//    setWindowFlags(!Qt::WindowStaysOnTopHint);

    //    QTimer::singleShot(5000,m_mainWindow,SLOT(show()));
    m_mainWindow->close();
    finish();
}

void AbulEduSplashScreenV1::slotChangeLogin()
{
    abeApp->getAbeNetworkAccessManager()->abeSSOLogout();
    abeApp->getAbeNetworkAccessManager()->abeSSOLogin();
    m_mainWindow->showMaximized();
    finish();
}

void AbulEduSplashScreenV1::slotAfficheIdentite()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    QString id = abeApp->getAbeIdentite()->abeGetNom();
    if (m_localDebug) {
        qDebug()  << "  " << id;
    }
}

void AbulEduSplashScreenV1::slotGrandAvatar()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus() == 1)
    {
        QPixmap photoRet;
        photoRet = abeApp->getAbeIdentite()->abeGetPhoto()->scaledToWidth(60,Qt::SmoothTransformation);
        m_labelPhoto->setStyleSheet("QLabel { background-color : transparent; }");
        m_labelPhoto->setPixmap(photoRet);
        m_labelPhoto->resize(60,60);
        m_labelPhoto->move(467,230);
    }
}

void AbulEduSplashScreenV1::slotPetitAvatar()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus() == 1)
    {
        QPixmap photoRetaillee(abeApp->getAbeIdentite()->abeGetPhoto()->scaledToWidth(54,Qt::SmoothTransformation));
        m_labelPhoto->setPixmap(photoRetaillee);
        m_labelPhoto->resize(54,54);
        m_labelPhoto->move(471,234);
    }
}

void AbulEduSplashScreenV1::slotFinChargement()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    //Evite le double appel entre propagation d'id d'alacarte
    //et "timeout"
    if(m_flagEnd == false) {
        m_flagEnd = true;
        //2013.01.16: en mode tablette on simplifie
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
        showButtonShop();
#endif
        finish();
    }
}

void AbulEduSplashScreenV1::finish()
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if(m_askAuth) {
        if (m_localDebug) {
            qDebug()  << "  " << "on ajoute les boutons d'authentification";
        }
        AbulEduFlatBoutonV1 *btnAuthenticated = new AbulEduFlatBoutonV1(this);
        btnAuthenticated->stackUnder(m_labelPhoto);
        connect(btnAuthenticated, SIGNAL(signalSurvolIn()),this,SLOT(slotGrandAvatar()));
        connect(btnAuthenticated, SIGNAL(signalSurvolOut()),this,SLOT(slotPetitAvatar()));
        btnAuthenticated->setObjectName("btnAuthenticated");
        btnAuthenticated->setCursor(Qt::PointingHandCursor);
        btnAuthenticated->setCouleursTexte(QColor("#ecf2f4"),Qt::white,Qt::white,Qt::lightGray);
        btnAuthenticated->setTextePaddingRight(5);
        btnAuthenticated->setTexteAlignement(Qt::AlignRight);
        if(abeApp->getAbeNetworkAccessManager()->abeSSOAuthenticationStatus() == 1)
        {
            QString intitule = abeApp->getAbeIdentite()->abeGetPreNom();
            if (!intitule.isEmpty() && !abeApp->getAbeIdentite()->abeGetNom().isEmpty())
            {
                intitule.append(" "+abeApp->getAbeIdentite()->abeGetNom());
            }
            // Est-il possible qu'une connection soit établie avec un login (et donc un mdp !!) valide mais que les champs nom et prénom n'aient pas été renseignés ?
            if (intitule.isEmpty())
            {
                intitule = abeApp->getAbeNetworkAccessManager()->abeGetSSOLogin().split("@").at(0);
            }
            btnAuthenticated->setText(intitule);
            btnAuthenticated->setIconeNormale(QString(":/button/id"));
            btnAuthenticated->setIconePressed(QString(":/button/id-over"));
            btnAuthenticated->setIconeSurvol(QString(":/button/id-over"));

            QPixmap photoRetaillee = abeApp->getAbeIdentite()->abeGetPhoto()->scaledToWidth(56,Qt::SmoothTransformation);

            m_labelPhoto->setPixmap(photoRetaillee);
            m_labelPhoto->resize(56,56);
            m_labelPhoto->move(438,233);
            m_labelPhoto->show();
            slotPetitAvatar();
        }
        else
        {
//2013.01.16 : decision pour la tablette pour eviter le bug qui fait que la sso popup se cache dessous
#ifdef __ABULEDUTABLETTEV1__MODE__
#else
            btnAuthenticated->setText(trUtf8("Authentification"));
            btnAuthenticated->setIconeNormale(QString(":/button/authenticate"));
            btnAuthenticated->setIconePressed(QString(":/button/authenticate-over"));
            btnAuthenticated->setIconeSurvol(QString(":/button/authenticate-over"));
#endif
        }

        bool tailleOK = false;
        do
        {
            m_buttonFont.setPointSize(m_buttonFont.pointSize()-1);
            QFontMetrics fm(m_buttonFont);
            if (fm.boundingRect(btnAuthenticated->text()).width() < 300) tailleOK = true;
        }
        while(tailleOK == false);
        btnAuthenticated->setFont(m_buttonFont);
        btnAuthenticated->setFixedSize(QPixmap(":/button/authenticate").size());
        btnAuthenticated->move(width()-21-btnAuthenticated->width(),230);
        btnAuthenticated->show();

        AbulEduFlatBoutonV1 *btnAnonymous = new AbulEduFlatBoutonV1(this);
        btnAnonymous->setObjectName("btnAnonymous");
        btnAnonymous->setCursor(Qt::PointingHandCursor);
        btnAnonymous->setFont(m_buttonFont);
        btnAnonymous->setCouleursTexte(QColor("#ecf2f4"),Qt::white,Qt::white,Qt::lightGray);
        btnAnonymous->setTextePaddingRight(5);
        btnAnonymous->setTexteAlignement(Qt::AlignRight);
        btnAnonymous->setText(trUtf8("Anonyme"));
        btnAnonymous->setIconeNormale(QString(":/button/anonymous"));
        btnAnonymous->setIconePressed(QString(":/button/anonymous-over"));
        btnAnonymous->setIconeSurvol(QString(":/button/anonymous-over"));
        btnAnonymous->setFixedSize(QPixmap(":/button/anonymous").size());
        btnAnonymous->move(438,290);
        btnAnonymous->show();

        QMetaObject::connectSlotsByName(this);

        //Et pour le prochain passage
        m_askAuth = false;
        repaint();
#if defined(Q_WS_X11)
        qt_x11_wait_for_window_manager(m_mainWindow);
#endif
    }
    else {
        if (m_mainWindow) {
#if defined(Q_WS_X11)
            if (m_localDebug) {
                qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
            }
            qt_x11_wait_for_window_manager(m_mainWindow);
#endif
            m_mainWindow->show();
            m_mainWindow->setWindowState(m_showMode);
        }
        close();
    }
}

/** @todo: remettre en service le msleep / multithread  /autre */
void AbulEduSplashScreenV1::launch(int msec)
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }

    QTimer::singleShot(msec,this,SLOT(slotFinChargement()));

    for (int i = 1; i <= 6; i++)
    {
        if(i < 6) {
            showMessage(QObject::trUtf8("Chargement en cours, merci de patienter ... %1%").arg(QString::number(i*20)), Qt::AlignBottom | Qt::AlignHCenter, Qt::black);
        }
        //tres important si vous voulez que le splash s'affiche !
        //20121107: d'apres moi (ES) ca n'est plus necessaire -> mais d'apres la doc ça l'est alors je le laisse
        qApp->processEvents();
        //si on le laisse, ca bloque le reste de l'appli et donc la requete d'authentification
        //        AbulEduSleeperThread::msleep(msec);
    }
    showMessage(QObject::trUtf8(""), Qt::AlignBottom | Qt::AlignHCenter, Qt::black);
}

void AbulEduSplashScreenV1::setMainWindow(QWidget *mainWindow)
{
    if (m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_mainWindow = mainWindow;
    connect(this,SIGNAL(signalSessionAuthenticated(bool)),mainWindow,SLOT(slotSessionAuthenticated(bool)));
    connect(this,SIGNAL(signalSessionAuthenticated(bool)),this,SLOT(finish()));
}
