/**
  * Outils(le terrier d'AbulEdu)
  *
  * @see https://redmine.ryxeo.com/projects/ryxeo/wiki/AbulEduTeteV1
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @author 2011-2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GPL)

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuledugraphicsteteswidgetv1.h"
#include <QDebug>
AbulEduGraphicsTetesWidgetV1::AbulEduGraphicsTetesWidgetV1(int nbTete, int taille,QGraphicsWidget *parent ) :
    QGraphicsWidget(parent)
{
    m_localDebug = false;

    Q_UNUSED(parent)
    m_taille =QSize(taille, taille);
    m_listeTetes.clear();
    suffixe = new QMap<int, QString>();
    suffixe->insert(abe::evalZ, "rien");
    suffixe->insert(abe::evalY, "attente");
    suffixe->insert(abe::evalA, "tresbien");
    suffixe->insert(abe::evalB, "bien");
    suffixe->insert(abe::evalC, "moyen");
    suffixe->insert(abe::evalD, "mal");

    marge = 10;
    m_contour=QColor(Qt::transparent); // On peut passer un QPen plus complexe
    m_background = QColor(Qt::transparent); //Pareil voir la doc sur les QPen et QBrush
    //qt 4.7 ou +
#if QT_VERSION >= 0x040700
    setAutoFillBackground(false);
#endif
    resetTetes(nbTete, parent);

}

AbulEduGraphicsTetesWidgetV1::~AbulEduGraphicsTetesWidgetV1()
{
    delete suffixe;
}

void AbulEduGraphicsTetesWidgetV1::resetTetes(int nbTete, QGraphicsWidget *parent)
{
    Q_UNUSED(parent)
    // On efface toutes les têtes
    foreach(QGraphicsPixmapItem *tmpItem, listePixmapTetes)
    {
        delete tmpItem;
    }

    listePixmapTetes.clear();
    m_listeTetes.clear();

    //20120302: modification structurelle (eric) pour converger vers la meme automatisation des
    //themes que la abuleduexercicev1 ... le fichier qrc a pour prefixe abuledutetev1/

    int x=marge;
    int y = marge;
    for(int i=0; i<nbTete; i++)
    {
        QGraphicsPixmapItem *pixTete = new QGraphicsPixmapItem();
        pixTete->setParentItem(this);
        pixTete->setPixmap(QPixmap(QString(":/evaluation/attente")).scaled(m_taille,Qt::KeepAspectRatio, Qt::SmoothTransformation));
        if (m_localDebug) qDebug() << "AbulEduGraphicsTetesWidgetV1 :: " << QString(":/evaluation/attente");
        pixTete->setPos(x, y);
        x=x+m_taille.width()+marge;

        listePixmapTetes.append(pixTete);
        m_listeTetes.append(abe::evalY); // Tete attente

        setGeometry(pos().x(), pos().y(),(m_taille.width() + 10)*nbTete + marge, m_taille.height() + marge*2 );
    }

}

void AbulEduGraphicsTetesWidgetV1::setNbTetes(int nbTete)
{
    resetTetes(nbTete,parentWidget());
}

void AbulEduGraphicsTetesWidgetV1::setEtatTete(int numTete, abe::ABE_EVAL tete)
{
    if (m_localDebug) {
        qDebug() << "AbulEduGraphicsTetesWidgetV1::setEtatTete " << numTete << " code " << tete;
    }
    if(numTete < listePixmapTetes.size())
    {
        QGraphicsPixmapItem *pixTete = listePixmapTetes[numTete];
        QFileInfo fi(":/evaluation/"+suffixe->value(tete));
        if(fi.exists()) {
            pixTete->setPixmap(QPixmap(fi.absoluteFilePath()).scaled(m_taille, Qt::KeepAspectRatio, Qt::SmoothTransformation));
        }
        else {
            qWarning() << "AbulEduGraphicsTetesWidgetV1::setEtatTete fichier manquant : " << fi.absoluteFilePath();
        }
        m_listeTetes[numTete]=tete;
    }
    if (m_localDebug) {
        qDebug() << tete << " -> " << m_listeTetes << ", liste de taille " << m_listeTetes.count();
    }
}

void AbulEduGraphicsTetesWidgetV1::resizeTetes(int taille)
{
    m_taille = QSize(taille, taille);
    int x = marge;
    int y = marge;
    for(int i = 0; i < listePixmapTetes.size(); i++)
    {
        listePixmapTetes[i]->setPos(x,y);
        x=x+m_taille.width()+marge;

        setEtatTete(i, m_listeTetes[i]); // Redessine la tête dans son état antérieur et à la nouvelle taille
    }
    setGeometry(pos().x(), pos().y(),(m_taille.width() + 10)*listePixmapTetes.size() + marge, m_taille.height() + marge*2 );
}

int AbulEduGraphicsTetesWidgetV1::getSizeTetes()
{
    return m_taille.width();
}

abe::ABE_EVAL AbulEduGraphicsTetesWidgetV1::getEtatTete(int numTete)
{
    if(numTete < m_listeTetes.count())
        return m_listeTetes[numTete];
    else
        return abe::evalZ;
}

QList<abe::ABE_EVAL> AbulEduGraphicsTetesWidgetV1::getEtatTetes()
{
    return m_listeTetes;
}

void AbulEduGraphicsTetesWidgetV1::setCouleurs(QPen contour, QBrush brosse)
{
    m_contour = contour;
    m_background = brosse;
}

void AbulEduGraphicsTetesWidgetV1::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option)
    Q_UNUSED(widget)
    painter->setPen(m_contour);
    painter->setBrush(m_background);
    painter->drawRoundedRect(boundingRect(), marge, marge);
}

QRectF AbulEduGraphicsTetesWidgetV1::boundingRect()
{
    return QRectF(0,0,geometry().width(), geometry().height());
}
