/**
  * Classe
  * @see https://redmine.ryxeo.com/projects/
  * @author 2012 Eric Seigne <eric.seigne@ryxeo.com>
  * @see The GNU Public License (GNU/GPL) v3
  *
  *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  *
  * You should have received a copy of the GNU General Public License along
  * with this program. If not, see <http://www.gnu.org/licenses/>.
  */

#include <QApplication>
#include "abuleduvirtualkeyboardv1.h"

AbulEduVirtualKeyboardV1::AbulEduVirtualKeyboardV1()
{
    m_localDebug = false;
    m_isEnabled = false;
    m_abuleduVirtualKeyboardGuiV1 = new AbulEduVirtualKeyboardGuiV1();
}

AbulEduVirtualKeyboardV1::~AbulEduVirtualKeyboardV1()
{
}

void AbulEduVirtualKeyboardV1::abeSetEnabled(bool enabled)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    m_isEnabled = enabled;
    if(m_isEnabled) {
        connect(qApp, SIGNAL(focusChanged(QWidget*,QWidget*)),
                this, SLOT(saveFocusWidget(QWidget*,QWidget*)),Qt::UniqueConnection);
    }
    else {
        disconnect(qApp, SIGNAL(focusChanged(QWidget*,QWidget*)), this, SLOT(saveFocusWidget(QWidget*,QWidget*)));
    }
}

QString AbulEduVirtualKeyboardV1::identifierName()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return "AbulEduVirtualKeyboardV1";
}

QString AbulEduVirtualKeyboardV1::language()
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return "fr_FR";
}

bool AbulEduVirtualKeyboardV1::isComposing() const
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    return false;
}

void AbulEduVirtualKeyboardV1::reset()
{
}

/** si event sur clavier ok. Si event pas sur clavier, on close le clavier
  */
bool AbulEduVirtualKeyboardV1::filterEvent(const QEvent* event)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
    }
    if (event->type() == QEvent::RequestSoftwareInputPanel) {
        if(m_abuleduVirtualKeyboardGuiV1->abeGetInputWidget() != 0) {
            if(m_localDebug) {
                qDebug()  << __FILE__ <<  " line : " << __LINE__ << __FUNCTION__;
            }
            m_abuleduVirtualKeyboardGuiV1->show();
            return true;
        }
    }
    return false;
}

void AbulEduVirtualKeyboardV1::saveFocusWidget(QWidget *a, QWidget *b)
{
    if(m_localDebug) {
        qDebug()  << __FILE__ <<  __LINE__ << __FUNCTION__;
        qDebug() <<"a :" << a;
        qDebug() <<"b :" << b;
        qDebug() << "celui qui a le input focus actuellement " << m_abuleduVirtualKeyboardGuiV1->abeGetInputWidget();
    }
    //Si le clavier virtuel est actif on laisse passer -> il ne faut pas capter les changements de focus
    //du clavier virtuel !
    if(!m_abuleduVirtualKeyboardGuiV1->isActiveWindow()) {
        if(a && b && a != b && ! b->property("isAbulEduVirtualKeyboardDisabled").toBool()) {
            if(b->inherits("QLineEdit") || b->inherits("QTextEdit") || b->inherits("QPlainTextEdit")
                    || b->inherits("AbulEduTexteV1") || b->inherits("QWebView")) {
                if(m_localDebug) {
                    qDebug() << "Focus de " << a->objectName() << " vers " << b->objectName();
                }
                m_abuleduVirtualKeyboardGuiV1->abeSetInputWidget(b);
            }
        }
        //Un cas particulier pour le 1er widget qui choppe le focus
        else if(a) {
            if(a->inherits("QLineEdit") || a->inherits("QTextEdit") || a->inherits("QPlainTextEdit")
                    || a->inherits("AbulEduTexteV1") || a->inherits("QWebView")) {
                m_abuleduVirtualKeyboardGuiV1->abeSetInputWidget(a);
                a->setFocus();
            }
        }
        //Autre cas particulier, le webview avec plusieurs champs de saisie ...
        else if(a && a->inherits("QWebView")) {
            m_abuleduVirtualKeyboardGuiV1->abeSetInputWidget(a);
            a->setFocus();
        }
    }
}
